/**
 * jQuery Erweiterung: Leichte Kiste mit tollen Sachen und Überraschungen
 *
 *
 * @author Patrick Taddey <p.taddey@affiliprint.de>
 * @version 1.0
 * @copyright Copyright (c) 2013 AffiliPRINT GmbH (http://www.affiliprint.de/)
 */
(function($)
{
	/**
	 * Plugin registrieren
	 */
	$.fn.extend({
		/**
		 * Plugin-Definition
		 *
		 * @param object options
		 * @return void
		 */
		LiteBox : function(options)
		{
			/*	Standard-Werte setzen */
			var defaults = {};

			/*	Standard-Werte erweitern */
			var options = $.extend(defaults, options);

			/* Close-Handler, um leichte Kiste wieder zu schließen  */
			$(this)._closeHandler();

			/* Elemente iterieren */
			return this.each(function(event)
			{
				/* Objekt-Variablen erweitern */
				var elementOptions = options;

				/* Click-Event an das Element binden */
				$(this).click(function(event)
				{
					$(this)._liteBoxHandler(event, elementOptions);
				});
			});
		},
		/**
		 * Leichte Kiste wieder schließen
		 *
		 * @return void
		 */
		_closeHandler : function()
		{
			/* Click-Event anb Element binden */
			$('#apLightBoxClose').live('click', function(event)
			{
				/* Standard-Click-Event unterbinden */
				event.preventDefault();
				hideLiteBox();
			});
			return;
		},
		/**
		 * Action generieren und LiteBox aufrufen
		 *
		 * @param object event
		 * @param object elementOptions
		 * @return void
		 */
		_liteBoxHandler : function(event, elementOptions)
		{
			/* Standard-Click-Event unterbinden */
			event.preventDefault();

			/* Methode bzw. Action generieren und ausführen */
			var elementAction = '$(this)._' + $(this).attr('liteboxtype') + 'Action(elementOptions)';

			/* versuche die Methode auszuführen */
			try {
				eval(elementAction);
			} catch(e) {
				consoleLog(elementAction + ' existiert nicht');
			}

			return;
		},
		/**
		 * Kiste für Bilder
		 *
		 * @param object elementOptions
		 * @return void
		 */
		_imageLiteBoxAction : function(elementOptions)
		{
			var imageHref = $(this).attr('href');

			/* Leichte Kiste anzeigen */
			$(this)._showLiteBox();
			$('#apLightBoxContainer').append('<img src="' + imageHref + '" border="0" /><div id="apLightBoxClose"></div>');
			$(this)._valigner('#apLightBoxContainer');
			return;
		},
		/**
		 * Kiste für Iframes
		 *
		 * @param object elementOptions
		 * @return void
		 */
		_iframeLiteBoxAction : function(elementOptions)
		{
			/* Id des Inhalts */
			var contentId = $(this).attr('href');
			
			/* Source für die Iframe beziehen */
			var iframeSource = $(this).attr('iframeSource');

			/* Leichte Kiste anzeigen */
			$(this)._showLiteBox();
			$('#apLightBoxContainer').append('<div class="apLightBoxIframe"><iframe border="0" id="apIframe" width="100%" src="' + iframeSource + '"></iframe><div id="apLightBoxClose"></div></div>');
			
			$(this)._valigner('.apLightBoxIframe');
			return;
		},
		/**
		 * Kiste für Inhalte
		 *
		 * @param object elementOptions
		 * @return void
		 */
		_contentLiteBoxAction : function(elementOptions)
		{
			/* Id des Inhalts */
			var contentId = $(this).attr('href');

			/* Leichte Kiste anzeigen */
			$(this)._showLiteBox();
			$('#apLightBoxContainer').append('<div class="apLightBoxContent">' + $(contentId).html() + '<div id="apLightBoxClose"></div></div>');
			$(this)._valigner('.apLightBoxContent');
			return;
		},
		/**
		 * Kiste für Lade-Animationen
		 *
		 * @param object elementOptions
		 * @return void
		 */
		_loadingLiteBoxAction : function(elementOptions)
		{
			/* Leichte Kiste anzeigen */
			$(this)._showLiteBox();
			$('#apLightBoxContainer').append('<div class="apLightBoxContent loading"><div id="apLoading">' + MESSAGE_LOADING + '</div></div>');
			$(this)._valigner('.apLightBoxContent');
			return;
		},
		/**
		 * Leichte Kiste anzeigen
		 *
		 * @return void
		 */
		_showLiteBox : function()
		{
			if($('#apLightBox').length > 0) {
				$('#apLightBoxContainer').html('&nbsp;');
				$('#apLightBox').show();
			} else {
				$('body').append('<div id="apLightBox"><div id="apLightBoxContainer"></div></div>');
			}

			/* Höhe des Hintergrundes bestimmen */
			$('#apLightBox').css('height', $(document).height());

			return;
		},
		/**
		 * Leichte Kiste anzeigen
		 *
		 * @var String element
		 * @return void
		 */
		_valigner : function(element)
		{
			/* Element zentrieren */
			valign(element);
			/* Element zentrieren, wenn Fenstergröße geändert wird */
			$(window).resize(function()
			{
				valign(element);
			});

			return;
		}
	});
})(jQuery);
