<?php
/* --------------------------------------------------------------
   GMTSService.php 2013-11-14 gambio
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2013 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------


   based on:
   (c) 2000-2001 The Exchange Project  (earlier name of osCommerce)
   (c) 2002-2003 osCommerce(ot_cod_fee.php,v 1.02 2003/02/24); www.oscommerce.com
   (C) 2001 - 2003 TheMedia, Dipl.-Ing Thomas Plnkers ; http://www.themedia.at & http://www.oscommerce.at
   (c) 2003 XT-Commerce - community made shopping http://www.xt-commerce.com ($Id: ot_cod_fee.php 1003 2005-07-10 18:58:52Z mz $)

   Released under the GNU General Public License
   ---------------------------------------------------------------------------------------*/


class GMTSService {
	const _ts_service_url_sandbox = 'https://qa.trustedshops.de/ts/services/TsProtection?wsdl';
	const _ts_service_url         = 'https://www.trustedshops.de/ts/services/TsProtection?wsdl';
	const _ts_fe_service_url_sandbox = 'https://protection-qa.trustedshops.com/ts/protectionservices/ApplicationRequestService?wsdl';
	const _ts_fe_service_url         = 'https://protection.trustedshops.com/ts/protectionservices/ApplicationRequestService?wsdl';
	const _ts_rating_service_url_sandbox = 'https://www.trustedshops.de/ts/services/TsRating?wsdl';
	const _ts_rating_service_url = 'https://qa.trustedshops.de/ts/services/TsRating?wsdl';
	const _ts_rating_user = 'gambio-gmbh';
	const _ts_rating_pass = 'sN7HOUSp';
	const _ts_rating_package = 'GAMBIO';
	const _ts_rating_activation = '1';
	const _shop_system_version = 'Gambio GX2';
	
	protected $_languages = array('de', 'en', 'es', 'fr', 'pl');
	
	protected $_service;
	protected $_service_fe;
	protected $_service_rating;
	protected $_sandboxmode = false;
	protected $_logger;
	
	public function __construct() {
		$this->_logger = new FileLog('trusted_shops', true);
	}
	
	protected function _log($text) {
		$prefix = date('c') . ' | ';
		$this->_logger->write($prefix . $text ."\n");
	}
	
	public function log($text) {
		$this->_log($text);
	}
	
	public function getKey() {
		return $this->_logger->get_secure_token();
	}
	
	/**
	 * Get SOAP client object (administration)
	 */
	protected function _getService() {
		if(!($this->_service instanceof SoapClient)) {
			if($this->_sandboxmode) {
				$wsdl_url = self::_ts_service_url_sandbox;
			}
			else {
				$wsdl_url = self::_ts_service_url;
			}
			try {
				$this->_service = new SoapClient($wsdl_url);
			}
			catch(SoapFault $sf) {
				//die(print_r($sf, true));
				$this->_log('could not connect to web service');
				return false;
			}
		}
		return $this->_service;
	}
	
	/**
	 * Get SOAP client object (front end)
	 */
	protected function _getServiceFE() {
		if(!($this->_service_fe instanceof SoapClient)) {
			if($this->_sandboxmode) {
				$wsdl_url = self::_ts_fe_service_url_sandbox;
			}
			else {
				$wsdl_url = self::_ts_fe_service_url;
			}
			try {
				$this->_service_fe = new SoapClient($wsdl_url);
			}
			catch(SoapFault $sf) {
				//die(print_r($sf, true));
				$this->_log('could not connect to web service');
				return false;
			}
		}
		return $this->_service_fe;
	}
	
	/**
	 * Get SOAP client object (rating)
	 */
	protected function _getServiceRating() {
		if(!($this->_service_rating instanceof SoapClient)) {
			if($this->_sandboxmode) {
				$wsdl_url = self::_ts_rating_service_url_sandbox;
			}
			else {
				$wsdl_url = self::_ts_rating_service_url;
			}
			try {
				$this->_service_rating = new SoapClient($wsdl_url);
			}
			catch(SoapFault $sf) {
				//die(print_r($sf, true));
				$this->_log('could not connect to web service');
				return false;
			}
		}
		return $this->_service_rating;
	}

	/**
	 * Check a certificate (TS-ID) 
	 */
	public function checkCertificate($tsid) {
		$service = $this->_getService();
		try {
			$result = $service->checkCertificate($tsid);
			$this->_log("checked $tsid, new state is ". $result->stateEnum);
			$this->_log("full check data for $tsid:\n".print_r($result, true));
		}
		catch(SoapFault $sf) {
			$this->_log('error checking certificate '. $tsid .' - '. $sf->getMessage());
			return false;
		}
		return $result;
	}
	
	/**
	 * Check a certificate wrt Rating feature 
	 */
	public function updateRatingWidgetState($tsid) {
		$service = $this->_getServiceRating();
		try {
			$result = $service->updateRatingWidgetState($tsid, self::_ts_rating_activation, self::_ts_rating_user, self::_ts_rating_pass, self::_ts_rating_package);
			$this->_log("rating widget state for $tsid:\n".print_r($result, true));
		}
		catch(SoapFault $sf) {
			$this->_log('error checking certificate\'s rating widget state '. $tsid .' - '. $sf->getMessage());
			return false;
		}
		return $result;
	}
	
	/**
	 * Check all certificates 
	 */
	public function checkAllCertificates($cronmode = false) {
		$query = "SELECT * FROM ts_certs";
		if($cronmode) {
			$query .= " WHERE date_checked < DATE_SUB(NOW(), INTERVAL 1 DAY)";
			$this->_log('checking all certificates in cron mode');
		}
		$result = xtc_db_query($query);
		while($row = xtc_db_fetch_array($result)) {
			$cert = $this->checkCertificate($row['tsid']);
			if($cert !== false) {
				$rating_state = $this->updateRatingWidgetState($row['tsid']);
				$cert->rating_ok = $rating_state == 'OK' ? 1 : 0;
				$login_state = $this->checkLogin($row['tsid'], $row['user'], $row['password']);
				$cert->login_ok = $login_state >= 0;
				$this->storeCertificate($cert);
			}
			else {
				$this->_log('ERROR checking certificate with TS ID '. $row['tsid']);
			}
		}
	}
	
	/**
	 * Check web service login credentials for Excellence service 
	 */
	public function checkLogin($tsid, $user, $password) {
		$service = $this->_getService();
		try {
			$this->_log('checking login ('."$tsid, $user, $password".')');
			$result = $service->checkLogin($tsid, $user, $password);
			$this->_log('result: '. $result);
		}
		catch(SoapFault $sf) {
			$this->_log('error checking login ('."$tsid, $user, $password".')'.' - '. $sf->getMessage());
			return false;
		}
		return $result;
	}
	
	/**
	 * Retrieve data on Buyer Protection products 
	 */
	public function getProtectionItems($tsid, $cached = true) {
		if($cached) {
			// use data from local cache if it's present and valid
			$items = $this->_getProtectionItemsFromDb($tsid);
			if(!empty($items)) {
				return $items;
			}
		}
		// can't use cached items, retrieve them from web service
		$service = $this->_getService();
		try {
			$result = $service->getProtectionItems($tsid);
		}
		catch(SoapFault $sf) {
			$this->_log('error retrieving protection item ('."$tsid".')'.' - '. $sf->getMessage());
			return false;
		}
		if(is_array($result->item)) {
			$this->_storeProtectionItemsInDb($tsid, $result->item);
			$items = $this->_getProtectionItemsFromDb($tsid);
			return $items;
		}
		else {
			return false;
		}
	}
	
	/**
	 * Reload protection items for all Excellence IDs (refresh cache contents
	 */
	public function reloadProtectionItems() {
		$certs = $this->retrieveCertificates();
		foreach($certs as $cert) {
			if($cert['type'] == 'EXCELLENCE') {
				$this->getProtectionItems($cert['tsid'], false);
			}
		}
	}
	
	protected function _getProtectionItemsFromDb($tsid) {
		$result = xtc_db_query("SELECT * FROM ts_items WHERE ts_id = '".$tsid."' AND retrievaldate >= DATE_SUB(NOW(), INTERVAL 30 DAY)", 'db_link', false);
		$items = array();
		while($row = xtc_db_fetch_array($result)) {
			$items[] = $row;
		}
		return $items;
	}
	
	protected function _storeProtectionItemsInDb($tsid, $items) {
		// delete old data
		xtc_db_query("DELETE FROM ts_items WHERE ts_id = '".$tsid."'");
		// insert new data
		foreach($items as $item) {
			$data = array(
				'ts_id' => $tsid,
				'retrievaldate' => 'now()',
				'creationdate' => $item->creationDate,
				'id' => $item->id,
				'currency' => $item->currency,
				'netfee' => $item->netFee,
				'grossfee' => $item->grossFee,
				'protectedamount' => $item->protectedAmountDecimal,
				'protectionduration' => $item->protectionDurationInt,
				'tsproductid' => $item->tsProductID,
			);
			xtc_db_perform('ts_items', $data, 'insert');
		}
	}
	
	/**
	 * Determine Buyer Protection product
	 */
	public function findProtectionProduct($tsid, $amount, $currency) {
		$all_products = $this->getProtectionItems($tsid);
		$matching_products = array();
		// filter by currency
		foreach($all_products as $product) {
			if($product['currency'] == $currency) {
				$matching_products[(int)$product['protectedamount']] = $product;
			}
		}
		ksort($matching_products);
		$result = null;
		foreach($matching_products as $pamount => $product) {
			$result = $product;
			if($amount <= $product['protectedamount']) {
				break;
			}
		}
		
		return $result;
	}
		
	/**
	 * Store certificate in local db 
	 */
	public function storeCertificate($certificate) {
		$certs = $this->retrieveCertificates();
		$data = array(
			'tsid' => $certificate->tsID,
			'language' => $certificate->certificationLanguage,
			'state' => $certificate->stateEnum,
			'type' => $certificate->typeEnum,
			'url' => $certificate->url,
			'rating_ok' => $certificate->rating_ok,
			'login_ok' => $certificate->login_ok,
		);
		if(isset($certs[$certificate->tsID])) {
			$data = array_merge($certs[$certificate->tsID], $data);
		}
		$data['date_checked'] = 'now()';
		xtc_db_perform('ts_certs', $data, 'replace');
		return $data;
	}
	
	/**
	 * Retrieve all certificates from local db 
	 */
	public function retrieveCertificates() {
		$result = xtc_db_query("SELECT * FROM ts_certs");
		$certs = array();
		while($row = xtc_db_fetch_array($result)) {
			$certs[$row['tsid']] = $row;
		}
		return $certs;
	}
	
	/**
	 * Determine number of certificates matching language and URL 
	 */
	public function numCertsByLanguageURL($lang, $url) {
		$query = "SELECT COUNT(*) AS num FROM ts_certs WHERE language = '$lang' AND url = '$url'";
		$this->_log($query);
		$result = xtc_db_query($query);
		$row = xtc_db_fetch_array($result);
		return $row['num'];
	}
	
	/**
	 * Delete certificate from local db
	 * 
	 * @param	string	$tsid	A Trusted Shops ID
	 * @return	bool	true if a certificate was deleted
	 */
	public function deleteCertificate($tsid) {
		xtc_db_query("DELETE FROM ts_certs WHERE tsid = '". xtc_db_input($tsid) ."'");
		$affrows = mysql_affected_rows();
		return $affrows > 0;
	}
	
	/**
	 * Change username and password for certificate in local db
	 * 
	 * @param	$tsid		string	A Trusted Shops ID
	 * @param	$username	string	User name
	 * @param	$password	string	Password
	 * @return				bool	true if credentials were changed
	 */
	public function changeCredentials($tsid, $username, $password) {
		$data = array(
			'user' => xtc_db_input($username),
			'password' => xtc_db_input($password),
			'login_ok' => '1',
		);
		xtc_db_perform('ts_certs', $data, 'update', "tsid = '".xtc_db_input($tsid)."'");
		$affrows = mysql_affected_rows();
		return $affrows > 0;
	}
	
	/**
	 * Find TS ID for use in a Seal box 
	 */
	public function findSealID($language) {
		$tsid = $this->findExcellenceID($language);
		if($tsid === false) {
			$tsid = $this->findClassicID($language);
		}
		return $tsid;
	}
	
	/**
	 * Find TS ID for use in Buyer Protection Classic
	 */
	public function findClassicID($language) {
		if(!in_array($language, $this->_languages)) {
			return false;
		}
		$result = xtc_db_query("SELECT * FROM ts_certs WHERE language = '".$language."' AND state IN ('PRODUCTION', 'TEST', 'INTEGRATION') AND type = 'CLASSIC'");
		if(xtc_db_num_rows($result) > 0) {
			$row = xtc_db_fetch_array($result);
			return $row['tsid'];
		}
		return false;
	}
	
	/**
	 * Find TS ID for use in Buyer Protection Excellence
	 */
	public function findExcellenceID($language) {
		if(!in_array($language, $this->_languages)) {
			return false;
		}
		$result = xtc_db_query("SELECT * FROM ts_certs WHERE language = '".$language."' AND state IN ('PRODUCTION', 'TEST', 'INTEGRATION') AND type = 'EXCELLENCE' AND login_ok = 1");
		if(xtc_db_num_rows($result) > 0) {
			$row = xtc_db_fetch_array($result);
			return $row['tsid'];
		}
		return false;
	}
	
	/**
	 * Find TS ID for use in buyer rating 
	 */
	public function findRatingID($language) {
		if(!in_array($language, $this->_languages)) {
			return false;
		}
		$result = xtc_db_query("SELECT * FROM ts_certs WHERE (language = '".$language."' OR language = '') AND rating_ok = 1");
		if(xtc_db_num_rows($result) > 0) {
			$row = xtc_db_fetch_array($result);
			return $row['tsid'];
		}
		/*
		$result = xtc_db_query("SELECT * FROM ts_certs WHERE language = '".$language."' AND type = 'EXCELLENCE' AND rating_ok = 1");
		if(xtc_db_num_rows($result) > 0) {
			$row = xtc_db_fetch_array($result);
			return $row['tsid'];
		}
		$result = xtc_db_query("SELECT * FROM ts_certs WHERE language = '".$language."' AND type = 'CLASSIC' AND rating_ok = 1");
		if(xtc_db_num_rows($result) > 0) {
			$row = xtc_db_fetch_array($result);
			return $row['tsid'];
		}*/
		return false;
	}
	
    /**
     * Mapping of Gambio payment method identifiers to Trusted Shops constants 
     */
    public function getPaymentMapping($gm_payment_method) {
		if(strpos($gm_payment_method, 'moneybookers') !== false || strpos($gm_payment_method, 'skrill') !== false) {
			$gm_payment_method = 'moneybookers';
		}
		if(preg_match('/^hp.*/', $gm_payment_method) == 1) {
			$gm_payment_method = 'hp';
		}
		
        switch($gm_payment_method) {
			case 'banktransfer':
				$payment = 'DIRECT_DEBIT';
				break;
			case 'cash':
				$payment = 'CASH_ON_PICKUP';
				break;
			case 'cc':
				$payment = 'CREDIT_CARD';
				break;
			case 'cod':
				$payment = 'CASH_ON_DELIVERY';
				break;
			case 'eustandardtransfer':
				$payment = 'PREPAYMENT';
				break;
			case 'hp':
				$payment = 'OTHER';
				break;
			case 'invoice':
				$payment = 'INVOICE';
				break;
			case 'moneybookers':
				$payment = 'MONEYBOOKERS';
				break;
			case 'moneyorder':
				$payment = 'PREPAYMENT';
				break;
			case 'paypalexpress':
				$payment = 'PAYPAL';
				break;
			case 'paypalgambio_alt':
				$payment = 'PAYPAL';
				break;
			case 'paypal':
				$payment = 'PAYPAL';
				break;
			case 'pn_sofortueberweisung':
				$payment = 'DIRECT_E_BANKING';
				break;
			case 'sofort_lastschrift':
				$payment = 'DIRECT_DEBIT';
				break;
			case 'sofort_sofortlastschrift':
				$payment = 'DIRECT_DEBIT';
				break;
			case 'sofort_sofortrechnung':
				$payment = 'INVOICE';
				break;
			case 'sofort_sofortueberweisung':
				$payment = 'DIRECT_E_BANKING';
				break;
			case 'sofort_sofortvorkasse':
				$payment = 'PREPAYMENT';
				break;
			case 'vrepay_elv':
				$payment = 'DIRECT_DEBIT';
				break;
			case 'vrepay_giropay':
				$payment = 'GIROPAY';
				break;
			case 'vrepay_kreditkarte':
				$payment = 'CREDIT_CARD';
				break;
            default:
                $payment = 'OTHER';
        }
        return $payment;
    }
	
	/**
	 * Request buyer protection (Excellence) 
	 */
	public function requestForProtection($tsid, $tsproductid, $amount, $currency, $paymentType, $buyerEmail, $shopCustomerID, $orderDate, $order_id) {
		$this->_log("Application for Buyer Protection: $tsid # $tsproductid # $amount # $currency # $paymentType # $buyerEmail # $shopCustomerID # $orderDate");
		$service = $this->_getServiceFE();
		$credentials = $this->_getCredentials($tsid);
		if($credentials == false) {
			$this->_log("ERROR: requestForProtection called w/ invalid TSID (missing credentials)");
			return false;
		}
		try {
			$result = $service->requestForProtectionV2(
					$tsid,
					$tsproductid,
					$amount,
					$currency,
					$paymentType,
					$buyerEmail,
					$shopCustomerID,
					$order_id,
					$orderDate,
					self::_shop_system_version,
					$credentials['user'],
					$credentials['password']);
		}
		catch(SoapFault $sf) {
			$this->_log('error in requestForProtection - '. $sf->getMessage());
			return false;
		}
		
		if($result < 0) {
			$this->_log('error in requestForProtection - '. $this->errorText($result));
			return false;
		}
		$this->_log("request for protection granted, application number: $result");
		xtc_db_perform('ts_protection', array('orders_id' => $order_id, 'application_number' => $result, 'tsid' => $tsid));
		return $result;
	}
	
	/*
	 * get request state
	 */
	public function getRequestState($application_id) {
		$query = xtc_db_query("SELECT * FROM ts_protection WHERE application_number = ". (int)$application_id);
		if(xtc_db_num_rows($query) > 0) {
			$data = xtc_db_fetch_array($query);
			$service = $this->_getServiceFE();
			try {
				$result = $service->getRequestState($data['tsid'], $data['application_number']);
			}
			catch(SoapFault $sf) {
				$this->_log('error in getRequestState - '. $sf->getMessage());
				return false;
			}
			xtc_db_query("UPDATE ts_protection SET result = ". (int)$result ." WHERE application_number = ". (int)$application_id);
			return $result;
		}
		else {
			return false;
		}
	}
	
	
	/**
	 * Get login user name and password for a given TSID 
	 */
	protected function _getCredentials($tsid) {
		$result = xtc_db_query("SELECT user, password FROM ts_certs WHERE tsid = '". $tsid ."'");
		if(xtc_db_num_rows($result) > 0) {
			$row = xtc_db_fetch_array($result);
			return $row;
		}
		else {
			return false;
		}
	}

	
	public function errorText($resultcode) {
		$errorcodes = array(
			'-10001' => 'Web service login or TS-ID invalid',
			'-10002' =>	'The shop\'s credit limit has been suspended by Trusted Shops',
			'-10003' => 'Order number already used',
			'-10004' => 'Unsupported Buyer Protection product',
			'-10005' => 'Inactive payment method',
			'-10006' => 'Unsupported payment method',
			'-10007' => 'The currency of the Buyer Protection product does not match with the currency of the shopping basket',
			'-10008' => 'This currency is not supported in this shop',
			'-10009' => 'This exchange rate is not supported',
			'-10010' => 'This payment method is not supported',
			'-10011' => 'No credit limit available for this certificate',
			'-10012' => 'The delivery date is in the past',
			'-10013' => 'The guarantee is for a purchase that was made over 3 days ago',
			'-10014' => 'The email address contains an error',
			'-10015' => 'No order number was assigned',
			'-10016' => 'No customer number was assigned',
			'-10017' => 'The credit limit for this certificate has been exceeded',
			'-10018' => 'No email address was assigned',
			'-10019' => 'Non-applicable Buyer Protection product',
			'-11001' => 'Invalid security token',
			'-11111' => 'General system error, please contact Trusted Shops',
		);
		
		if(isset($errorcodes[$resultcode])) {
			$text = $errorcodes[$resultcode];
		}
		else {
			$text = 'unknown error ('.$resultcode.')';
		}

		return $text;
	}
	
	/**
	 * Generates rating link button
	 */
	public function getRatingLink($orders_id, $customer_email) {
		$link_url = array(
			'de' => 'https://www.trustedshops.de/bewertung/bewerten_',
			'en' => 'https://www.trustedshops.com/buyerrating/rate_',
			'es' => 'https://www.trustedshops.es/evaluacion/evaluar_',
			'fr' => 'https://www.trustedshops.fr/evaluation/evaluer_',
			'pl' => 'https://www.trustedshops.pl/opinia/ocen_',
		);
		$button_url = array(
			'de' => 'https://www.trustedshops.com/bewertung/widget/img/bewerten_de.gif',
			'en' => 'https://www.trustedshops.com/bewertung/widget/img/bewerten_en.gif',
			'es' => 'https://www.trustedshops.com/bewertung/widget/img/bewerten_es.gif',
			'fr' => 'https://www.trustedshops.com/bewertung/widget/img/bewerten_fr.gif',
			'pl' => 'https://www.trustedshops.com/bewertung/widget/img/bewerten_pl.gif',
		);
		$language = $_SESSION['language_code'];
		$tsid = $this->findRatingID($language);
		if($tsid == false) {
			return '';
		}
		$link = '<a href="'. $link_url[$language] .$tsid.'.html&buyerEmail='.urlencode(base64_encode($customer_email)).'&shopOrderID='.urlencode(base64_encode($orders_id)).'" target="_new">';
		$link .= '<img src="'. $button_url[$language] .'">';
		$link .= '</a>';
		return $link;
	}
	
	/**
	 * Generates rating widget 
	 */
	public function getWidgetLink() {
		$link_url = array(
			'de' => 'https://www.trustedshops.de/bewertung/info_',
			'en' => 'https://www.trustedshops.com/buyerrating/info_',
			'es' => 'https://www.trustedshops.es/evaluacion/informacion_',
			'fr' => 'https://www.trustedshops.fr/evaluation/info_',
			'pl' => 'https://www.trustedshops.pl/opinia/info_',
		);
		$language = $_SESSION['language_code'];
		$tsid = $this->findRatingID($language);
		if($tsid == false) {
			return '';
		}
		$widget_img = 'https://www.trustedshops.com/bewertung/widget/widgets/'.$tsid.'.gif';
		$html = '<a href="'. $link_url[$language] . $tsid .'.html" target="_blank">';
		$html .= '<img src="'. $widget_img .'">';
		$html .= '</a>';
		
		return $html;
	}
}
