<?php
define('SHOPGATE_PLUGIN_VERSION', '2.1.42');

/**
 * GambioGX Plugin for Shopgate
 */
class ShopgatePluginGambioGX extends ShopgatePlugin {

	/**
	 * @var ShopgateConfigGambioGx
	 */
	protected $config;

	private $languageId;
	private $countryId;
	private $zoneId;

	private $currency;

	private $language = "german";

	public function startup() {
		global $gmLangFileMaster, $lng;
		
		if(!defined('TABLE_PRODUCTS_PROPERTIES_COMBIS'))		define('TABLE_PRODUCTS_PROPERTIES_COMBIS',			'products_properties_combis');
		if(!defined('TABLE_PRODUCTS_PROPERTIES_COMBIS_VALUES'))	define('TABLE_PRODUCTS_PROPERTIES_COMBIS_VALUES',	'products_properties_combis_values');
		if(!defined('TABLE_PROPERTIES_VALUES'))					define('TABLE_PROPERTIES_VALUES',					'properties_values');
		if(!defined('TABLE_PROPERTIES_VALUES_DESCRIPTION'))		define('TABLE_PROPERTIES_VALUES_DESCRIPTION',		'properties_values_description');
		if(!defined('TABLE_PROPERTIES'))						define('TABLE_PROPERTIES',							'properties');
		if(!defined('TABLE_PROPERTIES_DESCRIPTION'))			define('TABLE_PROPERTIES_DESCRIPTION',				'properties_description');
		if(!defined('TABLE_ORDERS_PRODUCTS_PROPERTIES'))		define('TABLE_ORDERS_PRODUCTS_PROPERTIES',			'orders_products_properties');
		
		$requiredFiles = array(
			DIR_FS_CATALOG.'inc/xtc_validate_password.inc.php',
			DIR_FS_CATALOG.'inc/xtc_format_price_order.inc.php',
			DIR_FS_CATALOG.'inc/xtc_db_prepare_input.inc.php',
			DIR_FS_CATALOG.'inc/xtc_get_tax_class_id.inc.php',
			DIR_FS_CATALOG.'includes/classes/xtcPrice.php',
			DIR_FS_CATALOG.'gm/classes/GMSEOBoost.php',
			DIR_FS_CATALOG.'includes/modules/shipping/dp.php',
			DIR_FS_CATALOG.'gm/inc/gm_get_conf.inc.php',
			DIR_FS_CATALOG.'inc/xtc_get_products_stock.inc.php',
		);
		
		// language class could be already
		if(!class_exists('language')) {
			$requiredFiles[] = DIR_WS_CLASSES.'language.php';
		}

		if(file_exists(dirname(__FILE__).'/../release_info.php')){
			include(dirname(__FILE__).'/../release_info.php');

			if(isset($gx_version) && mb_substr($gx_version, 0, 2) == 'v2'){
				// only on version 2.xxxx
				require_once DIR_FS_CATALOG.'system/core/Debugger.inc.php';
				$GLOBALS['coo_debugger'] = new Debugger();
			}
		}

		foreach($requiredFiles as $file) {
			require_once($file);
		}

		require_once(DIR_FS_CATALOG.'shopgate/gambiogx/shopgate_config.php');
		$this->config = new ShopgateConfigGambioGx();
		if (!isset($_REQUEST['shop_number'])) {
			$this->config->loadFile();
		} else {
			$this->config->loadByShopNumber($_REQUEST['shop_number']);
		}
		
		// check if the shopgate table is on
		if($this->config->getShopgateTableVersion() != SHOPGATE_PLUGIN_VERSION) {
			// reinstall the payment module
			require_once(DIR_FS_CATALOG.'shopgate/gambiogx/includes/modules/payment/shopgate.php');
			$shopgatePaymentModule = new shopgate();
			if($shopgatePaymentModule->check()) {
				$shopgatePaymentModule->remove();
			}
			$shopgatePaymentModule->install();
		}
		
		// fetch country
		$qry = "SELECT * FROM `".TABLE_COUNTRIES."` WHERE UPPER(countries_iso_code_2) = UPPER('".$this->config->getCountry()."')";
		$result = xtc_db_query($qry);
		$qry = xtc_db_fetch_array($result);
		$this->countryId = !empty($qry['countries_id']) ? $qry['countries_id'] : 'DE';

		// fetch language
		$qry = "SELECT * FROM `".TABLE_LANGUAGES."` WHERE UPPER(code) = UPPER('".$this->config->getLanguage()."')";
		$result = xtc_db_query($qry);
		$qry = xtc_db_fetch_array($result);
		$this->languageId = !empty($qry['languages_id']) ? $qry['languages_id'] : 2;
		$this->language = !empty($qry['directory']) ? $qry['directory'] : 'german';

		// fetch currency
		$qry = "SELECT * FROM `".TABLE_CURRENCIES."` WHERE UPPER(code) = UPPER('".$this->config->getCurrency()."')";
		$result = xtc_db_query($qry);
		$qry = xtc_db_fetch_array($result);
		$this->exchangeRate = !empty($qry['value']) ? $qry['value'] : 1;
		$this->currencyId = !empty($qry['currencies_id']) ? $qry['currencies_id'] : 1;
		$this->currency = !empty($qry)
			? $qry
			: array('code' => 'EUR', 'symbol_left' => '', 'symbol_right' => ' EUR', 'decimal_point' => ',', 'thousands_point' => '.', 'decimal_places' => '2', 'value' => 1.0);
		
		$this->zoneId = $this->config->getTaxZoneId();

		if (!defined('DIR_FS_LANGUAGES')) {
			define('DIR_FS_LANGUAGES', rtrim(DIR_FS_CATALOG, '/'). '/lang/');
		}
		
		$langFiles = array(
			'admin/categories.php',
			'admin/content_manager.php',
		);

		foreach($langFiles as $langFile) {
			include_once rtrim(DIR_FS_LANGUAGES, '/')."/$this->language/$langFile";
		}

		// Load shopgate-admin-language-files additionaly
		include_once("gambiogx/lang/$this->language/admin/$this->language.php");
		
		// Get Gambio GX version
		$this->gambioGXVersion = array(
			'main_version' => '1',
			'sub_version' => '0',
			'revision' => '0',
		);
		$gxVersionFileDestination = '/'.trim(DIR_FS_CATALOG, '/').'/release_info.php';
		if(file_exists($gxVersionFileDestination)) {
			require_once $gxVersionFileDestination;
			if(preg_match('/(?P<main_version>[1-9]+).(?P<sub_version>[0-9]+).(?P<revision>[0-9]+)/', $gx_version, $matches)) {
				$this->gambioGXVersion = array(
					'main_version' => $matches['main_version'],
					'sub_version' => $matches['sub_version'],
					'revision' => $matches['revision'],
				);
			}
		}

		return true;
	}

	public function createPluginInfo() {
		$aInfos = array();
	
		if(file_exists(dirname(__FILE__).'/../release_info.php')){
			include(dirname(__FILE__).'/../release_info.php');
		}
		
		if(isset($gx_version)){
			$aInfos['shop_version'] = $gx_version;
		}
	
		return $aInfos;
	}
	
	protected function createCategoriesCsv() {
		$this->gmSEOBoost = new GMSEOBoost();
		$this->gmSEOBoostCategoriesEnabled = false;
		if(file_exists(DIR_FS_CATALOG.'.htaccess') && function_exists('gm_get_conf')) {
			$this->gmSEOBoostCategoriesEnabled = gm_get_conf('GM_SEO_BOOST_CATEGORIES')?1:0;
		}
		
		if($this->config->getReverseCategoriesSortOrder()){
			$maxOrder = 0;
		} else {
			$qry = "SELECT MAX( sort_order ) sort_order FROM " . TABLE_CATEGORIES;
			$result = xtc_db_query( $qry );
			$maxOrder = xtc_db_fetch_array( $result );
			$maxOrder = $maxOrder["sort_order"] + 1;
		}
		
		$this->_buildCategoriesTree(0, $maxOrder);
	}

	private function _buildCategoriesTree($parentId = 0, $maxOrder = 0) {
		$qry = "
		SELECT DISTINCT
		c.categories_id,
		c.parent_id,
		c.categories_image,
		c.categories_status,
		c.sort_order,
		cd.categories_name
		FROM ".TABLE_CATEGORIES." c
		INNER JOIN ".TABLE_CATEGORIES_DESCRIPTION." cd ON (c.categories_id = cd.categories_id
		AND cd.language_id = $this->languageId)
		WHERE c.parent_id = $parentId" .
		// Prevent from taking categories that are inactive for the desired customer price group
		((GROUP_CHECK == "true")
			? (" AND c.group_permission_{$this->config->getCustomerPriceGroup()} = 1")
			: ('')
		) .
		"
		ORDER BY c.categories_id ASC
		";

		$qry = xtc_db_query( $qry );

		while( $item = xtc_db_fetch_array( $qry ) ) {

			$row = $this->buildDefaultCategoryRow();

			$row["category_number"] = $item["categories_id"];
			$row["parent_id"] = (empty($item["parent_id"]) || ($item['parent_id'] == $item['categories_id']))
				? ""
				: $item["parent_id"];
			$row["category_name"] = htmlentities($item["categories_name"], ENT_NOQUOTES, $this->config->getEncoding());

			if(!empty($item["categories_image"])){
				$row["url_image"] = HTTP_SERVER.DIR_WS_CATALOG.DIR_WS_IMAGES."categories/".$item["categories_image"];
			}

			if (!empty($item["sort_order"]) || ((string) $item['sort_order'] === '0')) {
				if($this->config->getReverseCategoriesSortOrder()){
					// reversed means the contrary to ordering system in shopgate - order_index is a priority system - high number = top position
					// so just taking over the values means reversing the order
					$row["order_index"] = $item["sort_order"];
				} else {
					$row["order_index"] = $maxOrder - $item["sort_order"];
				}
			}
			
			$row["is_active"] = $item["categories_status"];
			if($this->gmSEOBoostCategoriesEnabled) {
				$row["url_deeplink"] = $this->gmSEOBoost->get_boosted_category_url($item["categories_id"], $this->languageId);
			} else {
				$row["url_deeplink"] = "index.php?cat=c" . $item["categories_id"];
			}
			$row["url_deeplink"] = HTTP_SERVER.DIR_WS_CATALOG.$row["url_deeplink"];
			
			$this->addCategoryRow($row);
			
			if ($item['parent_id'] != $item['categories_id']) {
				$this->_buildCategoriesTree($item["categories_id"], $maxOrder);
			}
		}
	}

	/**
	 * returns all sub categories including the given parent as a list that is a mapping from one category to a higher category if a given depth is exceeded
	 * @param int $maxDepth
	 * @param int $parentId
	 * @param int $copyId
	 * @param int $depth
	 * @throws ShopgateLibraryException
	 * @return array
	 */
	private function _getCategoryReducementMap($maxDepth = null, $parentId = null, $copyId = null, $depth = null) {
		$circularDepthStop = 50;
		if(empty($depth)) {
			$depth = 1;
		} elseif($depth > $circularDepthStop) {
			// disallow circular category connections (detect by a maximum depth)
			throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_DATABASE_ERROR, 'error on loading sub-categories: Categories-Depth exceedes a value of '.$circularDepthStop.'. Check if there is a circular connection (referenced categories ids: '.$parentId.'=>'.$row['categories_id'].')', true);
		}
		
		$qry =
			"SELECT `categories_id` FROM `".TABLE_CATEGORIES."` WHERE" .
			// select by parent id, if set
			(!empty($parentId)
				? " (`parent_id` = '{$parentId}')"
				: " (`parent_id` IS NULL OR `parent_id` = 0 OR `parent_id` = '')"
			)
		;
		
		$qryResult = xtc_db_query($qry);
		if(!$qryResult) {
			throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_DATABASE_ERROR, 'error on selecting categories', true);
		}
	
		// add all sub categories to a simple one-dimensional array
		$categoryMap = array();
		while($row = xtc_db_fetch_array($qryResult)) {
			// copy only if a maximum depth is set, yet
			if(!empty($maxDepth)) {
				if($depth == $maxDepth) {
					$copyId = $row['categories_id'];
				}
			}
			// Check if a mapping to a higher category needs to be applied
			if(!empty($copyId) && !empty($row['categories_id'])) {
				$categoryMap[$row['categories_id']] = $copyId;
			} else {
				// no mapping to other categories, map to itself!
				$categoryMap[$row['categories_id']] = $row['categories_id'];
			}
			
			$subCategories = $this->_getCategoryReducementMap($maxDepth, $row['categories_id'], $copyId, $depth+1);
			if(!empty($subCategories)) {
				$categoryMap = $categoryMap+$subCategories;
			}
		}
		
		return $categoryMap;
	}

	/**
	 * @see ShopgatePluginApi::createItemsCsv()
	 */
	protected function createItemsCsv() {
		$this->gmSEOBoost = new GMSEOBoost();
		$this->gmSEOBoostProductsEnabled = false;
		if(file_exists(DIR_FS_CATALOG.'.htaccess') && function_exists('gm_get_conf')) {
			$this->gmSEOBoostProductsEnabled = gm_get_conf('GM_SEO_BOOST_PRODUCTS')?1:0;
		}

		// get article properties (only for gambio gx 2_0_7 or higher)
		$gxProductsPropertiesSupportEnabled = false;
		if($this->gambioGXVersion['main_version'] > 2
		|| $this->gambioGXVersion['main_version'] == 2 && ($this->gambioGXVersion['sub_version'] > 0 OR $this->gambioGXVersion['revision'] >= 7)) {
			$gxProductsPropertiesSupportEnabled = true;
		}
		
		$gxCustomizerSets = $this->getGxCustomizerSets();
		$inputFieldsFromGxCustomizer = $this->getInputFieldsFromGxCustomizerSets($gxCustomizerSets);
		
		$customerGroupMaxPriceDiscount = 0;
		$customerGroupDiscountAttributes = false;

		// get customer-group first
		$qry = "SELECT"
		. " status.customers_status_name,"
		. " status.customers_status_discount,"
		. " status.customers_status_discount_attributes"
		. " FROM " . TABLE_CUSTOMERS_STATUS . " AS status"
		. " WHERE status.customers_status_id = " . $this->config->getCustomerPriceGroup()
		. " AND status.language_id = " . $this->languageId
		. ";";
		
		// Check if the customer group exists (ignore if not)
		$queryResult = xtc_db_query($qry);
		if($queryResult) {
			$customerGroupResult = xtc_db_fetch_array($queryResult);
			if(!empty($customerGroupResult) && isset($customerGroupResult['customers_status_discount'])) {
				$customerGroupMaxPriceDiscount = $customerGroupResult['customers_status_discount'];
			}
			if(!empty($customerGroupResult) && isset($customerGroupResult['customers_status_discount'])) {
				$customerGroupDiscountAttributes = $customerGroupResult['customers_status_discount_attributes'] ? true : false;
			}
		}
		
		// Group check for products (also do not export any products thate are stored in blacklisted categories for the actual customer group)
		$categoriesBlacklist = $this->getCategoriesBlacklist($this->config->getCustomerPriceGroup());
		
		$categoryReducementMap = array();
		$maxCatDepth = $this->config->getMaximumCategoryExportDepth();
		if(!empty($maxCatDepth)) {
			$categoryReducementMap = $this->_getCategoryReducementMap($maxCatDepth);
		}
		
		$qry = "
			SELECT DISTINCT
				p.products_id,
				p.products_model,
				p.products_ean,
				p.products_quantity,
				p.products_image,
				p.products_price,
				DATE_FORMAT(p.products_last_modified, '%Y-%m-%d') as products_last_modified,
				p.products_weight,
				p.products_status,
				sp.specials_new_products_price,
				sp.specials_quantity,
				pdsc.products_keywords,
				pdsc.products_name,
				pdsc.products_description,
				pdsc.products_short_description,
				shst.shipping_status_name,
				mf.manufacturers_name,
				p.products_tax_class_id,
				p.products_fsk18,
				p.products_vpe_status,
				p.products_vpe_value,
				vpe.products_vpe_name,
				p.gm_price_status,
				p.products_sort,
				p.products_startpage,
				p.products_startpage_sort,
				p.products_discount_allowed,
				p.products_date_available,
				p.gm_show_weight,
				p.gm_show_qty_info,
				p.gm_min_order,
				p.gm_graduated_qty,
				p.products_discount_allowed,
				pa.products_id AS attributes_products_id" .
				($gxProductsPropertiesSupportEnabled
				? (",
				ppc.products_id AS properties_products_id,
				p.properties_dropdown_mode,
				p.properties_show_price,
				p.use_properties_combis_weight,
				p.use_properties_combis_quantity,
				p.use_properties_combis_shipping_time")
				: "").
				(!empty($gxCustomizerSets)
				? (",
				sgtp.gm_gprint_surfaces_groups_id")
				: "").
				"
			FROM ".TABLE_PRODUCTS." p
			LEFT JOIN ".TABLE_PRODUCTS_DESCRIPTION." pdsc ON (p.products_id = pdsc.products_id AND pdsc.language_id = '".$this->languageId."')
			LEFT JOIN ".TABLE_SHIPPING_STATUS." shst ON (p.products_shippingtime = shst.shipping_status_id AND shst.language_id = '".$this->languageId."')
			LEFT JOIN ".TABLE_MANUFACTURERS." mf ON (mf.manufacturers_id = p.manufacturers_id)
			LEFT JOIN ".TABLE_SPECIALS." sp ON (sp.products_id = p.products_id AND sp.status = 1 AND (sp.expires_date > now() OR sp.expires_date = '0000-00-00 00:00:00' OR sp.expires_date IS NULL))
			LEFT JOIN ".TABLE_PRODUCTS_VPE." vpe ON (vpe.products_vpe_id = p.products_vpe AND vpe.language_id = pdsc.language_id)
			
			LEFT JOIN ".TABLE_PRODUCTS_ATTRIBUTES." pa ON (pa.products_id=p.products_id)
			" .
			// Get the connected gx customizer set if available
			((!empty($gxCustomizerSets))
				? ("LEFT JOIN gm_gprint_surfaces_groups_to_products sgtp ON (sgtp.products_id=p.products_id)
			")	: ("")
			) .
			// Get products property combinations if available
			(($gxProductsPropertiesSupportEnabled)
				? ("LEFT JOIN ".TABLE_PRODUCTS_PROPERTIES_COMBIS." ppc ON (ppc.products_id=p.products_id)
			")	: ("")
			) .
			// Exclude products in blacklisted categories if set up to do so
			((GROUP_CHECK == 'true' && !empty($categoriesBlacklist))
				? ("LEFT JOIN ".TABLE_PRODUCTS_TO_CATEGORIES." ptc ON (ptc.products_id=p.products_id AND ptc.categories_id NOT IN (".(implode(", ", $categoriesBlacklist))."))
			")	: ("")
			) .
			"WHERE p.products_status = 1
			" .
			// Exclude products for the actual price group if set up to do so
			((GROUP_CHECK == 'true')
				? ("AND p.group_permission_{$this->config->getCustomerPriceGroup()} = 1
			")	: ("")
			);
		
		// Filter by products-attributes or products-properties-combis
		if($this->config->getVariationType() == SHOPGATE_SETTING_VARIATION_TYPE_PROPERTY && $gxProductsPropertiesSupportEnabled) {
			$qry .= "	AND pa.products_attributes_id IS NULL
			";
		} elseif($this->config->getVariationType() == SHOPGATE_SETTING_VARIATION_TYPE_ATTRIBUTE && $gxProductsPropertiesSupportEnabled) {
			$qry .= "	AND ppc.products_properties_combis_id IS NULL
			";
		} elseif($gxProductsPropertiesSupportEnabled) {
			// export both variation types (products that used both variation types at the same time will be deactivated and no child products are appended, because the functionality would become too complex)
			// -> the export would have to do a cross-product on every attribute-value-pair and products-properties-combi and set a special dataset into the internal_order_info to recognize the right combination of these selections
		}
		
		// the product dataset will have two extra fields: properties_products_id and attributes_products_id, the id's are for each product as information about if the variant type is used by the product, null/empty otherwise
		
		// sp.specials_quantity > 0 deleted, to handle the check in the loop
		
		if(STOCK_CHECK == "true" && STOCK_ALLOW_CHECKOUT == 'false') {
			$qry .= " AND p.products_quantity > 0 ";
		}
		
		// Code for enabling to download specific products (for debugging purposes only, at this time)
		if(!empty($_REQUEST['item_numbers']) && is_array($_REQUEST['item_numbers'])) {
			$qry .= " AND p.products_id IN ('".implode("', '", $_REQUEST['item_numbers'])."') ";
		}
		
		// Fix for "Ahorn24" shop. 10 products were not found without sorting.
		$qry .= ' ORDER BY p.products_id ASC ';
		
		$result = xtc_db_query("SELECT MAX(products_id) max_id FROM ".TABLE_PRODUCTS);
		$maxId = xtc_db_fetch_array( $result );
		$maxId = $maxId["max_id"];
		
		// order_index for the products
		$result = xtc_db_query("SELECT MIN(products_sort) AS 'min_order', MAX(products_sort) AS 'max_order' FROM ".TABLE_PRODUCTS);
		$orderIndices = xtc_db_fetch_array( $result );
		$maxOrder = $orderIndices["max_order"]+1;
		$minOrder = $orderIndices["min_order"];
		$addToOrderIndex = 0;
		
		if($minOrder < 0) {
			// make the sort_order positive
			$addToOrderIndex += abs($minOrder);
		}
		
		if($this->splittedExport) $qry .= " LIMIT {$this->exportOffset}, {$this->exportLimit}";
		
		$query = xtc_db_query($qry);
		while($item = xtc_db_fetch_array($query)) {
			$itemArr = $this->buildDefaultItemRow();
			$orderInfos = array();
			
			$tax_rate = xtc_get_tax_rate($item["products_tax_class_id"], $this->countryId, $this->zoneId);
			
			// Get variantions and input fields
			$variations = $this->_getVariations($item, $tax_rate);
			
			// add imput fields if the product uses the customizer
			if(!empty($item['gm_gprint_surfaces_groups_id']) && !empty($inputFieldsFromGxCustomizer[$item['gm_gprint_surfaces_groups_id']])) {
				$inputFields = $inputFieldsFromGxCustomizer[$item['gm_gprint_surfaces_groups_id']];
			} else {
				$inputFields = $this->_getInputFields($item["products_id"]);
			}
			
			// Check if both variant types are used by the product
			$deactivateProduct = false;
			if(!empty($item['attributes_products_id']) && !empty($item['properties_products_id'])) {
				// set a warning message and deactivate the product
				$orderInfos['plugin_warning_information'] = "The product [products_id={$item['products_id']}] will be exported as deactivated, because it uses products attributes and products properties at the same time!";
				$deactivateProduct = true;
				
				// no variations are exported in this case!
				$variations = array();
			}
			
			// Get categories
			$categories = $this->_getProductPath($item["products_id"]);
			
			// Get Image Urls
			$images = $this->_getProductsImages($item);
			
			$deeplink = '';
			$httpServer = HTTP_SERVER;
			if(empty($httpServer)) $httpServer = "http://".$_SERVER["HTTP_HOST"];
			if($this->gmSEOBoostProductsEnabled) {
				$deeplink = $this->gmSEOBoost->get_boosted_product_url($item['products_id'], '', $this->languageId);
			} else {
				$deeplink = "product_info.php?". xtc_product_link($item['products_id'], $item['products_name']);
			}
			$deeplink = $httpServer.DIR_WS_CATALOG.$deeplink;
			
			// Calculate the price
			$price = $item["products_price"];
			$oldPrice = '';
			
			// Special offers for a Customer group
			$pOffers = $this->_getPersonalOffersPrice($item, $tax_rate);
			if(!empty($pOffers) && round($pOffers, 2) > 0) {
				$price = $pOffers;
				// Gambio also tells the old price if the offer is lower than the actual price for the customer group
				if($pOffers < $item["products_price"]) {
					$oldPrice = $item["products_price"];
				}
			}
			
			// General special offer or customer group price reduction
			$productDiscount = 0;
			if(!empty($item["specials_new_products_price"])) {
				if(STOCK_CHECK == 'true' && STOCK_ALLOW_CHECKOUT == 'false') {
					if($item["specials_quantity"] > 0){
						// Nur wenn die quantity > 0 ist dann specialprice setzen, ansonsten normalen Preis mit normalem Stock
						$item["products_quantity"] = $item["specials_quantity"] > $item["products_quantity"] ? $item["products_quantity"] : $item["specials_quantity"];
					}
				}
				// setting specialprice
				$oldPrice = $item["products_price"];
				$price = $item["specials_new_products_price"];
				
				$orderInfos['is_special_price'] = 1;
				
			} elseif(!empty($customerGroupMaxPriceDiscount) && round($customerGroupMaxPriceDiscount, 2) > 0
				  && !empty($item['products_discount_allowed']) && round($item['products_discount_allowed'], 2) > 0) {
				$productDiscount = round($item['products_discount_allowed'], 2);
				
				// Limit discount to the customer groups maximum discount
				if(round($customerGroupMaxPriceDiscount, 2) < $productDiscount) {
					$productDiscount = round($customerGroupMaxPriceDiscount, 2);
				}
				
				// IMPORTANT NOTICE: GambioGX shoppingsystem takes the higher price value between the offers and the default price,
				// when a discount price is set and then subtracts the discount set for the article
				if($item["products_price"] > $price) {
					$price = $item["products_price"];
				}
				$oldPrice = $price;
				if($oldPrice < $item['products_price']) {
					$oldPrice = $item['products_price'];
				}
				
				// Reduce price to the discounted price
				$price = $this->_getDiscountPrice($price, $productDiscount);
				
				$orderInfos['is_special_price'] = 1;
			}
			
			$category_numbers = $this->_getProductCategoryNumbers($item, $maxOrder, $addToOrderIndex);
			// check if there is a category replacement map to reduce categories depth
			if(!empty($categoryReducementMap)) {
				foreach($category_numbers as &$categoryNumber) {
					// can possibly contain a split symbol "=>"
					if(strpos($categoryNumber, '=>') !== FALSE) {
						$catNumberParts = explode('=>', $categoryNumber);
						$catNumberParts[0] = $categoryReducementMap[$catNumberParts[0]];
						$categoryNumber = implode('=>', $catNumberParts);
					} else {
						$categoryNumber = $categoryReducementMap[$categoryNumber];
					}
				}
			}
			
			$price *= $this->exchangeRate;
			$price = $price * ( 1 + ( $tax_rate / 100 ) );
			
			if(!empty($oldPrice)) {
				$oldPrice = $oldPrice * $this->exchangeRate;
				$oldPrice = $this->formatPriceNumber( $oldPrice * ( 1 + ( $tax_rate / 100 ) ) );
			}

			// create the description, based on the settings
			$desc = $this->removeTagsFromString($item["products_description"]);
			$shortDesc = $this->removeTagsFromString($item["products_short_description"]);
			$description = '';
			switch($this->config->getExportDescriptionType()) {
				case SHOPGATE_SETTING_EXPORT_DESCRIPTION:
					$description = $desc;
					break;
				case SHOPGATE_SETTING_EXPORT_SHORTDESCRIPTION:
					$description = $shortDesc;
					break;
				case SHOPGATE_SETTING_EXPORT_DESCRIPTION_SHORTDESCRIPTION:
					$description = $desc . "<br/><br/>" . $shortDesc;
					break;
				case SHOPGATE_SETTING_EXPORT_SHORTDESCRIPTION_DESCRIPTION:
					$description = $shortDesc . "<br/><br/>" . $desc;
					break;
			}
			
			$description = preg_replace("/\n|\r/", "", $this->_parseDescription($description));
			
			$itemArr['item_number'] = $item["products_id"];
			$itemArr['item_number_public'] = $item['products_model'];
			$itemArr['manufacturer'] = $item["manufacturers_name"];
			$itemArr['item_name'] = trim( preg_replace('/<[^>]+>/',' ', $item["products_name"]) );
			$itemArr['description'] =  $description;
			$itemArr['unit_amount'] = $this->formatPriceNumber($price);
			$itemArr['currency'] = $this->currency["code"];
			$itemArr['is_available'] = $item["products_status"];
			$itemArr['available_text'] = $this->_getAvailableText($item);
			$itemArr['url_deeplink'] = $deeplink;
			$itemArr['urls_images'] = $images;
			$itemArr['categories'] = $categories;
			$itemArr['category_numbers'] = implode("||", $category_numbers);
			$itemArr['use_stock'] = (STOCK_ALLOW_CHECKOUT == 'true' || STOCK_CHECK != 'true') ? 0 : 1;
			$itemArr['stock_quantity'] = $item['products_quantity'];
			$itemArr['minimum_order_quantity'] = (empty($item['gm_min_order']) || ceil($item['gm_min_order']) <= 0) ? '' : ceil($item['gm_min_order']);
			$itemArr['weight'] = $item["products_weight"]*1000;
			$itemArr['tags'] = trim($item["products_keywords"]);
			$itemArr['tax_percent'] = $tax_rate;
			$itemArr['shipping_costs_per_order'] = 0;
			$itemArr['additional_shipping_costs_per_unit'] = 0;
			$itemArr['ean'] = preg_replace("/\s+/i",'',$item["products_ean"]);
			$itemArr['last_update'] = $item["products_last_modified"];
			$itemArr['age_rating'] = $item["products_fsk18"] == 1 ? '18' : '';
			$itemArr['related_shop_item_numbers'] = $this->_getRelatedShopItems($item["products_id"]);
			$itemArr['basic_price'] = $this->_getProductVPE($item, $price);
			$itemArr['is_highlight'] = $item["products_startpage"];
			$itemArr['highlight_order_index'] = $item["products_startpage_sort"];
			
			switch($item['gm_price_status']) {
				case 2: // "not available for purchase":
					$itemArr['use_stock'] = '1';
					$itemArr['stock_quantity'] = '0';
					$itemArr['active_status'] = 'stock';
					$itemArr['available_text'] = SHOPGATE_PLUGIN_PRICE_STATUS_TEXT_NOT_AVAILABLE;
					$itemArr['basic_price'] = SHOPGATE_PLUGIN_PRICE_STATUS_TEXT_NOT_AVAILABLE;
					break;
				case 1: // "price on request"
					if($this->config->getExportPriceOnRequest() == SHOPGATE_SETTING_EXPORT_PRICE_ON_REQUEST_WITHOUT_PRICE) {
						$oldPrice = '';
						$itemArr['unit_amount'] = '0';
						$itemArr['basic_price'] = SHOPGATE_PLUGIN_PRICE_ON_REQUEST_BASIC_PRICE_TEXT;
						$itemArr['old_unit_amount'] = '';
						$itemArr['use_stock'] = '1';
						$itemArr['stock_quantity'] = '0';
						$itemArr['active_status'] = 'active';
						$itemArr['is_active'] = '0';
					} else {
						// Use Product as is and export with price
					}
					break;
			}
			
			// products can be exported as deactivated
			if($deactivateProduct) {
				$itemArr['active_status'] = 'inactive';
			}
			
			if($this->config->getReverseItemsSortOrder()){
				// $addToOrderIndex to make positive sort_order
				$itemArr['sort_order'] = $item["products_sort"] + $addToOrderIndex;
			} else {
				$itemArr['sort_order'] = ($maxOrder - $item["products_sort"])+$addToOrderIndex;
			}
			
			if(!empty($orderInfos)){
				$itemArr['internal_order_info'] = $orderInfos;
			}
			
			$itemNumber = 0 ;
			
			if(!empty($oldPrice) && round($oldPrice, 2) > 0) {
				$itemArr['old_unit_amount'] = $oldPrice;
			} else {
				$itemArr['old_unit_amount'] = '';
			}
			
			if($itemArr['available_text'] == 'Unbekannt') {
				$itemArr['is_available'] = 0;
			}
			
			if(!empty($inputFields)){
				$itemArr['has_input_fields'] = "1";
				$itemArr = array_merge($itemArr, $inputFields);
			}
			
			if(!empty($variations)) {
				if($variations["has_options"]) {
					$itemArr['has_options']=1;
					
					$itemArr = array_merge($itemArr, $variations);
					
					// calculate quantity staggering and modify product for this
					$itemArr = $this->_setQuantityStaggering($itemArr, $item['gm_graduated_qty']);
					
					if(!empty($itemArr['internal_order_info'])){
						$itemArr['internal_order_info'] = $this->jsonEncode($itemArr['internal_order_info']);
					}
					
					$this->addItemRow($itemArr);
				} else {
					if(isset($variations['has_options'])){
						unset($variations['has_options']);
					}
					$itemArr['has_children']=1;
					$isFirst = true;

					// Same input fields for child-products
					if(!empty($inputFields)){
						$itemArr['has_input_fields'] = "1";
						$itemArr = array_merge($itemArr, $inputFields);
					}

					// $itemArr is changed here to create a child-product, so save the parent to have the initial state of the product (parent)
					$parentItem = $itemArr;
					$basePrice = round($parentItem["unit_amount"], 2);
					$baseOldPrice = round($parentItem["old_unit_amount"], 2);
					foreach($variations as $key => $variation) {
						// Always start at a default item setting
						$itemArr = $parentItem;
						
						$price = 0;
						// Offset amount including tax and exchange rate but without discounts
						$originalOffsetAmount = 0;
						// Variation can include an offset amount where the exchange rate and tax is already included
						if(!empty($variation['offset_amount_with_tax'])) {
							$originalOffsetAmount = $variation['offset_amount_with_tax'];
							
							// Variations also need to be discounted if products discount is set
							// IMPORTANT NOTICE:
							// There is a customer group setting where the merchant can set if variations are also discounted by the specific value or not,
							// but seems to be forgotten here completely so, it is completely ignored for products properties
							// -> the products detail view shows always a discounted attribute, but the cart does not include the discount for the property-combination
//							if(!empty($productDiscount) && round($productDiscount, 2) > 0) {
//								if($customerGroupDiscountAttributes) { // Seems to be buggy in gambio so it is ignored here
//									$variation["offset_amount_with_tax"] = $this->_getDiscountPrice($variation["offset_amount_with_tax"], $productDiscount);
//									$variations[$key]["offset_amount_with_tax"] = $variation["offset_amount_with_tax"];
//								}
//							}
							
							$price = $variation['offset_amount_with_tax'];
						} elseif(!empty($variation["offset_amount"])) {
							if(!empty($this->exchangeRate)) {
								$variation["offset_amount"] *= $this->exchangeRate;
								$variations[$key]["offset_amount"] = $variation["offset_amount"];
							}
							$originalOffsetAmount = $variation["offset_amount"]*(1+($tax_rate/100));
							
							// Variations also need to be discounted if products discount is set
							// IMPORTANT NOTICE (different from above!):
							// There is a customer group setting where the merchant can set if variations are also discounted by the specific value or not,
							// but this setting seems to be ignored in the frontend, so it's ignored here, too.
							if(!empty($productDiscount) && round($productDiscount, 2) > 0) {
//								if($customerGroupDiscountAttributes) { // Seems to be buggy in gambio so it is ignored here
									$variation["offset_amount"] = $this->_getDiscountPrice($variation["offset_amount"], $productDiscount);
									$variations[$key]["offset_amount"] = $variation["offset_amount"];
//								}
							}
							
							$price = $variation["offset_amount"]*(1+($tax_rate/100));
						}

						$hash = "";

						for($i=1; $i < 10 && isset($variation["attribute_$i"]); $i++) {
							$hash .= $variation["attribute_$i"];
							$itemArr["attribute_$i"] = htmlentities($variation["attribute_$i"], ENT_NOQUOTES, $this->config->getEncoding());
						}

						$hash = md5($hash);
						$hash = substr($hash, 0, 5);
						if(empty($variation)) $variation = array("order_info" => array());

						// Set Order Info from parent product
						if(!empty($variation['order_info']) && is_array($variation['order_info'])){
							$variation["order_info"] = array_merge($orderInfos, $variation['order_info']);
						} else {
							$variation["order_info"] = $orderInfos;
						}
						
						$variation["order_info"]["base_item_number"] = $parentItem["item_number"];

						$imagesTemp = explode('||', $parentItem["urls_images"]);
						if(isset($variation["images"])) {
							$itemArr["urls_images"] = implode("||", array_merge($variation["images"], $imagesTemp));
						} else {
							$itemArr["urls_images"] = implode("||", $imagesTemp);
						}

						$itemArr['internal_order_info'] = $variation["order_info"];
						
						$itemArr["item_number"] = $parentItem["item_number"].($isFirst?"":"_".$hash);
						if(!empty($variation["item_number"])){
							$itemArr["item_number_public"] = $variation["item_number"];
						} else {
							$itemArr['item_number_public'] = $item['products_model'];
						}
						
						$itemArr["unit_amount"] = $this->formatPriceNumber($basePrice + $price);
						if(!empty($baseOldPrice) && round($baseOldPrice, 2) > 0) {
							$itemArr["old_unit_amount"] = $this->formatPriceNumber($baseOldPrice + $originalOffsetAmount);
						} else {
							$itemArr["old_unit_amount"] = '';
						}
						
						// Check if there is a weight that completely replacec the product weight
						if(isset($variation["fixed_weight"])) {
							$itemArr["weight"]		= $variation["fixed_weight"] * 1000;
						} else {
							// Get offset weight data for attribute otherwise
							$weight = 0;
							if(isset($variation["offset_weight"])) {
								$weight = $variation["offset_weight"] * 1000;
								$itemArr["weight"]		= $parentItem["weight"] + $weight;
							}
						}
						
						// Stock differs in products_properties_combis
						if(!empty($variation['order_info']['is_property_attribute'])) {
							if(isset($variation['use_stock']) && $variation['use_stock'] == '0') {
								$itemArr["use_stock"]	= 0;
							} else {
								$itemArr["use_stock"]	= $parentItem['use_stock'];
							}
							
							if(!empty($variation['use_parents_stock_quantity'])) {
								$itemArr["stock_quantity"]		= $parentItem["stock_quantity"];
							} else {
								$itemArr["stock_quantity"]		= $variation["stock_quantity"];
							}
						} else {
							if($isFirst == false){
								$itemArr["use_stock"]	= (STOCK_ALLOW_CHECKOUT == 'true' || ATTRIBUTE_STOCK_CHECK != 'true') ? 0 : 1;
							}
							
							// Overwrite stock only if its set up in the configuration
							if(ATTRIBUTE_STOCK_CHECK == 'true' && $isFirst == false){
								if(!empty($item["specials_new_products_price"]) && $item["specials_quantity"] > 0){
									$itemArr["stock_quantity"] = $variation["stock_quantity"] > $item["specials_quantity"] ? $item["specials_quantity"] : $variation["stock_quantity"];
								} else {
									$itemArr["stock_quantity"]		= $variation["stock_quantity"];
								}
							}
						}
						
						// Take shipping status from variation if given
						if(!empty($variation['shipping_status_name'])) {
							$itemArr['available_text'] = $this->_getAvailableText($item, $variation['shipping_status_name']);
						} else {
							$itemArr['available_text'] = $this->_getAvailableText($item, $parentItem['available_text']);
						}
						
						$itemArr['properties'] = $this->_buildProperties($item, $itemArr);
						
						// recalculate basic price
						$itemArr['basic_price'] = $this->_getProductVPE($item, $itemArr['unit_amount'], $variation);
						
						// First element is the parent
						if(!$isFirst) {
							$itemArr['has_children']=0;
							$itemArr["parent_item_number"] = $parentItem["item_number"];
						} else {
							$isFirst = false;
						}
						
						// calculate quantity staggering and modify product for this
						$itemArr = $this->_setQuantityStaggering($itemArr, $item['gm_graduated_qty']);
						
						$itemArr['internal_order_info'] = $this->jsonEncode($itemArr['internal_order_info']);
						
						$this->addItemRow($itemArr);
					}
				}
			} else {
				$itemArr['has_children']=0;
				$itemArr['properties'] = $this->_buildProperties($item, $itemArr);
				
				// calculate quantity staggering and modify product for this
				$itemArr = $this->_setQuantityStaggering($itemArr, $item['gm_graduated_qty']);
				
				if(!empty($itemArr['internal_order_info'])){
					$itemArr['internal_order_info'] = $this->jsonEncode($itemArr['internal_order_info']);
				}
				
				$this->addItemRow($itemArr);
			}
		}
	}



##### GambioGX BOF #####
	/**
	 * Reads all the available Gx-Customizer sets from the database
	 * @return array[][][]
	 */
	protected function getGxCustomizerSets() {
		$gxCustomizerSets = array();
		
		// Not available in all versions
		if(!$this->tableExists('gm_gprint_surfaces_groups')) {
			return array();
		}
		
		$sql = "
			SELECT
				`sg`.`name` AS `surfaces_groups_name`,
				`sg`.`gm_gprint_surfaces_groups_id` AS `surfaces_groups_id`,
				`s`.`gm_gprint_surfaces_id` AS `surfaces_id`,
				`s`.`width` AS `surfaces_width`,
				`s`.`height` AS `surfaces_height`,
				`sd`.`name` AS `surfaces_name`,
				`e`.`gm_gprint_elements_id` AS `elements_id`,
				`e`.`gm_gprint_elements_groups_id` AS `elements_groups_id`,
				`e`.`position_x` AS `elements_position_x`,
				`e`.`position_y` AS `elements_position_y`,
				`e`.`height` AS `elements_position_height`,
				`e`.`width` AS `elements_position_width`,
				`e`.`z_index` AS `elements_position_z_index`,
				`e`.`show_name` AS `elements_show_name`,
				`eg`.`group_type` AS `elements_group_type`,
				`eg`.`group_name` AS `elements_group_name`,
				`ev`.`gm_gprint_elements_values_id` AS `elements_values_id`,
				`ev`.`name` AS `elements_values_name`,
				`ev`.`elements_value` AS `elements_values_value`
			FROM `gm_gprint_surfaces_groups` AS `sg`
				INNER JOIN `gm_gprint_surfaces` AS `s` ON (`s`.`gm_gprint_surfaces_groups_id` = `sg`.`gm_gprint_surfaces_groups_id`)
					INNER JOIN `gm_gprint_surfaces_description` AS `sd` ON (`sd`.`gm_gprint_surfaces_id` = `s`.`gm_gprint_surfaces_id`
						AND `sd`.`languages_id` = '{$this->languageId}')
					INNER JOIN `gm_gprint_elements` AS `e` ON (`e`.`gm_gprint_surfaces_id` = `s`.`gm_gprint_surfaces_id`)
						INNER JOIN `gm_gprint_elements_groups` AS `eg` ON (`eg`.`gm_gprint_elements_groups_id` = `e`.`gm_gprint_elements_groups_id`)
						INNER JOIN `gm_gprint_elements_values` AS `ev` ON (`ev`.`gm_gprint_elements_groups_id` = `e`.`gm_gprint_elements_groups_id`
							AND `ev`.`languages_id` = '{$this->languageId}')
			WHERE 1"
		;
		$sqlResult = xtc_db_query($sql);
		while($row = xtc_db_fetch_array($sqlResult)) {
			// create a 3 dimensional structure:
			// n surface-groups (sets) possible
			if(empty($gxCustomizerSets[$row['surfaces_groups_id']])) {
				$gxCustomizerSets[$row['surfaces_groups_id']] = array();
			}
			// n surfaces per surface-group (set)
			if(empty($gxCustomizerSets[$row['surfaces_groups_id']][$row['surfaces_id']])) {
				$gxCustomizerSets[$row['surfaces_groups_id']][$row['surfaces_id']] = array();
			}
			// n elements per surface
			$gxCustomizerSets[$row['surfaces_groups_id']][$row['surfaces_id']][] = $row;
		}
		
		return $gxCustomizerSets;
	}
##### GambioGX EOF #####

##### GambioGX BOF #####
	/**
	 * Takes all Gx-Customizer-Sets and returns all kinds of possible input fields indexed by the specific customizer set
	 * @param array $gxCustomizerSets[][][]
	 * @return array[customizer_set_id][inputfield_number][<inputfield_dataelement>]
	 */
	protected function getInputFieldsFromGxCustomizerSets($gxCustomizerSets = array()) {
		$inputFields = array();
		foreach($gxCustomizerSets as $gxCustomizerGroupId => $gxCustomizerGroup) {
			$inputFields[$gxCustomizerGroupId] = array();
			
			// start with input field number 0 on every new customizer set
			$inputFieldNumber = 0;
			if(is_array($gxCustomizerGroup)) {
				foreach($gxCustomizerGroup as $gxCustomizerSurfaceId => $gxCustomizerSurface) {
					$tmpLabel = array();
					// Sort all elements screen position (take the top-left pixel from labels and bottom right pixel from input fields)
					// -----------------------------------------------------------------------------------------------------------------
					// Calculate sort order indices
					// -> minimum and maximum x and y values need to be found first
					$minX = null;
					$maxX = null;
					$minY = null;
					$maxY = null;
					foreach($gxCustomizerSurface as $gxCustomizerElement) {
						// skip unsupported elements like file-uploads, images and dropdowns
						if($gxCustomizerElement['elements_group_type'] != 'text'
						&& $gxCustomizerElement['elements_group_type'] != 'text_input'
						&& $gxCustomizerElement['elements_group_type'] != 'textarea') {
							continue;
						}
						
						$pX = $gxCustomizerElement['elements_position_x'];
						$pY = $gxCustomizerElement['elements_position_y'];
						// get bottom right pixel if it's not a label
						if($gxCustomizerElement['elements_group_type'] != 'text') {
							$pX += $gxCustomizerElement['elements_width'];
							$pY += $gxCustomizerElement['elements_height'];
						}
						
						// get min/max values for x/y coordinates
						if($minX === null || $pX < $minX) {
							$minX = $pX;
						}
						if($maxX === null || $pX > $maxX) {
							$maxX = $pX;
						}
						if($minY === null || $pY < $minY) {
							$minY = $pY;
						}
						if($maxY === null || $pY > $maxY) {
							$maxY = $pY;
						}
					}
					
					// calculate lineWidth to be able to convert 2d coordinates to a 1d sort order array
					$lineWidth = $maxX-$minX;
					
					// calculate sort order for each element and add it to the mapper
					$sortOrderMapper = array();
					foreach($gxCustomizerSurface as $elementIndex => $gxCustomizerElement) {
						// skip unsupported elements like file-uploads, images and dropdowns
						if($gxCustomizerElement['elements_group_type'] != 'text'
						&& $gxCustomizerElement['elements_group_type'] != 'text_input'
						&& $gxCustomizerElement['elements_group_type'] != 'textarea') {
							continue;
						}
						
						$pX = $gxCustomizerElement['elements_position_x'];
						$pY = $gxCustomizerElement['elements_position_y'];
						
						// get bottom right pixel if it's not a label
						if($gxCustomizerElement['elements_group_type'] != 'text') {
							$pX += $gxCustomizerElement['elements_width'];
							$pY += $gxCustomizerElement['elements_height'];
						}
						
						// Re-position offsets
						$pX -= $minX;
						$pY -= $minY;
						
						// calculate the sort order index and store in the mapper
						$sortOrderIndex = $pY*$lineWidth + $pX; // Default formula in graphics development to get a 1d index from 2d coordinates
						if(empty($sortOrderMapper[$sortOrderIndex])) {
							$sortOrderMapper[$sortOrderIndex] = array();
						}
						// save into sub-array since it is possible to have multiple elements positioned on the exact same coordinates
						// -> in that case first comes first
						$sortOrderMapper[$sortOrderIndex][] = $elementIndex;
					}
					
					// sort the mapper by sortOrder
					ksort($sortOrderMapper);
					
					// create an array of elements sorted by their local position based on screen coordinates (from top-left to bottom-right linewise)
					// -> it will also be filtered to only contain supported elements
					$gxCustomizerSurfaceReordered = array();
					foreach($sortOrderMapper as $sortOrderMapElements) {
						// should only have one element at most
						foreach($sortOrderMapElements as $elementIndex) {
							$gxCustomizerSurfaceReordered[] = $gxCustomizerSurface[$elementIndex];
						}
					}
					
					// The reordered surface can now be converted to input fields
					$label = '';
					foreach($gxCustomizerSurfaceReordered as $gxCustomizerSurfaceElement) {
						// concatenate all labels until the first input field is located
						if($gxCustomizerSurfaceElement['elements_group_type'] == 'text') {
							if(!empty($label)) {
								$label .= ' ';
							}
							// Read priority: Take value if not empty -> take name else
							$label .=
								!empty($gxCustomizerSurfaceElement['elements_values_value'])
									? $gxCustomizerSurfaceElement['elements_values_value']
									: $gxCustomizerSurfaceElement['elements_values_name'];
						} else {
							$inputFieldNumber++;
							
							// always precede the surfaces description name if not empty
							if(!empty($gxCustomizerSurfaceElement['surfaces_name'])) {
								$textSpacing = '';
								if(!empty($label)) {
									$textSpacing = ' ';
								}
								// use uppercase for the surface name, since it is used as uppercase in the shop-frondend
								$label = strtoupper($gxCustomizerSurfaceElement['surfaces_name']).$textSpacing.$label;
							}
							
							// take at least some name if none could be found, yet
							if(empty($label)) {
								$label = $gxCustomizerSurfaceElement['surfaces_groups_name'];
							}
							
							$infotext = !empty($gxCustomizerSurfaceElement['elements_values_value'])
								? $gxCustomizerSurfaceElement['elements_values_value']
								: '';
							// remove line-feeds from the label and infotext elements
							$labelParts = explode("\n", str_replace("\r", '', $label));
							$label = '';
							foreach($labelParts as $labelPart) {
								$labelPart = trim($labelPart);
								if(strlen($labelPart) > 0) {
									$label .= " {$labelPart}";
								}
							}
							$infotextParts = explode("\n", str_replace("\r", '', $infotext));
							$infotext = '';
							foreach($infotextParts as $infotextPart) {
								$infotextPart = trim($infotextPart);
								if(strlen($infotextPart) > 0) {
									$infotext .= " {$infotextPart}";
								}
							}
							
							// now a label should be available for labeling the input field
							$inputFields[$gxCustomizerGroupId]["input_field_{$inputFieldNumber}_type"]			= 'text';
							$inputFields[$gxCustomizerGroupId]["input_field_{$inputFieldNumber}_number"]		= 'gxcust_el_val_id_#' .
																													$gxCustomizerSurfaceElement['surfaces_groups_id'] . '.' .
																													$gxCustomizerSurfaceElement['surfaces_id'] . '.' .
																													$gxCustomizerSurfaceElement['elements_values_id'];
							$inputFields[$gxCustomizerGroupId]["input_field_{$inputFieldNumber}_label"]			= $label;
							$inputFields[$gxCustomizerGroupId]["input_field_{$inputFieldNumber}_infotext"]		= $infotext;
							$inputFields[$gxCustomizerGroupId]["input_field_{$inputFieldNumber}_required"]		= 0;
							$inputFields[$gxCustomizerGroupId]["input_field_{$inputFieldNumber}_add_amount"]	= 0;
							
							// clear label for the following input field
							$label = '';
						}
					}
				}
			}
			// Set to "inputs available" if any input fields have been found to export
			if(!empty($inputFields[$gxCustomizerGroupId])) {
				$inputFields[$gxCustomizerGroupId]['has_input_fields'] = '1';
			}
		}
		
		return $inputFields;
	}
##### GambioGX EOF #####

	/**
	 * Checks all categories against the given customer group to find any categories that should not be exported for the set um  customer group
	 * @param integer $customerGroupId
	 * @return array
	 */
	private function getCategoriesBlacklist($customerGroupId) {
		$categoriesBlacklist = array();
		if(GROUP_CHECK == "true") {
			// Start at the root category and finish at the leaves
			$categoriesBlacklist = $this->_getBlacklistedSubCategoriyIds(0, $customerGroupId);
		}
		return $categoriesBlacklist;
	}

	/**
	 * returns all sub categories including the given parent as a list
	 * @param int $parentId
	 * @param int $customerGroupId
	 * @param array $excludeCategoryIds
	 * @throws ShopgateLibraryException
	 * @return array
	 */
	private function _getBlacklistedSubCategoriyIds($parentId, $customerGroupId = '', $excludeCategoryIdPath = array()) {
		$subCategoryBlacklistIds = array($parentId);
		$qry =
			"SELECT `categories_id` FROM `".TABLE_CATEGORIES."` WHERE" .
				// use filter by customer group if given (get only the categories that are to be blacklisted)
				(!empty($customerGroupId)
					? " (group_permission_{$customerGroupId} = 0) AND"
					: ''
				) .
				// select by parent id
				(!empty($parentId)
					? " (`parent_id` = '{$parentId}')"
					: " (`parent_id` IS NULL OR `parent_id` = 0 OR `parent_id` = '')"
				) .
				// exclude all ids that have been checked already (to prevent circular traversing)
				((!empty($excludeCategoryIds) && is_array($excludeCategoryIds))
					? (" AND (`categories_id` IN (" . implode(', ', $excludeCategoryIds)."))")
					: ''
				)
			;
		
		$qryResult = xtc_db_query($qry);
		if(!$qryResult) {
			throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_DATABASE_ERROR, 'error on selecting categories', true);
		}
		// add all sub categories
		while($row = xtc_db_fetch_array($qryResult)) {
			$subCategoryBlacklistIds = array_merge($subCategoryBlacklistIds, $this->_getBlacklistedSubCategoriyIds($row['categories_id'], $customerGroupId, array_merge($excludeCategoryIdPath, $subCategoryBlacklistIds)));
		}
		return $subCategoryBlacklistIds;
	}

	private function _getPackeges($product, $tax_rate) {
		$customerStatusId = $this->config->getCustomerPriceGroup();
		if($customerStatusId > 0) return '';

		$qry = "
			SELECT *
			FROM ".TABLE_PERSONAL_OFFERS_BY."$customerStatusId
			WHERE products_id = '".$product["products_id"]."'
			  AND quantity > 1
			ORDER BY quantity
		";

		$specialOffers = array();
		$_specialOffers = xtc_db_query($qry);

		while($specialOffer = xtc_db_fetch_array($_specialOffers)) {
			$specialOffers[] = implode("=>", array(
				"qty" => $specialOffer["quantity"],
				"personal_offer" => round($specialOffer["personal_offer"] * (1+($tax_rate/100)), 2),
			));
		}

		return implode("||", $specialOffers);
	}

	private function _getProductVPE($product, $price, $variation = array()) {
		$vpe = "";
		
		$vpeValue = '';
		if(!empty($variation["products_vpe_value"])) {
			$vpeValue = $variation["products_vpe_value"];
		} elseif(!empty($product["products_vpe_value"])) {
			$vpeValue = $product["products_vpe_value"];
		}
		$vpeName = '';
		if(!empty($variation["products_vpe_name"])) {
			$vpeName = $variation["products_vpe_name"];
		} elseif(!empty($product["products_vpe_name"])) {
			$vpeName = $product["products_vpe_name"];
		}

		if(!empty($vpeValue) && !empty($vpeName) && $vpeValue != 0.0000){

			if(!empty($product["products_vpe_status"]) && $product["products_vpe_status"] == 1){

				$factor = 1;
				switch(strtolower($vpeName)) {
					case "ml":
					case "mg":
						$factor = $vpeValue<250?100:1000;
						break;
				}
				
				$_price = ( $price / $vpeValue ) * $factor;

				$vpe  = $this->currency["symbol_left"];
				
				$vpe .= $this->formatPriceNumber(
					$_price,
					$this->currency["decimal_places"],
					$this->currency["decimal_point"],
					$this->currency["thousands_point"]
				);
				
				$vpe .= " " . trim($this->currency["symbol_right"]);
				$vpe .= ' pro '.(($factor == 1) ? '' : $factor.' ');
				$vpe .= $vpeName;
			}
		}

		return $vpe;
	}

	/**
	 * Generates an available text based on the date available field
	 *
	 * @param array $item
	 * @return string
	 */
	private function _getAvailableText($item = array(), $defaultStatusName = '') {
		if(empty($item) || empty($item['shipping_status_name']) && empty($defaultStatusName)) {
			return '';
		}
		
		if(!empty($defaultStatusName)) {
			$availableText = (string) $defaultStatusName;
		} else {
			$availableText = (string) $item['shipping_status_name'];
		}
		
		// Check if the product is available in the future
		if(!empty($item['products_date_available'])) {
			// Check if the date is in the future
			$availableOnTimestamp = strtotime(substr($item['products_date_available'], 0, 10) . ' 00:00:00'); // Take the date beginning at 00:00:00 o' clock
			// Set the "available on" text only if it is at least one day in the future
			if($availableOnTimestamp-time() > 60*60*24) { // 60sec * 60min * 24h == count seconds in 1 day
				switch(strtolower($this->config->getLanguage())) {
					case 'de':
						$dateAvailableFormatted = date('d.m.Y', $availableOnTimestamp);
						break;
					case 'en':
					default:
						$dateAvailableFormatted = date('m/d/Y', $availableOnTimestamp);
						break;
				}
				$availableText = str_replace('#DATE#', $dateAvailableFormatted, SHOPGATE_PLUGIN_FIELD_AVAILABLE_TEXT_AVAILABLE_ON_DATE);
				//$availableText .= ' - '.str_replace('#SHIPPINGTIME#', (string) $item['shipping_status_name'], SHOPGATE_PLUGIN_FIELD_AVAILABLE_TEXT_SHIPPING_DELAY);
			}
		}
		
		// return a default string as fallback
		return $availableText;
	}

	/**
	 * Function to Parse Options like [TAB:xxxx] in the Description
	 * @param string $descrtiption
	 */
	private function _parseDescription($description) {
		$tabs = array();
		$_tabs = preg_match_all("/\[TAB:[\w\s\d\&\;]*\]/", $description, $tabs);

		foreach($tabs[0] as $replace) {
			$replacement = preg_replace("/(\[TAB:)|\]/", "", $replace);
			$replacement = "<h1>".$replacement."</h1>";

			$description = preg_replace("/".preg_quote($replace)."/", $replacement, $description);
		}

		return $description;
	}

	/**
	 * Exportiere alle Produktbilder
	 *
	 * @param string $product
	 */
	private function _getProductsImages($product) {
		$qry = "
			SELECT *
			FROM ".TABLE_PRODUCTS_IMAGES."
			WHERE products_id = '".$product["products_id"]."'
			ORDER BY image_nr
		";

		$images = array();

		if(!empty($product['products_image'])){
			if(file_exists(DIR_FS_CATALOG.DIR_WS_ORIGINAL_IMAGES.$product['products_image'])){
				$images[] = HTTP_SERVER.DIR_WS_CATALOG.DIR_WS_ORIGINAL_IMAGES.$product['products_image'];
			}elseif(file_exists(DIR_FS_CATALOG.DIR_WS_POPUP_IMAGES.$product['products_image'])){
				$images[] = HTTP_SERVER.DIR_WS_CATALOG.DIR_WS_POPUP_IMAGES.$product['products_image'];
			}
		}

		$query = xtc_db_query($qry);
		while($image = xtc_db_fetch_array($query)) {
			if(file_exists(DIR_FS_CATALOG.DIR_WS_ORIGINAL_IMAGES.$image['image_name'])){
				$images[] = HTTP_SERVER.DIR_WS_CATALOG.DIR_WS_ORIGINAL_IMAGES.$image['image_name'];
			}elseif(file_exists(DIR_FS_CATALOG.DIR_WS_POPUP_IMAGES.$image['image_name'])){
				$images[] = HTTP_SERVER.DIR_WS_CATALOG.DIR_WS_POPUP_IMAGES.$image['image_name'];
			}
		}

		$images = implode("||", $images);

		return $images;
	}

	private function _getProductCategoryNumbers($item, $maxId, $addToOrderIndex) {
		$category_numbers = array();
		
		$catsQry = "
			SELECT DISTINCT
				ptc.categories_id,
				c.products_sorting2
			FROM ".TABLE_PRODUCTS_TO_CATEGORIES." ptc
			INNER JOIN ".TABLE_CATEGORIES." c ON (ptc.categories_id = c.categories_id)
			WHERE ptc.products_id = '".$item["products_id"]."'
				AND c.categories_status = 1
			";
		$catsQuery = xtc_db_query($catsQry);
		
		while($category = xtc_db_fetch_array($catsQuery)) {
			if(empty($category["categories_id"])) {
				continue;
			}
			
			$catNumber = "";
			
			if($category["products_sorting2"] != "ASC"){
				
				if($this->config->getReverseItemsSortOrder()){
					$sort = $maxId - $item["products_sort"];
				} else {
					$sort = $item["products_sort"];
				}
				
			} else {
				
				if($this->config->getReverseItemsSortOrder()){
					$sort = $item["products_sort"];
				} else {
					$sort = $maxId - $item["products_sort"];
				}
			}
			
			if (!empty($sort) || ((string) $sort === '0')) {
				$sort += $addToOrderIndex;
				$catNumber = "=>".$sort;
			}
			$catNumber = $category["categories_id"].$catNumber;
			$category_numbers[] = $catNumber;
		}
		
		return $category_numbers;
	}

	/**
	 *
	 * @param mixed[] $product
	 * @param mixed[] $tax
	 * @return float
	 */
	private function _getPersonalOffersPrice($product, $tax) {
		$customerStatusId = $this->config->getCustomerPriceGroup();
		if(empty($customerStatusId)) return false;

		$qry = "SELECT * FROM ".TABLE_PERSONAL_OFFERS_BY."$customerStatusId
		WHERE products_id = '".$product["products_id"]."'
		AND quantity = 1";
		
		$qry = xtc_db_query($qry);
		if(!$qry) return false;
		
		$specialOffer = xtc_db_fetch_array( $qry );
		
		return floatval($specialOffer["personal_offer"]);
	}

	/**
	 * Takes a price value and a discount percent value and returns the new discounted price
	 * @param float $price
	 * @param float $discountPercent
	 * @return float
	 */
	private function _getDiscountPrice($price, $discountPercent) {
		$discountedPrice = $price * (1-$discountPercent/100);
		return $discountedPrice;
	}

	/**
	 * Takes a product and a stack quantity and modifies the product so it can only be bought in n-packs
	 * @param array $product
	 * @param integer $stackQuantity
	 * @return array
	 */
	private function _setQuantityStaggering($product, $stackQuantity = '') {
		$stackQuantity = (!empty($stackQuantity) ? ceil($stackQuantity) : 1);
		$stackQuantity = intval($stackQuantity, 10);
		if($stackQuantity > 1) {
			// Save stack quantity amount
			$product['internal_order_info']['stack_quantity'] = $stackQuantity;
			
			// Additional text to the name
			$product['item_name'] .= sprintf(" (".SHOPGATE_PLUGIN_ITEM_NAME_ADDITION_STACK_QUANTITY_INFO.")", $stackQuantity);
			
			// Stock quantity must be divided by the stack quantity
			$product['stock_quantity'] = intval(floor($product['stock_quantity']/floatval($stackQuantity)), 10);
			
			// Price must be multiplied by the stack quantity
			$product['unit_amount'] *= $stackQuantity;
			if(!empty($product['old_unit_amount'])) {
				$product['old_unit_amount'] *= $stackQuantity;
			}
			
			// The pack is havier than just one product
			if(!empty($product['weight'])) {
				$product['weight'] *= $stackQuantity;
			}
			
			// Min order quantity reduced by the times the pack has been upsized
			if($product['minimum_order_quantity'] > 0) {
				$product['minimum_order_quantity'] = ceil($product['minimum_order_quantity'] / $stackQuantity);
			}
			
			// Shipping consts per unit is higher, since there is a whole pack of the items to order
			if($product['additional_shipping_costs_per_unit'] > 0) {
				$product['additional_shipping_costs_per_unit'] *= $stackQuantity;
			}
			
			// calculate options prices
			for($i = 1; $i <= 10; $i++) {
				if(!empty($product["option_{$i}_values"])) {
					$productOptions = explode('||', $product["option_{$i}_values"]);
					// Split all value-rows to calculate the new price offset
					foreach($productOptions as $key => $option) {
						// price is set after the '=>' mark
						$optionValues = explode('=>', $option);
						// get proce offset
						$offsetPrice = 0;
						if(!empty($optionValues[1])) {
							$offsetPrice = $optionValues[1];
						}
						// calculate new price offset
						$offsetPrice *= $stackQuantity;
						$optionValues[1] = $offsetPrice;
						
						// update value row
						$productOptions[$key] = implode('=>', $optionValues);
					}
					
					// update options-values
					$product["option_{$i}_values"] = implode('||', $productOptions);
				}
			}
		}
		return $product;
	}
	
	/**
	 * Load all Categories of the product and build its category-path
	 *
	 * The categories are seperated by a =>. The Paths are seperated b< a double-pipe ||
	 *
	 * Example: kategorie_1=>kategorie_2||other_1=>other_2
	 * @param int $productId
	 * @return string
	 */
	private function _getProductPath($productId) {
		$catsQry = "
			SELECT DISTINCT ptc.categories_id
			FROM ".TABLE_PRODUCTS_TO_CATEGORIES." ptc
			INNER JOIN ".TABLE_CATEGORIES." c ON ptc.categories_id = c.categories_id
			WHERE ptc.products_id = '$productId'
			  AND c.categories_status = 1
			ORDER BY products_sorting
		";
		$catsQuery = xtc_db_query($catsQry);

		$categories = "";
		while($category = xtc_db_fetch_array($catsQuery)) {
			$cats = xtc_get_category_path($category["categories_id"]);
			$cats = preg_replace("/\_/", ",", $cats);

			$q = "
				SELECT DISTINCT cd.categories_name
				FROM ".TABLE_CATEGORIES_DESCRIPTION." cd
				WHERE cd.categories_id IN (".$cats.")
					AND cd.language_id = ".$this->languageId."
				ORDER BY find_in_set(cd.categories_id, '$cats')
			";

			$q = xtc_db_query($q);
			
			$cats = "";
			while($cd = xtc_db_fetch_array($q)) {
				if(!empty($cats))$cats.="=>";
				$cats.=$cd["categories_name"];
			}
			if(!empty($categories))$categories.="||";
			$categories.=$cats;
		}

		return $categories;
	}

	/**
	 * Returns a array with all Variations of the Product
	 * @param array $item
	 */
	private function _getVariations($product, $tax_rate) {
		$sg_prod_var_attributes = array();
		$sg_prod_var_properties_combis = array();
		
		// The products most not have both variant types selected at the same time!
		if(!empty($product['attributes_products_id']) && !empty($product['properties_products_id'])) {
			// Don't return any data in this case
			return array();
		}
		
		// get article properties (only for gambio gx 2_0_7 or higher)
		$gxProductsPropertiesSupportEnabled = false;
		if($this->gambioGXVersion['main_version'] > 2
		|| $this->gambioGXVersion['main_version'] == 2 && ($this->gambioGXVersion['sub_version'] > 0 OR $this->gambioGXVersion['revision'] >= 7)) {
			$gxProductsPropertiesSupportEnabled = true;
		}
		
		// according to a blog post on the gambio blog, the attribute-vpe and -ean is supportet since GambioGX 1.0.14
		$gxAttributeVPESupportEnabled = false;
		if($this->gambioGXVersion['main_version'] > 1
		|| $this->gambioGXVersion['main_version'] == 1 && ($this->gambioGXVersion['sub_version'] > 0 OR $this->gambioGXVersion['revision'] >= 14)) {
			$gxAttributeVPESupportEnabled = true;
		}
		
		// Get attributes if selected
		if(($this->config->getVariationType() == SHOPGATE_SETTING_VARIATION_TYPE_ATTRIBUTE
		||  $this->config->getVariationType() == SHOPGATE_SETTING_VARIATION_TYPE_BOTH)
		&& !empty($product['attributes_products_id'])) {
			// get article attributes (options)
			$qry = "
				SELECT
					pa.products_attributes_id,
					po.products_options_id,
					pov.products_options_values_id,
					po.products_options_name,
					pov.products_options_values_name,
					pa.attributes_model,
					pa.options_values_price,
					pa.price_prefix,
					pa.options_values_weight,
					pa.attributes_stock,
					pa.weight_prefix,".
					($gxAttributeVPESupportEnabled
							? ( "\n					pa.gm_vpe_value as products_vpe_value,".
									"\n					vpe.products_vpe_name," )
							: ("")
					)."
					pov.gm_filename
				FROM ".TABLE_PRODUCTS_ATTRIBUTES." pa
				INNER JOIN ".TABLE_PRODUCTS_OPTIONS." po ON (pa.options_id = po.products_options_id AND po.language_id = $this->languageId)
				INNER JOIN ".TABLE_PRODUCTS_OPTIONS_VALUES." pov ON (pa.options_values_id = pov.products_options_values_id AND pov.language_id = $this->languageId)".
				($gxAttributeVPESupportEnabled
						? ("\n				LEFT JOIN ".TABLE_PRODUCTS_VPE." vpe ON (vpe.products_vpe_id = pa.products_vpe_id AND vpe.language_id = $this->languageId)")
						: ("")
				)."
				WHERE pa.products_id = '".$product['products_id']."'
					AND pov.products_options_values_name != 'TEXTFELD'
				ORDER BY po.products_options_id, pa.sortorder ASC
			";
			
			$query = xtc_db_query($qry);
			
			//		$options = array_pad(array(), 5, "");
			$options = array();
			
			$i=-1;
			$old = null;
			while($variation = xtc_db_fetch_array($query)) {
				if($variation["products_options_id"] != $old || is_null($old)){
					$i++;
					$old = $variation["products_options_id"];
				}
				$options[$i][] = $variation;
			}
			
			if(empty($options)) return array();
			
			// Find and rename duplicate option-value names
			foreach($options as $optionIndex => $singleOption) {
				// Check all option-value names for duplicate names
				foreach($singleOption as $key => $optionVariation) {
					if(!empty($optionVariation)) {
						// Compare with following entries
						$indexNumber = 1;
						for($i = $key+1; $i < count($singleOption); $i++) {
							if(trim($singleOption[$i]['products_options_values_name']) == trim($optionVariation['products_options_values_name'])) {
								$indexNumber++;
								$options[$optionIndex][$i]['products_options_values_name'] .= " $indexNumber";
							}
						}
						// Add index 1 to the actual name if duplicate name-entries found
						if($indexNumber > 1) {
							$options[$optionIndex][$key]['products_options_values_name'] .= " 1";
							
							// Refresh the working variable for further operation
							$singleOption = $options[$optionIndex];
						}
					}
				}
			}
			
			$countVariations = 1;
			foreach($options as $option){
				$countVariations *= count($option);
			}
			
			if($countVariations > $this->config->getMaxAttributes()) {
				$this->_buildOptions($sg_prod_var_attributes, $options, $tax_rate);
				$sg_prod_var_attributes["has_options"] = 1;
			} else {
				$this->_buildAttributes($sg_prod_var_attributes, $options);
				$sg_prod_var_attributes["has_options"] = 0;
			}
		}
		
		if(($this->config->getVariationType() == SHOPGATE_SETTING_VARIATION_TYPE_PROPERTY
		||  $this->config->getVariationType() == SHOPGATE_SETTING_VARIATION_TYPE_BOTH)
		&& $gxProductsPropertiesSupportEnabled
		&& !empty($product['properties_products_id'])) {
			$qry = "
				SELECT
					`pd`.`properties_id`,
					`pd`.`properties_name`,
					`ppc`.`products_properties_combis_id`,
					`ppc`.`products_id`,
					`ppc`.`combi_model`,
					`ppc`.`combi_quantity_type`,
					`ppc`.`combi_quantity`,
					`ppc`.`combi_weight`,
					`ppc`.`combi_price_type`,
					`ppc`.`combi_price`,
					`ppc`.`combi_image`,
					`ppc`.`vpe_value` as `products_vpe_value`,
					`vpe`.`products_vpe_name`,
					`pv`.`value_price_type` AS `display_price_type`,
					`pv`.`value_price` AS `display_price`,
					`pvd`.`values_name`,
					`pvd`.`values_image`,
					`ss`.`shipping_status_name`
				FROM `" . TABLE_PRODUCTS_PROPERTIES_COMBIS . "` AS `ppc`
					INNER JOIN `" . TABLE_PRODUCTS_PROPERTIES_COMBIS_VALUES . "` AS `ppcv` ON(`ppc`.`products_properties_combis_id` = `ppcv`.`products_properties_combis_id`)
					INNER JOIN `" . TABLE_PROPERTIES_VALUES . "` AS `pv` ON(`ppcv`.`properties_values_id` = `pv`.`properties_values_id`)
					INNER JOIN `" . TABLE_PROPERTIES_VALUES_DESCRIPTION . "` AS `pvd` ON(`ppcv`.`properties_values_id` = `pvd`.`properties_values_id` AND `pvd`.`language_id` = '$this->languageId')
					INNER JOIN `" . TABLE_PROPERTIES . "` AS `p` ON(`pv`.`properties_id` = `p`.`properties_id`)
					INNER JOIN `" . TABLE_PROPERTIES_DESCRIPTION . "` AS `pd` ON(`pv`.`properties_id` = `pd`.`properties_id` AND `pd`.`language_id` = '$this->languageId')
					LEFT JOIN `" . TABLE_SHIPPING_STATUS . "` AS `ss` ON(`ss`.`shipping_status_id` = `ppc`.`combi_shipping_status_id` AND `ss`.`language_id`='$this->languageId')
					LEFT JOIN `" . TABLE_PRODUCTS_VPE . "` AS `vpe` ON (`vpe`.`products_vpe_id` = `ppc`.`products_vpe_id` AND `vpe`.`language_id` = '$this->languageId')
				WHERE
					`ppc`.`products_id` = '".$product['products_id']."'
				ORDER BY `ppc`.`products_properties_combis_id`, `pd`.`properties_id`
			";
			
			$qResult = xtc_db_query($qry);
			
			$propertyList = array();
			while($propertiyCombo = xtc_db_fetch_array($qResult)) {
				// Group by products combination id
				if(empty($propertyList[$propertiyCombo['products_properties_combis_id']])) {
					$propertyList[$propertiyCombo['products_properties_combis_id']] = array();
				}
				$propertyList[$propertiyCombo['products_properties_combis_id']][] = $propertiyCombo;
			}
			
			if(!empty($propertyList)) {
				// Export as attributes only
				$this->_buildPropertiesAttributes($sg_prod_var_properties_combis, $product, $propertyList);
				$sg_prod_var_properties_combis["has_options"] = 0;
			}
		}
		
		return !empty($sg_prod_var_attributes) ? $sg_prod_var_attributes : $sg_prod_var_properties_combis;
	}

	/**
	 * Build the Productvariations using the products properties
	 *
	 * @param &array $sg_prod_var
	 * @param array $product // contains settings about how the properties are set up for the given product
	 * @param array $propertyList // two-dimensional, grouped by products_properties_combi_id
	 */
	private function _buildPropertiesAttributes(&$sg_prod_var, $product, $propertyList) {
		$sg_prod_var = array(0 => array());
		
		if(empty($product) || empty($propertyList)) {
			return FALSE;
		}
		
		// Move through all combination plus an additional first row as title
		$propertyIndexToKeyMap = array_keys($propertyList);
		for($i = 0; $i <= count($propertyList); $i++) {
			// Get actual property combi (make it start at index 1)
			if($i > 0) {
				$properyCombiRow = $propertyList[$propertyIndexToKeyMap[$i-1]];
			} else {
				// Take first combination row as data dummy, since the needed data is set to all combi lines
				$properyCombiRow = $propertyList[$propertyIndexToKeyMap[$i]];
			}
			
			// Go through all available columns
			foreach($properyCombiRow as $columnIndex => $combiColumn) {
				$attributeColumnIndex = $columnIndex+1;
				
				// Create title line of not done yet
				if($i == 0) {
					$sg_prod_var[0]["attribute_$attributeColumnIndex"] = $combiColumn['properties_name'];
				} else {
					$sg_prod_var[$i]["attribute_$attributeColumnIndex"] = $combiColumn['values_name'];
					
					// Include price to name if activated for the product
					if($product['properties_show_price'] == 'true') { // "true" or "false"/empty as string
						// Get price offset display prefix
						switch($combiColumn['display_price_type']) {
							case 'minus':
								$priceDisplayPrefix = '-';
								break;
							case 'plus':
							case 'fix':
							default:
								$priceDisplayPrefix = '+';
						}
						// price offset display prefix type seems not to be used yet!
						$priceDisplayPrefix = '+'; // reset to 1 to have no effect on this, yet
						
						// Get price offset
						$priceDisplayOffset = $combiColumn['display_price'];
						if(empty($priceDisplayOffset)) {
							$priceDisplayOffset = 0;
						}
						
						// Exchange rate must be used, tax rate is is ignored for the display
						$priceDisplayOffset = $this->formatPriceNumber(
							$priceDisplayOffset*$this->exchangeRate,
							$this->currency["decimal_places"],
							$this->currency["decimal_point"],
							$this->currency["thousands_point"]
						);
						
						// Price is displayed exactly as set in the database
						$sg_prod_var[$i]["attribute_$attributeColumnIndex"] .= " ($priceDisplayPrefix {$this->currency['symbol_left']}{$priceDisplayOffset} {$this->currency['symbol_right']})";
					}
				}
			}
			
			// Check for row (first row is the title)
			if($i == 0) {
				continue;
			}
			
			// All needed data is present in every each column, so just take the first
			$propertyCombinationData = $properyCombiRow[0];
			
			// Mark as "property attribute" to let the addOrder process know that it should search for properties instead of product options
			$sg_prod_var[$i]['order_info']['is_property_attribute'] = '1';
			$sg_prod_var[$i]['order_info']['products_properties_combis_id'] = $propertyCombinationData['products_properties_combis_id'];
			
			// Use the model as item_number (public)
			$sg_prod_var[$i]['item_number'] = $propertyCombinationData['combi_model'];
			
			// Get price offset prefix
			switch($propertyCombinationData['combi_price_type']) {
				case 'minus':
					$pricePrefix = -1;
					break;
				case 'plus':
				case 'fix':
				default:
					$pricePrefix = 1;
			}
			// price prefix type seems not to be used yet!
			$pricePrefix = 1; // reset to 1 to have no effect on this, yet
			
			// Get price offset
			$priceOffset = $propertyCombinationData['combi_price'];
			if(empty($priceOffset)) {
				$priceOffset = 0;
			}
			
			// Include prefix to price offset
			$priceOffset *= $pricePrefix;
			
			// IMPORTANT NOTICE: EXCHANGE RATE IS NOT USED FOR PRODUCT PROPERTY COMBINATIONS; SEEMS TO BE A BUG!
			// Calculate to the actual currency
				//$priceOffset *= $this->exchangeRate;
			
			// SECOND IMPORTANT NOTICE: BY DEFAULT THE TAX RATE IS NEEDED HERE TO REMOVE TAXES IF SET FOR THE USER GROUP; THIS IS ALSO NOT WORKING (BUG)
			// SINCE THE SETTING FROM THE CUSTOMER GROUP IS IGNORED IN GAMBIO CHECKOUT
				// if($priceExcludingTaxSettingIsSetInCustomerGroup) {
				// 	$priceOffset /= 1+$this->taxRate/100;
				// }
			
			// Use the calculated price offset
			$sg_prod_var[$i]['offset_amount'] = 0;
			$sg_prod_var[$i]['offset_amount_with_tax'] = $priceOffset;
			
			// Calculate offset weight
			switch($product['use_properties_combis_weight']) {
				case '1': // replace weight
					$sg_prod_var[$i]['offset_weight'] = '0';
					$sg_prod_var[$i]['fixed_weight'] = $propertyCombinationData['combi_weight'];
					break;
				case '0': // add/subtract weight
				default: // same as '0'
					$sg_prod_var[$i]['offset_weight'] = $propertyCombinationData['combi_weight'];
					break;
			}
			
			// Get image file if there is one
			if(!empty($propertyCombinationData['combi_image'])) {
				// Save as array even if it's only one image, so it can be merged to an existing image array
				$sg_prod_var[$i]['images'] = array(HTTP_SERVER.DIR_WS_CATALOG.DIR_WS_IMAGES.'/product_images/properties_combis_images/'.$propertyCombinationData['combi_image']);
			}
			
			// Calculate quantity
			switch($product['use_properties_combis_quantity']) {
				case '1': // use parents quantity
					$sg_prod_var[$i]['use_stock'] = 1;
					$sg_prod_var[$i]['use_parents_stock_quantity'] = 1;
					break;
				case '3': // no check
					$sg_prod_var[$i]['use_stock'] = 0;
					$sg_prod_var[$i]['use_parents_stock_quantity'] = 0;
					break;
				case '0': // default value (use combination quantity)
				case '2': // use combination quantity
				default: // do same as '0'
					$sg_prod_var[$i]['use_stock'] = 1;
					$sg_prod_var[$i]['use_parents_stock_quantity'] = 0;
					break;
			}
			if(!empty($propertyCombinationData['combi_quantity'])) {
				$sg_prod_var[$i]['stock_quantity'] = $propertyCombinationData['combi_quantity'];
			} else {
				$sg_prod_var[$i]['stock_quantity'] = 0;
			}
			
			// Get shipping time (if set for product and a shipping time given for the combination)
			if(!empty($propertyCombinationData['shipping_status_name']) && !empty($product['use_properties_combis_shipping_time'])) {
				$sg_prod_var[$i]['shipping_status_name'] = $propertyCombinationData['shipping_status_name'];
			}
			
			// Get VPE value and name
			if(!empty($propertyCombinationData['products_vpe_value']) && !empty($propertyCombinationData['products_vpe_name'])) {
				$sg_prod_var[$i]['products_vpe_value'] = $propertyCombinationData['products_vpe_value'];
				$sg_prod_var[$i]['products_vpe_name'] = $propertyCombinationData['products_vpe_name'];
			}
		}
	}
	
	/**
	 * Build the Productvariations as options
	 *
	 * @param &array $sg_prod_var
	 * @param array $variations
	 * @param float $tax_rate
	 */
	private function _buildOptions(&$sg_prod_var, $variations, $tax_rate) {
		$tmp=array();
		$i = 0;
		foreach($variations as $_variation) {
			$i++;
			$tmp["option_$i"] = $_variation[0]["products_options_id"] . '=' . strip_tags($_variation[0]["products_options_name"]);

			$options = array();
			foreach($_variation as $option) {
				// Currency and tax must be included here because the data is directly used for the item
				$optionOffsetPrice = $option["options_values_price"]*$this->exchangeRate*(1+($tax_rate/100)); // Include Tax
				$optionOffsetPrice = round($optionOffsetPrice * 100, 0); // get euro-cent

				$field  = strip_tags($option["products_options_values_id"])."=".strip_tags($option["products_options_values_name"]);
				$field .= ($option["options_values_price"] != 0)
					? "=>".$option["price_prefix"].$optionOffsetPrice
					: "";

				$options[] = $field;
			}
			$tmp["option_".$i."_values"] = implode("||", $options);
		}

		$sg_prod_var = $tmp;
	}

	/**
	 * Build the Productvariations recursively
	 *
	 * @param &array $sg_prod_var
	 * @param array $variations
	 * @param int $index
	 * @param array $tmp
	 */
	private function _buildAttributes(&$sg_prod_var, $variations, $index = 0, $baseVar = array()) {
		if($index == 0) {
			// Index 0 sind die Überschriften. Diese müssen als erstes hinzugefügt werden
			for($i = 0; $i < count($variations); $i++){
				$sg_prod_var[0]['attribute_'.($i+1)] = $variations[$i][0]['products_options_name'];
			}
		}
		
		foreach($variations[$index] as $variation) {
			$tmpNewVariation = array();
			
			// copy all prvious attributes (inclusive the order info)
			if(!empty($baseVar)) {
				for($i = 1; $i <= 10; $i++) {
					$keyName = 'attribute_'.$i;
					if(array_key_exists($keyName, $baseVar)) {
						$tmpNewVariation[$keyName] = $baseVar[$keyName];
						$tmpNewVariation['order_info'][$keyName] = $baseVar['order_info'][$keyName];
					} else {
						break;
					}
				}
			}
			
			if(count($variations) == 1){
				// only if 1 dimension
				$tmpNewVariation['item_number'] = $variation['attributes_model'];
			}
			
			if(!empty($baseVar['images'])) {
				$tmpNewVariation['images'] = $baseVar['images'];
			} else {
				$tmpNewVariation['images'] = array();
			}
			if(!empty($variation['gm_filename'])){
				$tmpNewVariation['images'][] = HTTP_SERVER.DIR_WS_CATALOG.DIR_WS_IMAGES.'/product_images/attribute_images/'.$variation['gm_filename'];
			}
			
			$tmpNewVariation['products_vpe_value'] = '';
			$tmpNewVariation['products_vpe_name'] = '';
			if(!empty($variation['products_vpe_value']) && !empty($variation['products_vpe_name'])){
				$tmpNewVariation['products_vpe_value'] = $variation['products_vpe_value'];
				$tmpNewVariation['products_vpe_name'] = $variation['products_vpe_name'];
			}
			
			$tmpNewVariation['attribute_'.($index+1)] = $variation['products_options_values_name'];
			$tmpNewVariation['order_info']['attribute_'.($index+1)] = array(
				$variation['products_attributes_id'] => array(
					'options_id' => $variation['products_options_id'],
					'options_values_id' => $variation['products_options_values_id'],
				),
			);
			
			$tmpNewVariation['stock_quantity'] = $variation['attributes_stock'];
			if(isset($baseVar['stock_quantity']) && $baseVar['stock_quantity'] < $variation['attributes_stock']){
				$tmpNewVariation['stock_quantity'] = $baseVar['stock_quantity'];
			}

			// Kalkuliere den Preisunterschied (Steuern und Währung werden noch nicht hier berücksichtigt)
			$price = $variation['options_values_price'];
			if($variation['price_prefix'] == '-'){
				$price = -1 * $price;
			}
			if(empty($baseVar['offset_amount'])) {
				$baseVar['offset_amount'] = 0;
			}
			$tmpNewVariation['offset_amount'] = $baseVar['offset_amount'] + $price;

			// Kalkuliere den Gewichtsunterschied
			$weight = (float) $variation['options_values_weight'];
			if($variation['weight_prefix'] == '-'){
				$weight = -1 * $weight;
			}
			if(empty($baseVar['offset_weight'])) {
				$baseVar['offset_weight'] = 0;
			}
			$tmpNewVariation['offset_weight'] = $baseVar['offset_weight'] + (double) $weight;

			if($index < (count($variations)-1)) {
				// Fahre mit nächstem Attribute fort (mit aktuellem Zwischenattribut als Basis für die Gewicht, Stock und Preisberechnung)
				// Das aktuelle Zwischenattribut enthält das Gesamtgewicht, den Gesamtpreis und den max-Stock, der für weitere Berechnungen notwendig ist
				$this->_buildAttributes($sg_prod_var, $variations, $index+1, $tmpNewVariation);
			} else {
				// Wenn kein Attribut mehr existiert, dieses auf den Stack legen
				$sg_prod_var[] = $tmpNewVariation;
			}
		}
	}
	
	private function _getRelatedShopItems($products_id) {
		$qry = "
			SELECT px.xsell_id
			FROM ".TABLE_PRODUCTS_XSELL." px
			INNER JOIN ".TABLE_PRODUCTS." p ON (px.products_id = p.products_id)
			WHERE p.products_id = '$products_id'
				AND (p.products_date_available < NOW() OR p.products_date_available IS NULL)
			ORDER BY px.sort_order
		";
		
		$xSellIds = array();
		
		$query = xtc_db_query($qry);
		for($i = 0; $i < xtc_db_num_rows($query); $i++) {
		$array = xtc_db_fetch_array($query);
		$xSellIds[] = $array["xsell_id"];
		}
		
		return implode("||", $xSellIds);
	}
	
	private function _buildProperties($product, $itemArr) {
		$properties = array();

		if(!empty($product["products_fsk18"]) && $product["products_fsk18"] == 1)
		$properties[] = preg_replace("/\:/", "", TEXT_FSK18)."=>".TEXT_YES;


		return implode("||", $properties);
	}


	private function _getInputFields($productId){
		$qry = "
			SELECT
				pa.products_attributes_id,
				po.products_options_id,
				pov.products_options_values_id,
				po.products_options_name,
				pov.products_options_values_name,
				pa.attributes_model,
				pa.options_values_price,
				pa.price_prefix,
				pa.options_values_weight,
				pa.attributes_stock,
				pa.weight_prefix
			FROM ".TABLE_PRODUCTS_ATTRIBUTES." pa
			INNER JOIN ".TABLE_PRODUCTS_OPTIONS." po ON pa.options_id = po.products_options_id
			INNER JOIN ".TABLE_PRODUCTS_OPTIONS_VALUES." pov ON (pa.options_values_id = pov.products_options_values_id AND pov.language_id = $this->languageId)
			WHERE pa.products_id = '$productId'
				AND pov.products_options_values_name = 'TEXTFELD'
			ORDER BY po.products_options_id, pa.sortorder
		";
		
		$query = xtc_db_query($qry);
		
		while($inputFields = xtc_db_fetch_array($query)) {
		if($inputFields["products_options_id"] != $old){
		$i++;
			$old = $inputFields["products_options_id"];
		}
		$inputFieldsAll[$i][] = $inputFields;
		}
		
		if(empty($inputFieldsAll)){
			return;
		}
		
		$sg_product_var = $this->_buildInputFields($inputFieldsAll);
		
		return $sg_product_var;
		
	}

	private function _buildInputFields($inputFieldsAll){
		$sg_product_var = array();
		$i = 0;
		foreach($inputFieldsAll as $inputField) {
			$i++;
			
			//			$sg_product_var["has_input_fields"] = 1;
			$sg_product_var["input_field_".$i."_type"] = 'text';
			$sg_product_var["input_field_".$i."_label"] = strip_tags($inputField[0]["products_options_name"]);
			$sg_product_var["input_field_".$i."_add_amount"] = ($inputField["options_values_price"] != 0)
				? "=>".$option["price_prefix"].round($inputField["options_values_price"], 2)
				: "";
			// keine Angabe möglich
			$sg_product_var["input_field_".$i."_infotext"] = '';
			$sg_product_var["input_field_".$i."_required"] = 0;
		}
		
		return $sg_product_var;
	}


	/**
	 * @see ShopgatePlugin::getCustomer()
	 */
	public function getCustomer($user, $pass) {
		// save the UTF-8 version for logging etc.
		$userUtf8 = $user;
		
		// decode the parameters if necessary to make them work with xtc_* functions
		$user = $this->stringFromUtf8($user, $this->config->getEncoding());
		$pass = $this->stringFromUtf8($pass, $this->config->getEncoding());

		// find customer
		$qry = "SELECT"

		// basic user information
		. " customer.customers_id,"
		. " customer.customers_cid,"
		. " status.customers_status_name,"
		. " status.customers_status_id,"
		. " customer.customers_gender,"
		. " customer.customers_firstname,"
		. " customer.customers_lastname,"
		. " date_format(customer.customers_dob,'%Y-%m-%d') as customers_birthday,"
		. " customer.customers_telephone,"
		. " customer.customers_email_address,"

		// additional information for password verification, default address etc.
		. " customer.customers_password,"
		. " customer.customers_default_address_id"

		. " FROM " . TABLE_CUSTOMERS . " AS customer"

		. " INNER JOIN " . TABLE_CUSTOMERS_STATUS . " AS status"
		
		. " ON customer.customers_status = status.customers_status_id"
		. " AND status.language_id = ".$this->languageId

		. " WHERE customers_email_address = '" . xtc_db_input($user) . "';";

		// user exists?
		$customerResult = xtc_db_query($qry);
		if (empty($customerResult)) {
			throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_WRONG_USERNAME_OR_PASSWORD, 'User: '.$userUtf8);
		}

		// password's correct?
		$customerData = xtc_db_fetch_array($customerResult);
		if (!xtc_validate_password($pass, $customerData['customers_password'])) {
			throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_WRONG_USERNAME_OR_PASSWORD, 'User: '.$userUtf8);
		}

		// fetch customers' addresses
		$qry = "SELECT"

		. " address.address_book_id,"
		. " address.entry_gender,"
		. " address.entry_firstname,"
		. " address.entry_lastname,"
		. " address.entry_company,"
		. " address.entry_street_address,"
		. " address.entry_postcode,"
		. " address.entry_city,"
		. " country.countries_iso_code_2,"
		. " zone.zone_code"


		. " FROM " . TABLE_ADDRESS_BOOK . " AS address"

		. " LEFT JOIN " . TABLE_COUNTRIES . " AS country"
		. " ON country.countries_id = address.entry_country_id"

		. " LEFT JOIN " . TABLE_ZONES . " AS zone"
		. " ON address.entry_zone_id = zone.zone_id"
		. " AND country.countries_id = zone.zone_country_id"

		. " WHERE address.customers_id = " . xtc_db_input($customerData['customers_id']) . ";";

		$addressResult = xtc_db_query($qry);
		if (empty($addressResult)) {
			throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_NO_ADDRESSES_FOUND, 'User: '.$userUtf8);
		}

		$addresses = array();
		while ($addressData = xtc_db_fetch_array($addressResult)) {
			try {
				$stateCode = ShopgateXtcMapper::getShopgateStateCode($addressData["countries_iso_code_2"], $addressData["zone_code"]);
			} catch (ShopgateLibraryException $e) {
				// if state code can't be mapped to ISO use xtc3 state code
				$stateCode = $addressData['zone_code'];
			}

			try {
				$address = new ShopgateAddress();
				$address->setId($addressData['address_book_id']);
				$address->setAddressType(ShopgateAddress::BOTH); // xtc3 doesn't make a difference
				$address->setGender($addressData["entry_gender"]);
				$address->setFirstName($addressData["entry_firstname"]);
				$address->setLastName($addressData["entry_lastname"]);
				$address->setCompany($addressData["entry_company"]);
				$address->setStreet1($addressData["entry_street_address"]);
				$address->setZipcode($addressData["entry_postcode"]);
				$address->setCity($addressData["entry_city"]);
				$address->setCountry($addressData["countries_iso_code_2"]);
				$address->setState($stateCode);
			} catch (ShopgateLibraryException $e) {
				// don't abort here
			}

			// put default address in front, append the others
			if ($address->getId() == $customerData['customers_default_address_id']) {
				array_unshift($addresses, $address);
			} else {
				$addresses[] = $address;
			}
		}

		try {
			$customer = new ShopgateCustomer();
			$customer->setCustomerId($customerData["customers_id"]);
			$customer->setCustomerNumber($customerData["customers_cid"]);
			$customer->setCustomerGroup($customerData['customers_status_name']);
			$customer->setCustomerGroupId($customerData['customers_status_id']);
			$customer->setGender($customerData["customers_gender"]);
			$customer->setFirstName($customerData["customers_firstname"]);
			$customer->setLastName($customerData["customers_lastname"]);
			$customer->setBirthday($customerData["customers_birthday"]);
			$customer->setPhone($customerData["customers_telephone"]);
			$customer->setMail($customerData["customers_email_address"]);
			$customer->setAddresses($addresses);

			// utf-8 encode the values recursively
			$customer = $customer->utf8Encode($this->config->getEncoding());
		} catch (ShopgateLibraryException $e) {
			// don't abort here
		}

		return $customer;
	}

	/**
	 * @see ShopgatePluginCore::addOrder()
	 */
	public function addOrder(ShopgateOrder $order) {
		// save UTF-8 payment infos (to build proper json)
		$paymentInfosUtf8 = $order->getPaymentInfos();
		
		$this->log('start add_order()', ShopgateLogger::LOGTYPE_DEBUG);
		
		// data needs to be utf-8 decoded for äöüß and the like to be saved correctly
		$order = $order->utf8Decode($this->config->getEncoding());
		if ($order instanceof ShopgateOrder); // for Eclipse auto-completion

		$this->log('db: duplicate_order', ShopgateLogger::LOGTYPE_DEBUG);
		
		// check that the order is not imported already
		$qry = "
			SELECT
			o.*,
			so.shopgate_order_number
			FROM ".TABLE_ORDERS." o
			INNER JOIN ".TABLE_SHOPGATE_ORDERS." so ON (so.orders_id = o.orders_id)
			WHERE so.shopgate_order_number = '{$order->getOrderNumber()}'
		";
		$result = xtc_db_query( $qry );
		$dbOrder = xtc_db_fetch_array( $result );

		if(!empty($dbOrder)) {
			throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_DUPLICATE_ORDER, 'external_order_number: '. $dbOrder["orders_id"], true);
		}

		// retrieve address information
		$delivery = $order->getDeliveryAddress();
		$invoice = $order->getInvoiceAddress();

		// find customer
		$customerId = $order->getExternalCustomerId();

		$shopCustomer = array();
		if (!empty($customerId)) {
			$this->log('db: customer', ShopgateLogger::LOGTYPE_DEBUG);
			$result = xtc_db_query("SELECT * FROM " . TABLE_CUSTOMERS . " WHERE customers_id = '{$customerId}'");
			$shopCustomer = xtc_db_fetch_array($result);
		}
		if (empty($shopCustomer)) {
			$this->log('create Guest User', ShopgateLogger::LOGTYPE_DEBUG);
			$shopCustomer = $this->_createGuestUser($order);
		}

		$phone_number = $order->getMobile();
		if(empty($phone_number)) {
			$phone_number = $order->getPhone();
		}

		// get customers address
		$qry = "
			SELECT
				*
			FROM `".TABLE_ADDRESS_BOOK."` AS `ab`
			WHERE `ab`.`customers_id` = '{$shopCustomer['customers_id']}'".(!empty($shopCustomer['customers_default_address_id']) ? ("
				AND `ab`.`address_book_id` = '{$shopCustomer['customers_default_address_id']}'") : "")."
		;";
		$qryResult = xtc_db_query($qry);
		$customersAddress = xtc_db_fetch_array($qryResult);
		// get address format
		if(!empty($customersAddress)) {
			$addressFormatCustomer = $this->_getAddressFormatId(null, $customersAddress['entry_country_id']);
		} else {
			$customersAddress = array(
				'entry_gender' => $shopCustomer['customers_gender'],
				'entry_company' => '',
				'entry_firstname' => $shopCustomer['customers_firstname'],
				'entry_lastname' => $shopCustomer['customers_lastname'],
				'entry_street_address' => '',
				'entry_suburb' => '',
				'entry_postcode' => '',
				'entry_city' => '',
				'entry_state' => '',
				'entry_country_id' => '',
				'entry_zone_id' => '',
			);
		}
		$addressFormatDelivery = $this->_getAddressFormatId($delivery->getCountry());
		$addressFormatInvoice = $this->_getAddressFormatId($invoice->getCountry());
		if(empty($addressFormatCustomer)) {
			$addressFormatCustomer = $addressFormatInvoice;
		}
		if(empty($addressFormatCustomer)) {
			$addressFormatCustomer = $addressFormatDelivery;
		}

		$this->log('db: customer_status', ShopgateLogger::LOGTYPE_DEBUG);
		
		$result = xtc_db_query("SELECT * FROM " . TABLE_CUSTOMERS_STATUS . " WHERE language_id = '{$this->languageId}' AND customers_status_id = '{$shopCustomer["customers_status"]}'");
		$customersStatus = xtc_db_fetch_array($result);
		if (empty($customersStatus)) throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_NO_CUSTOMER_GROUP_FOUND, print_r($shopCustomer,true));


		$this->log('before ShopgateMapper', ShopgateLogger::LOGTYPE_DEBUG);
		
		// map state codes (called "zone id" in shopsystem)
		$customersStateCode = $customersAddress['entry_state'];
		$invoiceStateCode  = $invoice->getState();
		$deliveryStateCode = $delivery->getState();

		$this->log('db: countries', ShopgateLogger::LOGTYPE_DEBUG);
		
		$result = xtc_db_query("SELECT * FROM ".TABLE_COUNTRIES." WHERE countries_id = '{$customersAddress['entry_country_id']}'");
		$customersCountry = xtc_db_fetch_array($result);
		$result = xtc_db_query("SELECT * FROM ".TABLE_COUNTRIES." WHERE countries_iso_code_2 = '{$delivery->getCountry()}'");
		$deliveryCountry = xtc_db_fetch_array($result);
		$result = xtc_db_query("SELECT * FROM ".TABLE_COUNTRIES." WHERE countries_iso_code_2 = '{$invoice->getCountry()}'");
		$invoiceCountry = xtc_db_fetch_array($result);
		if(empty($customersCountry)) {
			$customersCountry = $invoiceCountry;
		}
		if(empty($customersCountry)) {
			$customersCountry = $deliveryCountry;
		}

		///////////////////////////////////////////////////////////////////////
		// Save order
		///////////////////////////////////////////////////////////////////////

		$orderData = array();
		$orderData["customers_id"]					= $shopCustomer["customers_id"];
		$orderData["customers_cid"]					= $shopCustomer["customers_cid"];
		$orderData["customers_vat_id"]				= $shopCustomer["customers_vat_id"];
		$orderData["customers_status"]				= $customersStatus["customers_status_id"];
		$orderData["customers_status_name"]			= $customersStatus["customers_status_name"];
		$orderData["customers_status_image"]		= $customersStatus["customers_status_image"];
		$orderData["customers_status_discount"]		= 0;

		$orderData["customers_name"]				= $customersAddress['entry_firstname']." ".$customersAddress['entry_lastname'];
		$orderData["customers_firstname"]			= $customersAddress['entry_firstname'];
		$orderData["customers_lastname"]			= $customersAddress['entry_lastname'];
		$orderData["customers_company"]				= $customersAddress['entry_company'];
		$orderData["customers_street_address"]		= $customersAddress['entry_street_address'];
		$orderData["customers_suburb"]				= $customersAddress['entry_suburb'];
		$orderData["customers_city"]				= $customersAddress['entry_city'];
		$orderData["customers_postcode"]			= $customersAddress['entry_postcode'];
		$orderData["customers_state"]				= $customersStateCode;
		$orderData["customers_country"]				= $customersCountry['countries_name'];
		$orderData["customers_telephone"]			= $shopCustomer['customers_telephone'];
		$orderData["customers_email_address"]		= $shopCustomer['customers_email_address'];
		$orderData["customers_address_format_id"]	= $addressFormatCustomer;

		$orderData["delivery_name"]					= $delivery->getFirstName()." ".$delivery->getLastName();
		$orderData["delivery_firstname"]			= $delivery->getFirstName();
		$orderData["delivery_lastname"]				= $delivery->getLastName();
		$orderData["delivery_company"]				= $delivery->getCompany();
		$orderData["delivery_street_address"]		= $delivery->getStreet1() . (strlen($delivery->getStreet2()) > 0 ? (' ' . $delivery->getStreet2()) : '');
		$orderData["delivery_suburb"]				= "";
		$orderData["delivery_city"]					= $delivery->getCity();
		$orderData["delivery_postcode"]				= $delivery->getZipcode();
		$orderData["delivery_state"]				= $deliveryStateCode;
		$orderData["delivery_country"]				= $deliveryCountry["countries_name"];
		$orderData["delivery_country_iso_code_2"]	= $delivery->getCountry();
		$orderData["delivery_address_format_id"]	= $addressFormatDelivery;

		$orderData["billing_name"]					= $invoice->getFirstName()." ".$invoice->getLastName();
		$orderData["billing_firstname"]				= $invoice->getFirstName();
		$orderData["billing_lastname"]				= $invoice->getLastName();
		$orderData["billing_company"]				= $invoice->getCompany();
		$orderData["billing_street_address"]		= $invoice->getStreet1() . (strlen($invoice->getStreet2()) > 0 ? (' ' . $invoice->getStreet2()) : '');
		$orderData["billing_suburb"]				= "";
		$orderData["billing_city"]					= $invoice->getCity();
		$orderData["billing_postcode"]				= $invoice->getZipcode();
		$orderData["billing_state"]					= $invoiceStateCode;
		$orderData["billing_country"]				= $invoiceCountry["countries_name"];
		$orderData["billing_country_iso_code_2"]	= $invoice->getCountry();
		$orderData["billing_address_format_id"]		= $addressFormatInvoice;

		$orderData["shipping_method"]				= "Pauschal"; // TODO
		$orderData["shipping_class"]				= "flat_flat"; // TODO

		$orderData["cc_type"]						= "";
		$orderData["cc_owner"]						= "";
		$orderData["cc_number"]						= "";
		$orderData["cc_expires"]					= "";
		$orderData["cc_start"]						= "";
		$orderData["cc_issue"]						= "";
		$orderData["cc_cvv"]						= "";
		$orderData["comments"]						= "";

		$orderData["last_modified"]					= date('Y-m-d H:i:s');
		$orderData["date_purchased"]				= $order->getCreatedTime( 'Y-m-d H:i:s' );

		$orderData["currency"]						= $order->getCurrency();
		$orderData["currency_value"]				= $this->exchangeRate;

		$orderData["account_type"]					= "";

		$orderData["payment_method"]				= "shopgate";
		$orderData["payment_class"]					= "shopgate";

		$orderData["customers_ip"]					= "";
		$orderData["language"]						= $this->language;

		$orderData["afterbuy_success"]				= 0;
		$orderData["afterbuy_id"]					= 0;

		$orderData["refferers_id"]					= 0;
		$orderData["conversion_type"]				= "2";

		$orderData["orders_status"]					= $this->config->getOrderStatusOpen();

		$orderData["orders_date_finished"] 			= null;
		
		$orderData["gm_order_send_date"] 			= $orderData["last_modified"];
		$orderData["gm_send_order_status"] 			= '1';

		$this->log('db: save order', ShopgateLogger::LOGTYPE_DEBUG);
		
		// Speichere die Bestellung
		xtc_db_perform(TABLE_ORDERS,$orderData);
		$dbOrderId = xtc_db_insert_id();

		$this->log('db: save', ShopgateLogger::LOGTYPE_DEBUG);
		
		$ordersShopgateOrder = array(
			"orders_id" => $dbOrderId,
			"shopgate_order_number" => $order->getOrderNumber(),
			"is_paid" => $order->getIsPaid(),
			"is_shipping_blocked" => $order->getIsShippingBlocked(),
			"payment_infos" => $this->jsonEncode($paymentInfosUtf8, true),
			"is_sent_to_shopgate" => 0,
			"modified" => "now()",
			"created" => "now()",
		);
		xtc_db_perform(TABLE_SHOPGATE_ORDERS, $ordersShopgateOrder);
		
		$this->log('method: _insertStatusHistory() ', ShopgateLogger::LOGTYPE_DEBUG);
		$this->_insertStatusHistory($order, $dbOrderId, $orderData['orders_status']);
		
		$this->log('method: _setOrderPayment() ', ShopgateLogger::LOGTYPE_DEBUG);
		$this->_setOrderPayment($order, $dbOrderId, $orderData['orders_status']);
		
		$this->log('method: _insertOrderItems() ', ShopgateLogger::LOGTYPE_DEBUG);
		$this->_insertOrderItems($order, $dbOrderId, $orderData['orders_status']);
		
		$this->log('method: _insertOrderTotal() ', ShopgateLogger::LOGTYPE_DEBUG);
		$this->_insertOrderTotal($order, $dbOrderId);
		
		$this->log('db: update order ', ShopgateLogger::LOGTYPE_DEBUG);
		
		// Save status in order
		$orderUpdateData 					= array();
		$orderUpdateData["orders_status"]	= $orderData["orders_status"];
		$orderUpdateData["last_modified"]	= date( 'Y-m-d H:i:s' );
		xtc_db_perform(TABLE_ORDERS,$orderUpdateData, "update", "orders_id = {$dbOrderId}");
		
		$this->log('method: _pushOrderToAfterbuy', ShopgateLogger::LOGTYPE_DEBUG);
		$this->_pushOrderToAfterbuy($dbOrderId, $order);
		$this->log('method: _pushOrderToDreamRobot', ShopgateLogger::LOGTYPE_DEBUG);
		$this->_pushOrderToDreamRobot($dbOrderId, $order);
		
		$this->log('return: end addOrder()', ShopgateLogger::LOGTYPE_DEBUG);
		return array(
			'external_order_id'=>$dbOrderId,
			'external_order_number'=>$dbOrderId
		);
	}

	public function updateOrder(ShopgateOrder $order) {
		// save UTF-8 payment infos (to build proper json)
		$paymentInfosUtf8 = $order->getPaymentInfos();
		
		// data needs to be utf-8 decoded for äöüß and the like to be saved correctly
		$order = $order->utf8Decode($this->config->getEncoding());
		if ($order instanceof ShopgateOrder); // for Eclipse auto-completion
		
		$qry = "
		SELECT
			o.*,
			so.shopgate_order_id,
			so.shopgate_order_number,
			so.is_paid,
			so.is_shipping_blocked,
			so.payment_infos
		FROM ".TABLE_ORDERS." o
		INNER JOIN ".TABLE_SHOPGATE_ORDERS." so ON (so.orders_id = o.orders_id)
		WHERE so.shopgate_order_number = '{$order->getOrderNumber()}'
		";
		$result = xtc_db_query( $qry );
		$dbOrder = xtc_db_fetch_array( $result );
		
		if($dbOrder == false){
			throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_ORDER_NOT_FOUND, "Shopgate order number: '{$order->getOrderNumber()}'.");
		}
		
		$errorOrderStatusIsSent = false;
		$errorOrderStatusAlreadySet = array();
		$statusShoppingsystemOrderIsPaid = $dbOrder['is_paid'];
		$statusShoppingsystemOrderIsShippingBlocked = $dbOrder['is_shipping_blocked'];
		$status = $dbOrder["orders_status"];
		
		// check if shipping is already done, then throw at end of method a OrderStatusIsSent - Exception
		if($status == $this->config->getOrderStatusShipped() && ($statusShoppingsystemOrderIsShippingBlocked || $order->getIsShippingBlocked())){
			$errorOrderStatusIsSent = true;
		}

		if( $order->getUpdatePayment() == 1 ) {
			
			if(!is_null($statusShoppingsystemOrderIsPaid) && $order->getIsPaid() == $statusShoppingsystemOrderIsPaid &&
				!is_null($dbOrder['payment_infos']) && $dbOrder['payment_infos'] == $this->jsonEncode($paymentInfosUtf8)){
				$errorOrderStatusAlreadySet[] = 'payment';
			}
			
			if(!is_null($statusShoppingsystemOrderIsPaid) && $order->getIsPaid() == $statusShoppingsystemOrderIsPaid){
				// do not update is_paid
			} else {
				
				// Save order status
				$orderStatus = array();
				$orderStatus["orders_id"]					= $dbOrder["orders_id"];
				$orderStatus["orders_status_id"]			= $status;
				$orderStatus["date_added"]					= date( 'Y-m-d H:i:s');
				$orderStatus["customer_notified"]			= false;
				
				if($order->getIsPaid()){
					$orderStatus['comments'] = 'Bestellstatus von Shopgate geändert: Zahlung erhalten';
				} else {
					$orderStatus['comments'] = 'Bestellstatus von Shopgate geändert: Zahlung noch nicht erhalten';
				}
				
				$orderStatus['comments'] = $this->stringFromUtf8($orderStatus['comments'], $this->config->getEncoding());
				
				xtc_db_perform(TABLE_ORDERS_STATUS_HISTORY,$orderStatus);
			
				// update the shopgate order status information
				$ordersShopgateOrder = array(
					"is_paid" => (int)$order->getIsPaid(),
					"modified" => "now()",
				);
				xtc_db_perform(TABLE_SHOPGATE_ORDERS, $ordersShopgateOrder, "update", "shopgate_order_id = {$dbOrder['shopgate_order_id']}");
				
				// update var
				$statusShoppingsystemOrderIsPaid = $order->getIsPaid();
				
				// Save status in order
				$orderData 					= array();
				$orderData["orders_status"]	= $status;
				$orderData["last_modified"]	= date( 'Y-m-d H:i:s' );
				xtc_db_perform(TABLE_ORDERS,$orderData, "update", "orders_id = {$dbOrder['orders_id']}");
				
			}
			
			// update paymentinfos
			if(!is_null($dbOrder['payment_infos']) && $dbOrder['payment_infos'] != $this->jsonEncode($paymentInfosUtf8)){
					
				$dbPaymentInfos = $this->jsonDecode($dbOrder['payment_infos'], true);
				$paymentInfos = $order->getPaymentInfos();
				$histories = array();
					
				switch($order->getPaymentMethod()) {
					case ShopgateOrder::SHOPGATE:
					case ShopgateOrder::INVOICE:
					case ShopgateOrder::COD:
						break;
					case ShopgateOrder::PREPAY:
						if(isset($dbPaymentInfos['purpose']) && $paymentInfos['purpose'] != $dbPaymentInfos['purpose']){
							$comments  = $this->stringFromUtf8("Shopgate: Zahlungsinformationen wurden aktualisiert: \n\nDer Kunde wurde angewiesen Ihnen das Geld mit dem Verwendungszweck \"", $this->config->getEncoding());
							$comments .= $paymentInfos["purpose"];
							$comments .= $this->stringFromUtf8("\" auf Ihr Bankkonto zu überweisen", $this->config->getEncoding());
							
							// Order is not paid yet
							$histories[] = 	array(
								"orders_id"=> $dbOrder["orders_id"],
								"orders_status_id"=>$status,
								"date_added"=>date( 'Y-m-d H:i:s'),
								"customer_notified"=>false,
								"comments"=>xtc_db_prepare_input($comments)
							);
						}
							
						break;
					case ShopgateOrder::DEBIT:
						$qry = "
							SELECT
								*
							FROM banktransfer b
							WHERE b.orders_id = '{$dbOrder['orders_id']}'";
						$result = xtc_db_query( $qry );
						$dbBanktransfer = xtc_db_fetch_array( $result );

						if(!empty($dbBanktransfer)){
							$banktransferData = array();
							$banktransferData["banktransfer_owner"]		= $paymentInfos["bank_account_holder"];
							$banktransferData["banktransfer_number"]	= $paymentInfos["bank_account_number"];
							$banktransferData["banktransfer_bankname"]	= $paymentInfos["bank_name"];
							$banktransferData["banktransfer_blz"]		= $paymentInfos["bank_code"];
							xtc_db_perform("banktransfer", $banktransferData, "update", "orders_id = {$dbOrder['orders_id']}");
							
							$comments  = $this->stringFromUtf8("Shopgate: Zahlungsinformationen wurden aktualisiert: \n\n", $this->config->getEncoding());
							$comments .= $this->_createPaymentInfos($paymentInfos, $dbOrder['orders_id'], $status, false);
							
							$histories[] = 	array(
								"orders_id"=> $dbOrder["orders_id"],
								"orders_status_id"=>$status,
								"date_added"=>date( 'Y-m-d H:i:s'),
								"customer_notified"=>false,
								"comments"=>xtc_db_prepare_input($comments)
							);
						}
							
						break;
					case ShopgateOrder::PAYPAL:
								
						// Save paymentinfos in history
						$history = $this->_createPaymentInfos($paymentInfos, $dbOrder["orders_id"], $status);
						$history['comments'] = $this->stringFromUtf8("Shopgate: Zahlungsinformationen wurden aktualisiert: \n\n", $this->config->getEncoding()).$history['comments'];
						$histories[] = $history;
						
						$this->_updatePaypalOrder($order, $dbOrder["orders_id"]);
						
						break;
					default:
						// mobile_payment
							
						// Save paymentinfos in history
						$history = $this->_createPaymentInfos($paymentInfos, $dbOrder["orders_id"], $status);
						$history['comments'] = $this->stringFromUtf8("Shopgate: Zahlungsinformationen wurden aktualisiert: \n\n", $this->config->getEncoding()).$history['comments'];
						$histories[] = $history;
						
						break;
				}
								
				foreach($histories as $history){
					xtc_db_perform(TABLE_ORDERS_STATUS_HISTORY,$history);
				}
			}
			
			$ordersShopgateOrder = array(
				"payment_infos" => $this->jsonEncode($paymentInfosUtf8),
				"modified" => "now()",
			);
			xtc_db_perform(TABLE_SHOPGATE_ORDERS, $ordersShopgateOrder, "update", "shopgate_order_id = {$dbOrder['shopgate_order_id']}");
			
		}
		
		if($order->getUpdateShipping() == 1){
			if(!is_null($statusShoppingsystemOrderIsShippingBlocked) && $order->getIsShippingBlocked() == $statusShoppingsystemOrderIsShippingBlocked){
				$errorOrderStatusAlreadySet[] = 'shipping';
			} else {
				if($status != $this->config->getOrderStatusShipped()){
					if($order->getIsShippingBlocked() == 1){
						$status = $this->config->getOrderStatusShippingBlocked();
					} else {
						$status = $this->config->getOrderStatusOpen();
					}
				}
				
				$orderStatus = array();
				$orderStatus["orders_id"]					= $dbOrder["orders_id"];
				$orderStatus["date_added"]					= date( 'Y-m-d H:i:s');
				$orderStatus["customer_notified"]			= false;
				$orderStatus['orders_status_id'] 			= $status;
				
				if($order->getIsShippingBlocked() == 0){
					$orderStatus["comments"] = "Bestellstatus von Shopgate geändert: Versand ist nicht mehr blockiert!";
				} else {
					$orderStatus['comments'] = 'Bestellstatus von Shopgate geändert: Versand ist blockiert!';
				}

				$orderStatus['comments'] = $this->stringFromUtf8($orderStatus['comments'], $this->config->getEncoding());
				
				xtc_db_perform(TABLE_ORDERS_STATUS_HISTORY,$orderStatus);
					
				$ordersShopgateOrder = array(
					"is_shipping_blocked" => (int)$order->getIsShippingBlocked(),
					"modified" => "now()",
				);
				xtc_db_perform(TABLE_SHOPGATE_ORDERS, $ordersShopgateOrder, "update", "shopgate_order_id = {$dbOrder['shopgate_order_id']}");
				
				$statusShoppingsystemOrderIsShippingBlocked = $order->getIsShippingBlocked();
				
				// Save status in order
				$orderData 					= array();
				$orderData["orders_status"]	= $status;
				$orderData["last_modified"]	= date( 'Y-m-d H:i:s' );
				xtc_db_perform(TABLE_ORDERS,$orderData, "update", "orders_id = {$dbOrder['orders_id']}");
				
				$this->_pushOrderToAfterbuy($dbOrder["orders_id"], $order);
				$this->_pushOrderToDreamRobot($dbOrder["orders_id"], $order);
			}
		}
		
		if($errorOrderStatusIsSent){
			throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_ORDER_STATUS_IS_SENT);
		}
		
		if(!empty($errorOrderStatusAlreadySet)){
			throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_ORDER_ALREADY_UP_TO_DATE, implode(',', $errorOrderStatusAlreadySet), true);
		}

		return array(
			'external_order_id'=>$dbOrder["orders_id"],
			'external_order_number'=>$dbOrder["orders_id"]
		);
	}
	
	private function _insertStatusHistory(ShopgateOrder $order, $dbOrderId, &$currentOrderStatus) {
		///////////////////////////////////////////////////////////////////////
		// Speicher Kommentare zur Bestellung in der Historie
		///////////////////////////////////////////////////////////////////////
	
		$comment = "";
		if($order->getIsTest()){
			$comment .= "#### DIES IST EINE TESTBESTELLUNG ####\n";
		}
		$comment .= "Bestellung durch Shopgate hinzugefügt.";
		$comment .= "\nBestellnummer: ". $order->getOrderNumber();
	
		$paymentTransactionNumber = $order->getPaymentTransactionNumber();
		if(!empty($paymentTransactionNumber)){
			$comment .= "\nPayment-Transaktionsnummer: ". $paymentTransactionNumber."\n";
		}
		
		if($order->getIsShippingBlocked() == 0){
			$comment .= "\nHinweis: Der Versand der Bestellung ist bei Shopgate nicht blockiert!";
		} else {
			$comment .= "\nHinweis: Der Versand der Bestellung ist bei Shopgate blockiert!";
			$currentOrderStatus = $this->config->getOrderStatusShippingBlocked();
		}
		if ($order->getIsCustomerInvoiceBlocked()) {
			$comment .= "\nHinweis: Für diese Bestellung darf keine Rechnung versendet werden!";
		}
		
		$comment = $this->stringFromUtf8($comment, $this->config->getEncoding());
		
		$histories = array(
			array(
				"orders_id"=> $dbOrderId,
				"orders_status_id"=>$currentOrderStatus,
				"date_added"=>date( 'Y-m-d H:i:s'),
				"customer_notified"=>false,
				"comments"=>xtc_db_prepare_input($comment),
			)
		);
		
		foreach($histories as $history){
			xtc_db_perform(TABLE_ORDERS_STATUS_HISTORY,$history);
		}
	}
	
	private function _setOrderPayment(ShopgateOrder $order, $dbOrderId, &$currentOrderStatus) {
		$payment = $order->getPaymentMethod();
		$paymentGroup = $order->getPaymentGroup();
		$paymentInfos = $order->getPaymentInfos();

		$orderData = array();
		$defaultPayment = 'mobile_payment';

		$histories = array();
		switch($payment) {
			case ShopgateOrder::SHOPGATE:
				$orderData["payment_method"] = "shopgate";
				$orderData["payment_class"] = "shopgate";
				
				break;
			case ShopgateOrder::PREPAY:
				$orderData["payment_method"] = "eustandardtransfer";
				$orderData["payment_class"] = "eustandardtransfer";

				if(!$order->getIsPaid()){
					$comments  = $this->stringFromUtf8("Der Kunde wurde angewiesen Ihnen das Geld mit dem Verwendungszweck \"", $this->config->getEncoding());
					$comments .= $paymentInfos['purpose'];
					$comments .= $this->stringFromUtf8("\" auf Ihr Bankkonto zu überweisen", $this->config->getEncoding());

					// Order is not paid yet
					$histories[] = 	array(
						"orders_id"=> $dbOrderId,
						"orders_status_id"=>$currentOrderStatus,
						"date_added"=>date( 'Y-m-d H:i:s'),
						"customer_notified"=>false,
						"comments"=>xtc_db_prepare_input($comments)
					);
				}

				break;
			case ShopgateOrder::INVOICE:
				$orderData["payment_method"] = "invoice";
				$orderData["payment_class"] = "invoice";
				
				break;
			case ShopgateOrder::COD:
				$orderData["payment_method"] = "cod";
				$orderData["payment_class"] = "cod";
				
				break;
			case ShopgateOrder::DEBIT:
				$orderData["payment_method"] = "banktransfer";
				$orderData["payment_class"] = "banktransfer";

				$banktransferData = array();
				$banktransferData["orders_id"]				= $dbOrderId;
				$banktransferData["banktransfer_owner"]		= $paymentInfos["bank_account_holder"];
				$banktransferData["banktransfer_number"]	= $paymentInfos["bank_account_number"];
				$banktransferData["banktransfer_bankname"]	= $paymentInfos["bank_name"];
				$banktransferData["banktransfer_blz"]		= $paymentInfos["bank_code"];
				$banktransferData["banktransfer_status"]	= "0";
				$banktransferData["banktransfer_prz"]		= $dbOrderId;
				$banktransferData["banktransfer_fax"]		= null;
				xtc_db_perform("banktransfer", $banktransferData);

				$comments  = $this->stringFromUtf8("Sie müssen nun den Geldbetrag per Lastschrift von dem Bankkonto des Kunden abbuchen: \n\n", $this->config->getEncoding());
				$comments .= $this->_createPaymentInfos($paymentInfos, $dbOrderId, $currentOrderStatus, false);
				
				$histories[] = 	array(
						"orders_id"=> $dbOrderId,
						"orders_status_id"=>$currentOrderStatus,
						"date_added"=>date( 'Y-m-d H:i:s'),
						"customer_notified"=>false,
						"comments"=>xtc_db_prepare_input($comments)
				);

				break;
			case ShopgateOrder::PAYPAL:
				$orderData["payment_method"] = "paypal";
				$orderData["payment_class"] = "paypal";
				
				$this->_updatePaypalOrder($order, $dbOrderId);
				
				// Save paymentinfos in history
				$histories[] = $this->_createPaymentInfos($paymentInfos, $dbOrderId, $currentOrderStatus);
				
				break;
			default:
				$orderData["payment_method"] = "mobile_payment";
				$orderData["payment_class"] = "shopgate";
				
				// Save paymentinfos in history
				$histories[] = $this->_createPaymentInfos($paymentInfos, $dbOrderId, $currentOrderStatus);
				
				break;
		}

		foreach($histories as $history){
			xtc_db_perform(TABLE_ORDERS_STATUS_HISTORY, $history);
		}
				
		xtc_db_perform(TABLE_ORDERS, $orderData, "update", "orders_id = {$dbOrderId}");
	}

	/**
	 * Parse the paymentInfo - array and get as output a array or a string
	 *
	 * @param Array $paymentInfos
	 * @param Integer $dbOrderId
	 * @param Integer $currentOrderStatus
	 *
	 * @return mixed History-Array or String
	 */
	private function _createPaymentInfos($paymentInfos, $dbOrderId, $currentOrderStatus, $asArray = true){
		$paymentInformation = '';
		foreach($paymentInfos as $key => $value){
			$paymentInformation .= $key.': '.$value."\n";
		}
		
		if($asArray){
			return array(
				"orders_id"=> $dbOrderId,
				"orders_status_id"=>$currentOrderStatus,
				"date_added"=>date('Y-m-d H:i:s'),
				"customer_notified"=>false,
				"comments"=>xtc_db_prepare_input($paymentInformation)
			);
		} else {
			return $paymentInformation;
		}
	}
	
	/**
	 * Check if the column exists in the specified table
	 *
	 * @param string $columnName
	 * @param string $table
	 */
	private function checkColumn($columnName, $table = TABLE_ORDERS_SHOPGATE_ORDER) {
		$result = xtc_db_query("show columns from `{$table}`");
	
		$exists = false;
		while ($field = xtc_db_fetch_array($result)) {
			if ($field['Field'] == $columnName) {
				$exists = true;
				break;
			}
		}
	
		return $exists;
	}
	
	private function _updatePaypalOrder(ShopgateOrder $order, $dbOrderId){
		$address = $order->getDeliveryAddress();
		
		$paymentInfos = $order->getPaymentInfos();
		
		$paypal = array(
			'xtc_order_id' => $dbOrderId,
			'txn_type' => (!empty($paymentInfos['txn_type']) ? $paymentInfos['txn_type'] : ''),
			'reason_code' => 'None',
			'payment_type' =>  (!empty($paymentInfos['payment_type']) ? $paymentInfos['payment_type'] : ''),
			'payment_status' => (!empty($paymentInfos['payment_status']) ? $paymentInfos['payment_status'] : ''),
			'pending_reason' => 'None',
			'invoice' => (!empty($paymentInfos['invnum']) ? $paymentInfos['invnum'] : null),
			'mc_currency' => (!empty($paymentInfos['mc_currency']) ? $paymentInfos['mc_currency'] : ''),
			'first_name' => (!empty($paymentInfos['first_name']) ? $paymentInfos['first_name'] : ''),
			'last_name' => (!empty($paymentInfos['last_name']) ? $paymentInfos['last_name'] : ''),
			'payer_business_name' => null,
			'address_name' => (!empty($paymentInfos['address_name']) ? $paymentInfos['address_name'] : null),
			'address_street' => (!empty($paymentInfos['address_street']) ? $paymentInfos['address_street'] : null),
			'address_city' => (!empty($paymentInfos['address_city']) ? $paymentInfos['address_city'] : null),
			'address_state' => (!empty($paymentInfos['address_state']) ? $paymentInfos['address_state'] : null),
			'address_zip' => (!empty($paymentInfos['address_zip']) ? $paymentInfos['address_zip'] : null),
			'address_country' => (!empty($paymentInfos['address_country']) ? $paymentInfos['address_country'] : null),
			'address_status' => (!empty($paymentInfos['address_status']) ? $paymentInfos['address_status'] : null),
			'payer_email' => (!empty($paymentInfos['payer_email']) ? $paymentInfos['payer_email'] : ''),
			'payer_id' => (!empty($paymentInfos['payer_id']) ? $paymentInfos['payer_id'] : ''),
			'receiver_email' => (!empty($paymentInfos['receiver_email']) ? $paymentInfos['receiver_email'] : ''),
			'receiver_id' => (!empty($paymentInfos['receiver_id']) ? $paymentInfos['receiver_id'] : ''),
			'payer_status' => (!empty($paymentInfos['payer_status']) ? $paymentInfos['payer_status'] : ''),
			'payment_date' => ($order->getPaymentTime() ? $order->getPaymentTime('Y-m-d H:i:s') : ''),
			'business' => '',
			'txn_id' => (!empty($paymentInfos['transaction_id']) ? $paymentInfos['transaction_id'] : ''),
			'num_cart_items' => (!empty($paymentInfos['num_cart_items']) ? $paymentInfos['num_cart_items'] : '0'),
			'mc_gross' => (!empty($paymentInfos['mc_gross']) ? $paymentInfos['mc_gross'] : '0.00'),
			'mc_fee' => (!empty($paymentInfos['mc_fee']) ? $paymentInfos['mc_fee'] : '0.00'),
			'payment_gross' => (!empty($paymentInfos['payment_gross']) ? $paymentInfos['payment_gross'] : null),
			'payment_fee' => (!empty($paymentInfos['payment_fee']) ? $paymentInfos['payment_fee'] : null),
			'settle_currency' => '',
			'exchange_rate' => '0.00',
			'notify_version' => '3.7',
			'verify_sign' => '',
			'memo' => '',
			'mc_authorization' => '0.00',
			'mc_captured' => '0.00',
		);
		
		if($this->checkColumn('mc_shipping', TABLE_PAYPAL)){
			$paypal['mc_shipping'] = (!empty($paymentInfos['mc_shipping']) ? $paymentInfos['mc_shipping'] : '0.00');
		}
		
		// check that the order is not imported already
		$qry = "
		SELECT
		p.paypal_ipn_id
		FROM ".TABLE_PAYPAL." p
		WHERE p.xtc_order_id = '{$dbOrderId}'
		";
		$result = xtc_db_query( $qry );
		$paypalEntry = xtc_db_fetch_array( $result );
		
		if(!empty($paypalEntry)){
			xtc_db_perform(TABLE_PAYPAL, $paypal, "update", "xtc_order_id = '".$dbOrderId."'");
		} else {
			xtc_db_perform(TABLE_PAYPAL, $paypal, "insert");
		}
		
		//$paypalIpnId = xtc_db_insert_id();
	}
	
	private function _insertOrderItems(ShopgateOrder $order, $dbOrderId, &$currentOrderStatus) {
		///////////////////////////////////////////////////////////////////////
		// Speichert die Produkte
		///////////////////////////////////////////////////////////////////////
		$errors = '';
		foreach($order->getItems() as $orderItem) {
			
			$order_infos = $orderItem->getInternalOrderInfo();
			$order_infos = $this->jsonDecode($order_infos, true);
			
			// The product is possibly stacked
			$stackQuantity = !empty($order_infos['stack_quantity']) ? $order_infos['stack_quantity'] : 1;
			
			$item_number = $orderItem->getItemNumber();
			if(isset($order_infos["base_item_number"])){
				$item_number = $order_infos["base_item_number"];
			}
			
			$this->log('db: get product ', ShopgateLogger::LOGTYPE_DEBUG);
			
			$qry = xtc_db_query(
				"SELECT p.*, pd.products_name FROM ".TABLE_PRODUCTS." p"
				. " INNER JOIN ".TABLE_PRODUCTS_DESCRIPTION." pd ON (pd.products_id=p.products_id AND pd.language_id='$this->languageId')"
				. " WHERE"
				. " p.products_id = '" . $item_number ."'"
				. " LIMIT 1");
			
			$dbProduct = xtc_db_fetch_array($qry);
			if(empty($dbProduct) && ($item_number == 'COUPON' || $item_number == 'PAYMENT_FEE')){
				$this->log('product is COUPON or PAYMENTFEE', ShopgateLogger::LOGTYPE_DEBUG);
				
				// workaround for shopgate coupons
				$dbProduct = array();
				$dbProduct['products_id'] = 0;
				$dbProduct['products_model'] = $item_number;
			} else if(empty($dbProduct)){
				$this->log('no product found', ShopgateLogger::LOGTYPE_DEBUG);
				
				$this->log(ShopgateLibraryException::buildLogMessageFor(ShopgateLibraryException::PLUGIN_ORDER_ITEM_NOT_FOUND, 'Shopgate-Order-Number: '. $order->getOrderNumber() .', DB-Order-Id: '. $dbOrderId .'; item (item_number: '.$products_model.'). The item will be skipped.'));
				$errors .= "\nItem (item_number: ".$item_number.") can not be found in your shoppingsystem. Please contact Shopgate. The item will be skipped.";

				$dbProduct['products_id'] = 0;
				$dbProduct['products_model'] = $item_number;
			}
			
			$this->log('db: orders_products', ShopgateLogger::LOGTYPE_DEBUG);
			
			$productData = array(
				"orders_id"				=>	$dbOrderId,
				"products_model"		=>	$dbProduct["products_model"],
				"products_id"			=>	$item_number,
				"products_name"			=>	($stackQuantity > 1) ? $dbProduct['products_name'] : xtc_db_prepare_input($orderItem->getName()), // The name contains additional text on quantity-stacked products
				"products_price"		=>	$orderItem->getUnitAmountWithTax()/floatval($stackQuantity), // calculate the price of a single, non stacked, product
				"products_discount_made"=>	0,
				"final_price"			=>	$orderItem->getQuantity() * ($orderItem->getUnitAmountWithTax()),
				"products_shipping_time"=>	"",
				"products_tax"			=>	$orderItem->getTaxPercent(),
				"products_quantity"		=>	$orderItem->getQuantity()*$stackQuantity,
				"allow_tax"				=>	1,
			);
			
			$qry = xtc_db_perform(TABLE_ORDERS_PRODUCTS, $productData);
			$productsOrderId = xtc_db_insert_id();

			$options = $orderItem->getOptions();
			$userInputs = $orderItem->getInputs();
			
			// Separate all gx-customizer Inputs from the default inputs
			if(!empty($userInputs)) {
				$gxCustomizerInputs = array();
				$nonGxCustomizerInputs = array();
				// do only if the customizer is available (customizer set table exists in that case)
				if(!isset($gxCustomizerEnable)) {
					$gxCustomizerEnable = $this->tableExists('gm_gprint_surfaces_groups');
				}
				foreach($userInputs as $userInput) {
					if($gxCustomizerEnable && strpos(strtolower($userInput->getInputNumber()), 'gxcust_el_val_id_') !== false) {
						$gxCustomizerInputs[] = $userInput;
					} else {
						$nonGxCustomizerInputs = $userInput;
					}
				}
				
				// Import all gx Customizer data for the ordered product
				if(!empty($gxCustomizerInputs)) {
					// load gx customizer sets first to be able to create a structure for the customizer order structure
					// do only once, since it loads all customizer sets in a single call (data is constant after loading and this call is inside a loop)
					if(empty($gxCustomizerSets)) {
						$gxCustomizerSets = $this->getGxCustomizerSets();
					}
					
					// Maximum of one customizer set per ordered product
					$gxCustomizerOrderSet = array();
					// each set can contain multiple surfaces
					$gxCustomizerOrderSurfaces = array();
					//
					$gxCustomizerOrderValuesPerSurface = array();
					
					foreach($gxCustomizerInputs as $userInput) {
						// left and right part are separated by the hash-sign (#); the right part contains a pattern with the id's
						$splitInputNumber = explode('#', $userInput->getInputNumber());
						// the indexing is done using the following pattern: <customizer_set_id>.<surface_id>.<element_id>
						$customizerIndexSet = explode('.', $splitInputNumber[1]);
						if(!empty($customizerIndexSet[0])) {
							$customizerSetId = $customizerIndexSet[0];
						}
						if(!empty($customizerIndexSet[1])) {
							$customizerSurfaceId = $customizerIndexSet[1];
						}
						if(!empty($customizerIndexSet[2])) {
							$customizerSurfaceElementId = $customizerIndexSet[2];
						}
						
						// Every id must be set and every id must exist inside the current customizer set as well as the customizer set must exist
						if(empty($customizerSetId) || empty($customizerSurfaceId) || empty($customizerSurfaceElementId)
						|| empty($gxCustomizerSets[$customizerSetId]) || empty($gxCustomizerSets[$customizerSetId][$customizerSurfaceId])) {
							// add the user input to an alternate way for expressing user inputs (by using the orders comments)
							$nonGxCustomizerInputs[] = $userInput;
						} else {
							// save each customizer set only once
							if(empty($gxCustomizerOrderSet)) {
								// "surfaces_groups_name" is identical in each array of a surface in a customizer set (array-elements are indexed -> just use the first element to get the set-name)
								$gxCustomizerOrderSet['name'] = $gxCustomizerSets[$customizerSetId][$customizerSurfaceId][0]['surfaces_groups_name'];
								$gxCustomizerOrderSet['orders_products_id'] = $productsOrderId;
								
								// save set to db
								$sqlResult = xtc_db_perform('gm_gprint_orders_surfaces_groups', $gxCustomizerOrderSet);
								$gxCustomizerOrderSet['gm_gprint_orders_surfaces_groups_id'] = xtc_db_insert_id();
							}
							
							// save each surface only once
							if(empty($gxCustomizerOrderSurfaces[$customizerSurfaceId])) {
								$gxCustomizerOrderSurfaces[$customizerSurfaceId] = array(
									// take the first value-element in each surface since the surface data is identical in each surface_values-element
									'gm_gprint_orders_surfaces_groups_id'	=> $gxCustomizerOrderSet['gm_gprint_orders_surfaces_groups_id'],
									'name'									=> $gxCustomizerSets[$customizerSetId][$customizerSurfaceId][0]['surfaces_name'],
									'width'									=> $gxCustomizerSets[$customizerSetId][$customizerSurfaceId][0]['surfaces_width'],
									'height'								=> $gxCustomizerSets[$customizerSetId][$customizerSurfaceId][0]['surfaces_height'],
								);
								
								// save surface directly to db
								$sqlResult = xtc_db_perform('gm_gprint_orders_surfaces', $gxCustomizerOrderSurfaces[$customizerSurfaceId]);
								$gxCustomizerOrderSurfaces[$customizerSurfaceId]['gm_gprint_orders_surfaces_id'] = xtc_db_insert_id();
							}
							
							// buffer all value-elements, grouped by containing surfaces (unsupported elements will be saved with empty value)
							if(empty($gxCustomizerOrderValuesPerSurface[$customizerSurfaceId])) {
								$gxCustomizerOrderValuesPerSurface[$customizerSurfaceId] = array();
							}
							// insert all elements and /or fill with input data
							$processedGroups = array();
							foreach($gxCustomizerSets[$customizerSetId][$customizerSurfaceId] as $key => $gxCustomSurfaceValue) {
								// create each surface-element-value (unsupported elements will include dummy data); TEXT-field elements will include the right data, since the text is predefined
								if(empty($gxCustomizerOrderValuesPerSurface[$customizerSurfaceId][$gxCustomSurfaceValue['elements_values_id']]) && empty($processedGroups[$gxCustomSurfaceValue['elements_groups_id']])) {
									$gxCustomizerOrderValuesPerSurface[$customizerSurfaceId][$gxCustomSurfaceValue['elements_values_id']] = array(
										'gm_gprint_orders_surfaces_id'	=> $gxCustomizerOrderSurfaces[$customizerSurfaceId]['gm_gprint_orders_surfaces_id'],
										'position_x'					=> $gxCustomSurfaceValue['elements_position_x'],
										'position_y'					=> $gxCustomSurfaceValue['elements_position_y'],
										'height'						=> $gxCustomSurfaceValue['elements_position_height'],
										'width'							=> $gxCustomSurfaceValue['elements_position_width'],
										'z_index'						=> $gxCustomSurfaceValue['elements_position_z_index'],
										'show_name'						=> $gxCustomSurfaceValue['elements_show_name'],
										'group_type'					=> $gxCustomSurfaceValue['elements_group_type'],
										'name'							=> $gxCustomSurfaceValue['elements_values_name'],
										'elements_value'				=> '', // leave empty at first
										//'gm_gprint_uploads_id'		=> null, // not to be set; so this field will be a NULL sql-value
									);
									if(strtolower($gxCustomSurfaceValue['elements_group_type']) == 'text') {
										$gxCustomizerOrderValuesPerSurface[$customizerSurfaceId][$gxCustomSurfaceValue['elements_values_id']]['elements_value'] = $gxCustomSurfaceValue['elements_values_value'];
									}
								}
								
								// allow only one "empty" value per product for dropdown groups
								if(strtolower($gxCustomSurfaceValue['elements_group_type']) == 'dropdown') {
									$processedGroups[$gxCustomSurfaceValue['elements_groups_id']] = $gxCustomSurfaceValue['elements_groups_id'];
								}
							}
							
							// Insert the value for the actual input field after all available surface_value-elements are preset
							if(!empty($gxCustomizerOrderValuesPerSurface[$customizerSurfaceId][$customizerSurfaceElementId])) {
								$gxCustomizerOrderValuesPerSurface[$customizerSurfaceId][$customizerSurfaceElementId]['elements_value'] = xtc_db_input($userInput->getUserInput());
							}
						}
					}
					
					// Save all surface-value elements into db if there are any
					if(!empty($gxCustomizerOrderValuesPerSurface)) {
						foreach($gxCustomizerOrderValuesPerSurface as $gxCustSurfaceId => $gxCustomSurfaceElements) {
							foreach($gxCustomSurfaceElements as $gxCustElementId => $gxCustomElement) {
								// save surface directly to db
								$sqlResult = xtc_db_perform('gm_gprint_orders_elements', $gxCustomElement);
								$gxCustomizerOrderValuesPerSurface[$gxCustSurfaceId][$gxCustElementId] = xtc_db_insert_id();
							}
						}
						
						// need to insert a "dummy item variation" to enable the display on the admin page if there are no item variations available
						$attributesAvailable = false;
						if(!empty($options)) {
							$attributesAvailable = true;
						} elseif(empty($order_infos['is_property_attribute']) || $order_infos['is_property_attribute'] == 0) {
							// checks attributes only if it is not a "propperty attribute", because attributes and properties can't be used at the same time for a sigle product
							for($i = 1; $i <= 10; $i++) {
								if(!empty($order_infos["attribute_$i"])) {
									$attributesAvailable = true;
									break;
								}
							}
						}
						// insert the dummy attribute if needed
						if(!$attributesAvailable) {
							$productAttributeData = array(
								"orders_id"=>$dbOrderId,
								"orders_products_id"=>$productsOrderId,
								"products_options"=>'',
								"products_options_values"=>'',
								"options_values_price"=>'0.0000',
								"price_prefix"=>'',
							);
							xtc_db_perform(TABLE_ORDERS_PRODUCTS_ATTRIBUTES, $productAttributeData);
						}
					}
				}
				
				// TODO: non gx-customizer inputs should be inserted as order comment
				if(!empty($nonGxCustomizerInputs)) {
					// .... see todo comment above
				}
			}
			
			if(!empty($options)) {
				$this->log('process options', ShopgateLogger::LOGTYPE_DEBUG);
				foreach($options as $option) {
					$attribute_model = $option->getValueNumber();
					$attribute_number = $option->getOptionNumber();
					
					$this->log('db: get attributes', ShopgateLogger::LOGTYPE_DEBUG);
					
					// Hole das Attribut aus der Datenbank
					$qry = "
						SELECT
							po.products_options_name,
							pov.products_options_values_name,
							pa.options_values_price,
							pa.price_prefix
						FROM ".TABLE_PRODUCTS_ATTRIBUTES." pa
						INNER JOIN ".TABLE_PRODUCTS_OPTIONS." po ON pa.options_id = po.products_options_id AND po.language_id = $this->languageId
						INNER JOIN ".TABLE_PRODUCTS_OPTIONS_VALUES_TO_PRODUCTS_OPTIONS." povtpo ON povtpo.products_options_id = po.products_options_id
						INNER JOIN ".TABLE_PRODUCTS_OPTIONS_VALUES." pov ON (povtpo.products_options_values_id = pov.products_options_values_id AND pa.options_values_id = pov.products_options_values_id AND pov.language_id = $this->languageId)
						WHERE pa.products_id = '".$dbProduct["products_id"]."'
						" . (!empty($attribute_number)
							? "AND pa.options_id = '{$attribute_number}'
						"
							: "") .
						"AND pa.options_values_id = '{$attribute_model}'
						LIMIT 1
					";
					
					$qry = xtc_db_query($qry);
					$dbattribute = xtc_db_fetch_array($qry);
					if(empty($dbattribute)) continue; //Fehler
					
					$this->log('db: save order product attributes', ShopgateLogger::LOGTYPE_DEBUG);
					
					$productAttributeData = array(
						"orders_id"=>$dbOrderId,
						"orders_products_id"=>$productsOrderId,
						"products_options"=>$dbattribute['products_options_name'],
						"products_options_values"=>$dbattribute["products_options_values_name"],
						"options_values_price"=>$dbattribute["options_values_price"],
						"price_prefix"=>$dbattribute["price_prefix"],
					);
					xtc_db_perform(TABLE_ORDERS_PRODUCTS_ATTRIBUTES, $productAttributeData);
				}
			} else {
				// Variations depend on variation type, that has been exported
				if(!empty($order_infos['is_property_attribute'])) {
					$this->log('properties?', ShopgateLogger::LOGTYPE_DEBUG);
					$attribute_model = $order_infos['products_properties_combis_id'];
					
					// Property Daten aus der DB laden
					$qry = "
						SELECT
							`ppc`.`products_properties_combis_id`,
							`ppc`.`combi_quantity`,
							`pd`.`properties_name`,
							`pvd`.`values_name`,
							`pr`.`use_properties_combis_quantity`
						FROM `" . TABLE_PRODUCTS_PROPERTIES_COMBIS_VALUES . "` AS `ppcv`
							INNER JOIN `" . TABLE_PRODUCTS_PROPERTIES_COMBIS . "` AS `ppc` ON (`ppcv`.`products_properties_combis_id` = `ppc`.`products_properties_combis_id` AND `ppc`.`products_id` = '$item_number')
							INNER JOIN `" . TABLE_PROPERTIES_VALUES . "` AS `pv` ON(`ppcv`.`properties_values_id` = `pv`.`properties_values_id`)
							INNER JOIN `" . TABLE_PROPERTIES_VALUES_DESCRIPTION . "` AS `pvd` ON(`ppcv`.`properties_values_id` = `pvd`.`properties_values_id` AND `pvd`.`language_id` = '$this->languageId')
							INNER JOIN `" . TABLE_PROPERTIES . "` AS `p` ON(`pv`.`properties_id` = `p`.`properties_id`)
							INNER JOIN `" . TABLE_PROPERTIES_DESCRIPTION . "` AS `pd` ON(`pv`.`properties_id` = `pd`.`properties_id`  AND `pd`.`language_id` = '$this->languageId')
							INNER JOIN `" . TABLE_PRODUCTS . "` AS `pr` ON(`pr`.`products_id` = `ppc`.`products_id`)
						WHERE `ppcv`.`products_properties_combis_id` = '$attribute_model'
						ORDER BY `p`.`sort_order`, `pv`.`sort_order`
					;";
					
					$qResult = xtc_db_query($qry);
					if(xtc_db_num_rows($qResult) <= 0) continue; //Fehler
					while($dbProperty = xtc_db_fetch_array($qResult)) {
						$lastCombiQuantity = $dbProperty['combi_quantity'];
						// use_properties_combis_quantity == 1 => Products Quantity!
						switch($dbProperty['use_properties_combis_quantity']) {
							case '0': // default stock setting (considers the STOCK_CHECK constant)
							default:
								// Do not update items stock based on the STOCK_CHECK setting (no update if check is off)!
								if(STOCK_CHECK != 'true') {
									$noUpdateItemsStock = true;
								} else {
									$noUpdateItemsStock = false;
								}
								// Do not update properties stock!
								$noUpdatePropertiesStock = true;
								break;
							case '1': // products stock (ignores the STOCK_CHECK constant)
								// Do update items stock (because of ignoring the STOCK_CHECK constant in this case)!
								// Do not update properties stock!
								$noUpdateItemsStock = false;
								$noUpdatePropertiesStock = true;
								break;
							case '2': // combi stock
								// Do not update items stock!
								// Do update properties stock!
								$noUpdateItemsStock = true;
								$noUpdatePropertiesStock = false;
							case '3': // no check at all
								// Don't update anything!
								$noUpdateItemsStock = true;
								$noUpdatePropertiesStock = true;
						}
						$productPropertyData = array(
							'orders_products_id' => $productsOrderId,
							'products_properties_combis_id' => $dbProperty['products_properties_combis_id'],
							'properties_name' => $dbProperty['properties_name'],
							'values_name' => $dbProperty['values_name'],
							'properties_price_type' => '',
							'properties_price' => '0',
						);
						xtc_db_perform(TABLE_ORDERS_PRODUCTS_PROPERTIES, $productPropertyData);
					}
					
					// Update properties stock (only if set active in stock options)
					if(STOCK_LIMITED == 'true' && !empty($productPropertyData) && empty($noUpdatePropertiesStock)) {
						$updateProductsPropertiesCombisData = array(
							'combi_quantity' => $lastCombiQuantity - $orderItem->getQuantity(),
						);
						xtc_db_perform(TABLE_PRODUCTS_PROPERTIES_COMBIS, $updateProductsPropertiesCombisData, 'update', "products_properties_combis_id = '$productPropertyData[products_properties_combis_id]'");
					}
					
					$this->log('db: save order product property combi', ShopgateLogger::LOGTYPE_DEBUG);
				} else {
					$this->log('attributes?', ShopgateLogger::LOGTYPE_DEBUG);
					
					for($i=1;$i<=10;$i++) {
						if(!isset($order_infos["attribute_$i"])){
							break;
						}
						$tmpAttr = $order_infos["attribute_$i"];
						// Code for support of the old internal_order_info structure
						if(!is_array($tmpAttr)) {
							$attribute_model = $tmpAttr;
						} else {
							// Den ersten und einzigen key nutzen (zur Sicherheit auf den start des Arrays setzen)
							reset($tmpAttr);
							$attribute_number = $tmpAttr[key($tmpAttr)]['options_id'];
							$attribute_model = $tmpAttr[key($tmpAttr)]['options_values_id'];
						}
						
						$this->log('db: get attribute', ShopgateLogger::LOGTYPE_DEBUG);
						
						// Hole das Attribut aus der Datenbank
						$qry = "
							SELECT
								po.products_options_name,
								pov.products_options_values_name,
								pa.options_values_price,
								pa.price_prefix
							FROM ".TABLE_PRODUCTS_ATTRIBUTES." pa
							INNER JOIN ".TABLE_PRODUCTS_OPTIONS." po ON pa.options_id = po.products_options_id AND po.language_id = $this->languageId
							INNER JOIN ".TABLE_PRODUCTS_OPTIONS_VALUES_TO_PRODUCTS_OPTIONS." povtpo ON povtpo.products_options_id = po.products_options_id
							INNER JOIN ".TABLE_PRODUCTS_OPTIONS_VALUES." pov ON (povtpo.products_options_values_id = pov.products_options_values_id AND pa.options_values_id = pov.products_options_values_id AND pov.language_id = $this->languageId)
							WHERE pa.products_id = '".$dbProduct["products_id"]."'
							" .
							// Still support the old internal_order_info structure
							(!empty($attribute_id)
								? "AND pa.products_attributes_id = '".$attribute_model."'
							"
								: "AND pa.options_id = '{$attribute_number}'
							AND pa.options_values_id = '{$attribute_model}'
							") .
							"LIMIT 1
						";
						
						$qry = xtc_db_query($qry);
						$dbattribute = xtc_db_fetch_array($qry);
						if(empty($dbattribute)) continue; //Fehler
						
						
						$this->log('db: save order product attributes', ShopgateLogger::LOGTYPE_DEBUG);
						
						$productAttributeData = array(
							"orders_id"=>$dbOrderId,
							"orders_products_id"=>$productsOrderId,
							"products_options"=>$dbattribute["products_options_name"],
							"products_options_values"=>$dbattribute["products_options_values_name"],
							"options_values_price"=>$dbattribute["options_values_price"],
							"price_prefix"=>$dbattribute["price_prefix"],
						);
						xtc_db_perform(TABLE_ORDERS_PRODUCTS_ATTRIBUTES, $productAttributeData);
					}
				}
			}
		}
		
		$this->log('method: updateItemsStock', ShopgateLogger::LOGTYPE_DEBUG);
		if(empty($noUpdateItemsStock)) {
			$noUpdateItemsStock = false;
		} else {
			// Never update attributes stock when items stock is not updated
			$noUpdateAttributes = true;
		}
		// Specials have to be reduced always
		$noUpdateSpecials = false;
		$this->updateItemsStock($order, $noUpdateItemsStock, $noUpdateAttributes, $noUpdateSpecials);
		
		if(!empty($errors)){
			$this->log('db: save errors in history', ShopgateLogger::LOGTYPE_DEBUG);
			$comments  = $this->stringFromUtf8('Es sind Fehler beim Importieren der Bestellung aufgetreten: ', $this->config->getEncoding());
			$comments .= $errors;

			$history = array(
				"orders_id"=> $dbOrderId,
				"orders_status_id"=>$currentOrderStatus,
				"date_added"=>date("Y-m-d H:i:s", time()-5),// "-5" Damit diese Meldung als erstes oben angezeigt wird
				"customer_notified"=>false,
				"comments"=>xtc_db_prepare_input($comments),
			);

			xtc_db_perform(TABLE_ORDERS_STATUS_HISTORY,$history);
		}
	}
	
	private function _insertOrderTotal(ShopgateOrder $order, $dbOrderId){
		///////////////////////////////////////////////////////////////////////
		// Speicher den Gesamtbetrag
		///////////////////////////////////////////////////////////////////////
		
		$amountWithTax = $order->getAmountComplete();
		$shippingTaxRate = $this->_getOrderShippingTaxRate($order);
		$taxes = $this->_getOrderTaxes($order, $dbOrderId, $shippingTaxRate);
		$xtPrice = new xtcPrice($this->currency["code"], 1);
		$shippingCosts = $order->getAmountShipping();
		
		$this->log('_insertOrderTotal(): add subtotal', ShopgateLogger::LOGTYPE_DEBUG);
		
		$sort = 10;
		
		$ordersTotal = array();
		$ordersTotal["orders_id"]		= $dbOrderId;
		$ordersTotal["title"]			= xtc_db_prepare_input("Zwischensumme:");
		$ordersTotal["text"]			= $xtPrice->xtcFormat($order->getAmountItems(), true);
		$ordersTotal["value"]			= $order->getAmountItems();
		$ordersTotal["class"]			= "ot_subtotal";
		$ordersTotal["sort_order"]		= $sort++;
		xtc_db_perform(TABLE_ORDERS_TOTAL, $ordersTotal);
		
		$this->log('_insertOrderTotal(): add shipping costs total', ShopgateLogger::LOGTYPE_DEBUG);
		
		$ordersTotal = array();
		$ordersTotal["orders_id"]		= $dbOrderId;
		$ordersTotal["title"]			= xtc_db_prepare_input("Versand:");
		$ordersTotal["text"]			= $xtPrice->xtcFormat($shippingCosts, true);
		$ordersTotal["value"]			= $shippingCosts;
		$ordersTotal["class"]			= "ot_shipping";
		$ordersTotal["sort_order"]		= $sort++;
		xtc_db_perform(TABLE_ORDERS_TOTAL, $ordersTotal);
		
		// insert payment costs.
		//
		//WARNING: On modify: Change the taxes calculation too!
		if($order->getAmountShopPayment() != 0){
			$this->log('db: save payment fee', ShopgateLogger::LOGTYPE_DEBUG);
				
			$paymentInfos = $order->getPaymentInfos();
				
			$ordersTotal = array();
			$ordersTotal["orders_id"]		= $dbOrderId;
			$ordersTotal["title"]			= xtc_db_prepare_input('Zahlungsartkosten'. (!empty($paymentInfos['shopgate_payment_name']) ? ' ('.$paymentInfos['shopgate_payment_name'].'):' : ''));
			$ordersTotal["text"]			= $xtPrice->xtcFormat($order->getAmountShopPayment(), true);
			$ordersTotal["value"]			= $order->getAmountShopPayment();
			$ordersTotal["class"]			= "ot_shipping";
			$ordersTotal["sort_order"]		= $sort++;
			xtc_db_perform(TABLE_ORDERS_TOTAL, $ordersTotal);
		
		}
		
		$this->log('_insertOrderTotal(): add tax totals', ShopgateLogger::LOGTYPE_DEBUG);
		
		// Shipping taxes are already stored in $taxes!
		$unitAmountWithoutTax = $amountWithTax;
		foreach($taxes as $percent => $tax_value) {
			$ordersTotal = array();
			$ordersTotal["orders_id"]		= $dbOrderId;
			$ordersTotal["title"]			= "inkl. UST {$percent} %";
			$ordersTotal["text"]			= $xtPrice->xtcFormat($tax_value, true);
			$ordersTotal["value"]			= $tax_value;
			$ordersTotal["class"]			= "ot_tax";
			$ordersTotal["sort_order"]		= $sort++;
			xtc_db_perform(TABLE_ORDERS_TOTAL, $ordersTotal);
			
			$unitAmountWithoutTax -= $tax_value;
		}
		
		$this->log('_insertOrderTotal(): add order total', ShopgateLogger::LOGTYPE_DEBUG);
		
		$ordersTotal = array();
		$ordersTotal["orders_id"]		= $dbOrderId;
		$ordersTotal["title"]			= xtc_db_prepare_input("Summe netto:");
		$ordersTotal["text"]			= $xtPrice->xtcFormat($unitAmountWithoutTax, true);
		$ordersTotal["value"]			= $unitAmountWithoutTax;
		$ordersTotal["class"]			= "ot_total_netto";
		$ordersTotal["sort_order"]		= $sort++;
		xtc_db_perform(TABLE_ORDERS_TOTAL, $ordersTotal);
		
		$ordersTotal = array();
		$ordersTotal["orders_id"]		= $dbOrderId;
		$ordersTotal["title"]			= "<b>Summe:</b>";
		$ordersTotal["text"]			= "<b>".$xtPrice->xtcFormat($amountWithTax, true)."</b>";
		$ordersTotal["value"]			= $amountWithTax;
		$ordersTotal["class"]			= "ot_total";
		$ordersTotal["sort_order"]		= $sort++;
		xtc_db_perform(TABLE_ORDERS_TOTAL, $ordersTotal);

	}
	
	private function _getOrderTaxes(ShopgateOrder $order, $dbOrderId, $shippingTaxRate = 0) {
		$taxes = array();
		
		foreach($order->getItems() as $orderItem) {
			$tax		= $orderItem->getTaxPercent();
			$tax		= (string) (round($tax*100)/100);
			$tax_value	= $orderItem->getUnitAmountWithTax() - $orderItem->getUnitAmount();
			
			if(!isset($taxes[$tax])) $taxes[$tax]= 0;
			$taxes[$tax] += $tax_value * $orderItem->getQuantity();
		}
		
		if(!empty($shippingTaxRate)) {
			$shippingTaxRate = (string) (round($shippingTaxRate*100)/100);
			if(!isset($taxes[$shippingTaxRate])) $taxes[$shippingTaxRate]= 0;
			$taxes[$shippingTaxRate] += $order->getAmountShipping()-$this->_getOrderShippingAmountWithoutTax($order, $shippingTaxRate);
		}
		
		// set taxes for payment method
		if($order->getAmountShopPayment() != 0){
			$tax		= $order->getPaymentTaxPercent();
			$tax		= (string) (round($tax*100)/100);
			$tax_value	= $order->getAmountShopPayment() - round(($order->getAmountShopPayment()*100)/($order->getPaymentTaxPercent()+100),2);
			
			if(!isset($taxes[$tax])) $taxes[$tax]= 0;
			$taxes[$tax] += $tax_value;
		}
		
		return $taxes;
	}
	
	private function _getOrderShippingTaxRate(ShopgateOrder $order) {
		$shippingTaxRate = 0;
	
		// Check if a shipping method is set in config
		$shippingMethod = $this->config->getShipping();
		$orderCountryCode2 = $order->getInvoiceAddress()->getCountry();
		
		if(!empty($shippingMethod)) {
			// Get tax value from shipping module
			$qry =
			"SELECT `c`.`configuration_value`, `tr`.`tax_rate` " .
			"FROM `" . TABLE_CONFIGURATION . "` AS `c` " .
			"INNER JOIN `" . TABLE_TAX_RATES . "` AS `tr` ON(`c`.`configuration_value`=`tr`.`tax_class_id`) " .
			"INNER JOIN `" . TABLE_ZONES_TO_GEO_ZONES . "` AS `geozones` ON(`tr`.`tax_zone_id`=`geozones`.`geo_zone_id`) " .
			"INNER JOIN `" . TABLE_COUNTRIES . "` AS `co` ON(`geozones`.`zone_country_id`=`co`.`countries_id`) " .
			"WHERE " .
			"`c`.`configuration_key` = 'MODULE_SHIPPING_".strtoupper($shippingMethod)."_TAX_CLASS' " .
			"AND " .
			"`co`.`countries_iso_code_2`='$orderCountryCode2';";
			$result = xtc_db_query($qry);
			$moduleTaxSetting = xtc_db_fetch_array($result);
			if(!empty($moduleTaxSetting) && !empty($moduleTaxSetting['configuration_value']) && !empty($moduleTaxSetting['tax_rate'])) {
				// get the tax rate for the shipping costs
				$shippingTaxRate = intval($moduleTaxSetting['tax_rate']*100)/100;
			}
		}
	
		return $shippingTaxRate;
	}
	
	private function _getOrderShippingAmountWithoutTax(ShopgateOrder $order, $shippingTaxRate = 0) {
		$shippingAmountWithoutTax = $order->getAmountShipping();
		
		// Check if a shipping method is set in config
		$shippingMethod = $this->config->getShipping();
		if(!empty($shippingTaxRate)) {
			// remove tax from shipping costs
			$shippingAmountWithoutTax /= 1+$shippingTaxRate/100;
		}
		
		return $shippingAmountWithoutTax;
	}
	
	private function updateItemsStock(ShopgateOrder $order, $ignoreProducts = false, $ignoreAttributes = false, $ignoreSpecials = false) {
		foreach($order->getItems() as $item) {
			// Skip "coupon" and "payment_fee" items
			if($item->getItemNumber() == 'COUPON' || $item->getItemNumber() == 'PAYMENT_FEE'){
				continue;
			}
			
			// Attribute ids are set inside the internal order info
			$internalOrderInfo = $this->jsonDecode($item->getInternalOrderInfo(), true);
			
			$usesProductsAttributes = false;
			
			// Get id (parent id for child products)
			$productId = $item->getItemNumber();
			if(!empty($internalOrderInfo['base_item_number'])) {
				$productId = $internalOrderInfo['base_item_number'];
				$usesProductsAttributes = true;
			}
			
			$itemOptions = $item->getOptions();
			if(!empty($itemOptions)) {
				$usesProductsAttributes = true;
			}
			
			// Update "products_ordered" field that is used to display all bestsellers (always update this)
			$qry = "
				UPDATE `" . TABLE_PRODUCTS . "` AS `p`
					SET `p`.`products_ordered` = `p`.`products_ordered` + ".($item->getQuantity())."
				WHERE `p`.`products_id` = '{$productId}'
			;";
			xtc_db_query($qry);
			
			// Update products stock if reduction enabled
			if(STOCK_LIMITED == 'true' && !$ignoreProducts) {
				// Reduce products stock
				$qry = "
					UPDATE `" . TABLE_PRODUCTS . "` AS `p`
						SET `p`.`products_quantity` = `p`.`products_quantity` - {$item->getQuantity()}
					WHERE `p`.`products_id` = '{$productId}'
				;";
				xtc_db_query($qry);
				
				// TODO: Create notify emails when "STOCK_REORDER_LEVEL" is passed for the product
				// -> The following code shows how the gambio shoppingsystem handles email notifications for products
				// --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
						//if($stock_left <= STOCK_REORDER_LEVEL) {
						//	$gm_get_products_name = xtc_db_query("SELECT products_name
						//																			FROM products_description
						//																			WHERE
						//																				products_id = '" . xtc_get_prid($order->products[$i]['id']) . "'
						//																				AND language_id = '" . $_SESSION['languages_id'] . "'");
						//	$gm_stock_data	= mysql_fetch_array($gm_get_products_name);
						//
						//	$gm_subject		= GM_OUT_OF_STOCK_NOTIFY_TEXT .' '. $gm_stock_data['products_name'];
						//	$gm_body		= GM_OUT_OF_STOCK_NOTIFY_TEXT .': ' . (double)$stock_left . "\n\n".
						//			HTTP_SERVER.DIR_WS_CATALOG . 'product_info.php?info=p' . xtc_get_prid($order->products[$i]['id']);
						//
						//	// send mail
						//	xtc_php_mail(STORE_OWNER_EMAIL_ADDRESS, STORE_NAME, STORE_OWNER_EMAIL_ADDRESS, STORE_NAME, '', STORE_OWNER_EMAIL_ADDRESS, STORE_NAME, '', '', $gm_subject, nl2br(htmlentities($gm_body)), $gm_body);
						//}
				// --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
				
				// Deactivate product if checkout is not allowed and the stock level reaches zero
				if(STOCK_ALLOW_CHECKOUT == 'false') {
					// gambiogx has an additional constant that tells if the product may be deactivated (GM_SET_OUT_OF_STOCK_PRODUCTS)
					if(GM_SET_OUT_OF_STOCK_PRODUCTS == 'true') { // don't update if defined and not true
						$qry = "
							UPDATE `" . TABLE_PRODUCTS . "` AS `p`
								SET `p`.`products_status` = '0'
							WHERE `p`.`products_id` = '{$productId}' AND `p`.`products_quantity` <= 0
						;";
						xtc_db_query($qry);
					}
				}
			}
			
			// Attribute items also need to be reduced in stock
			if($usesProductsAttributes && !$ignoreAttributes) {
				// Build additional SQL snippets to update the attributes stock (not using the products_attributes_id because they all change on each update of any attribute in the backend of the shoppingsystem)
				$attributeSQLQueryParts = array();
				if(!empty($internalOrderInfo['base_item_number'])) {
					for($i = 1; $i <= 10; $i++) {
						if(!empty($internalOrderInfo["attribute_{$i}"])) {
							$tmpAttr = $internalOrderInfo["attribute_{$i}"];
							if(!is_array($tmpAttr)) {
								$attributeSQLQueryParts[] = " ATTRIBUTES_ID='{$tmpAttr}'";
							} else {
								// Only the first element is relevant since there can only be one per attribute-number
								reset($tmpAttr);
								$attributeSQLQueryParts[] = 'OPTIONS_ID=\''.$tmpAttr[key($tmpAttr)]['options_id'] . '\' AND OPTIONS_VALUES_ID=\''.$tmpAttr[key($tmpAttr)]['options_values_id'].'\'';
							}
						}
					}
				} else {
					// Attributes was exported as options
					foreach($itemOptions as $itemOption) {
						$attributeSQLQueryParts[] = 'OPTIONS_ID=\''.$itemOption->getOptionNumber() . '\' AND OPTIONS_VALUES_ID=\''.$itemOption->getValueNumber().'\'';
					}
				}
				if(!empty($attributeSQLQueryParts)) {
					// Attribute stock is ALWAYS reduced (no matter what is set as STOCK_LIMITED or the other constants)!
					$attributeSQLConditionSnippet = '(' . str_replace(array('OPTIONS_ID', 'OPTIONS_VALUES_ID', 'ATTRIBUTES_ID'), array('`pa`.`options_id`', '`pa`.`options_values_id`', '`pa`.`products_attributes_id`'), implode(') OR (', $attributeSQLQueryParts)) . ')';
					
					// Update attributes stock
					$qry = "
						UPDATE `" . TABLE_PRODUCTS_ATTRIBUTES . "` AS `pa`
							SET `pa`.`attributes_stock` = `pa`.`attributes_stock` - {$item->getQuantity()}
						WHERE `pa`.`products_id` = '{$productId}'
							AND ({$attributeSQLConditionSnippet})
					;";
					xtc_db_query($qry);
				}
				
				// TODO: Create notify emails when "STOCK_REORDER_LEVEL" is passed for the attribute
				// -> The following code shows how the gambio shoppingsystem handles email notifications for product options
				// --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
						//$gm_get_attributes_stock = xtc_db_query("SELECT
						//																						pd.products_name,
						//																						pa.attributes_stock,
						//																						po.products_options_name,
						//																						pov.products_options_values_name
						//																					FROM
						//																						products_description pd,
						//																						products_attributes pa,
						//																						products_options po,
						//																						products_options_values pov
						//																					WHERE pa.products_id = '" . $order->products[$i]['id'] . "'
						//						                               AND pa.options_values_id = '" . $order->products[$i]['attributes'][$j]['value_id'] . "'
						//						                               AND pa.options_id = '" . $order->products[$i]['attributes'][$j]['option_id'] . "'
						//																					 AND po.products_options_id = '" . $order->products[$i]['attributes'][$j]['option_id'] . "'
						//																					 AND po.language_id = '" . $_SESSION['languages_id'] . "'
						//																					 AND pov.products_options_values_id = '" . $order->products[$i]['attributes'][$j]['value_id'] . "'
						//																					 AND pov.language_id = '" . $_SESSION['languages_id'] . "'
						//																					 AND pd.products_id = '" . $order->products[$i]['id'] . "'
						//																					 AND pd.language_id = '" . $_SESSION['languages_id'] . "'");
						//if(xtc_db_num_rows($gm_get_attributes_stock) == 1){
						//	$gm_attributes_stock_data = xtc_db_fetch_array($gm_get_attributes_stock);
						//
						//	if($gm_attributes_stock_data['attributes_stock'] <= STOCK_REORDER_LEVEL) {
						//		$gm_subject = GM_OUT_OF_STOCK_NOTIFY_TEXT .' '. $gm_attributes_stock_data['products_name'] . ' - ' . $gm_attributes_stock_data['products_options_name'] . ': ' . $gm_attributes_stock_data['products_options_values_name'];
						//		$gm_body 		= GM_OUT_OF_STOCK_NOTIFY_TEXT .': ' . (double)$gm_attributes_stock_data['attributes_stock'] .' ('. $gm_attributes_stock_data['products_name'] . ' - ' . $gm_attributes_stock_data['products_options_name'] . ': ' . $gm_attributes_stock_data['products_options_values_name'] . ")\n\n".
						//									HTTP_SERVER.DIR_WS_CATALOG . 'product_info.php?info=p' . xtc_get_prid($order->products[$i]['id']);
						//
						//		xtc_php_mail(STORE_OWNER_EMAIL_ADDRESS, STORE_NAME, STORE_OWNER_EMAIL_ADDRESS, STORE_NAME, '', STORE_OWNER_EMAIL_ADDRESS, STORE_NAME, '', '', $gm_subject, nl2br(htmlentities($gm_body)), $gm_body);
						//	}
						//}
				// --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
			}
			
			// Specials stock and active status
			if(!empty($internalOrderInfo['is_special_price']) && !$ignoreSpecials){
				// Always update specials quantity if it is a special
				$qry = "
					UPDATE `" . TABLE_SPECIALS . "` AS `s`
						SET `s`.`specials_quantity` = `s`.`specials_quantity` - {$item->getQuantity()}
					WHERE `s`.`products_id` = '{$productId}'
				;";
				xtc_db_query($qry);
				
				// Always deactivate specials that have turned to a value equal to or less than zero and deactivate all specials that are expired
				$qry = "
					UPDATE `" . TABLE_SPECIALS . "` AS `s`
						SET `s`.`status` = 0
					WHERE
						`s`.`status` != 0
						AND
							(`s`.`expires_date` < NOW() AND `s`.`expires_date` != '0000-00-00 00:00:00' AND `s`.`expires_date` IS NOT NULL
							OR
							`s`.specials_quantity <= 0 AND `s`.`products_id` = '{$productId}')
				;";
				xtc_db_query($qry);
			}
		}
	}

	protected function createReviewsCsv() {
		$sql = "
		SELECT
			r.reviews_id,
			r.products_id,
			r.customers_name,
			r.reviews_rating,
			r.date_added,
			rd.reviews_text
		FROM
		" . TABLE_REVIEWS . " as r
		INNER JOIN
		" . TABLE_REVIEWS_DESCRIPTION . " as rd ON r.reviews_id = rd.reviews_id
		WHERE rd.languages_id = '".$this->languageId."'
		ORDER BY r.products_id ASC";
	
		$limit 	= 10;
		$page  	= 1;
		$offset = ($page-1)*$limit;
		$pg 	= " LIMIT $offset,$limit";
	
		while($query = xtc_db_query($sql.$pg)) {
			$count = xtc_db_num_rows($query);
			if($count == 0) {
				break;
			}
	
			$reviews = array();
			while($entry = xtc_db_fetch_array($query)) {
				$review = $this->buildDefaultReviewRow();
	
	
				$review['item_number'] 		= $entry['products_id'] ;
				$review['update_review_id'] = $entry['reviews_id'];
				$review['score'] 			= $entry['reviews_rating']*2;
				$review['name'] 			= $entry['customers_name'];
				$review['date'] 			= $entry['date_added'];
				$review['title'] 			= '';
				$review['text'] 			= $entry['reviews_text'];
	
				$reviews[] = $review;
			}
	
			foreach($reviews as $review) {
				$this->addReviewRow($review);
			}
	
			$page++;
			$offset = ($page-1)*$limit;
			$pg		= " LIMIT $offset,$limit";
		}
	}

	protected function createPagesCsv() {
	}

	private function _getAddressFormatId($isoCode2 = 'DE', $countryId = null) {
		$isoCode2 = strtoupper($isoCode2);
		if(!empty($countryId)) {
			$qry = "
				SELECT c.address_format_id
				FROM ".TABLE_COUNTRIES." c
				WHERE c.countries_id = '$countryId'
			";
		} else {
			$qry = "
				SELECT c.address_format_id
				FROM ".TABLE_COUNTRIES." c
				WHERE UPPER(c.countries_iso_code_2) = '$isoCode2'
			";
		}

		$result = xtc_db_query($qry);
		$item = xtc_db_fetch_array($result);
		return $item["address_format_id"];
	}

	private function _createGuestUser(ShopgateOrder $order) {
		//		$order = new ShopgateOrder();
		$address = $order->getInvoiceAddress();

		$customerStatus = $this->config->getCustomersStatusId();
		if($customerStatus === -1) $customerStatus = DEFAULT_CUSTOMERS_STATUS_ID;

		$customer = array();
		$customer["customers_vat_id_status"] = 0;
		$customer["customers_status"] = $customerStatus;
		$customer["customers_gender"] =  $address->getGender();
		$customer["customers_firstname"] = $address->getFirstName();
		$customer["customers_lastname"] = $address->getLastName();
		$customer["customers_email_address"] = $order->getMail();
		$customer["customers_default_address_id"] = "";
		$customer["customers_telephone"] = $order->getPhone();
		$customer["customers_fax"] = "";
		$customer["customers_newsletter"] = 0;
		$customer["customers_newsletter_mode"] = 0;
		$customer["member_flag"] = 0;
		$customer["delete_user"] = 1;
		$customer["account_type"] = 0;
		$customer["refferers_id"] = 0;
		$customer["customers_date_added"] = date( 'Y-m-d H:i:s' );
		$customer["customers_last_modified"] = date( 'Y-m-d H:i:s' );

		xtc_db_perform(TABLE_CUSTOMERS, $customer);
		$customerId = xtc_db_insert_id();

		$qry = "SELECT countries_id FROM ".TABLE_COUNTRIES." WHERE UPPER(countries_iso_code_2) = UPPER('".$address->getCountry() ."')";
		$qry = xtc_db_query($qry);
		$country = xtc_db_fetch_array($qry);
		if(empty($country)) {
			$country = array(
				'countries_id' => 81,
			);
		}
		
		$qry = "SELECT zone_id, zone_name FROM ".TABLE_ZONES." WHERE zone_country_id = {$country['countries_id']} AND zone_code = '" . ShopgateXtcMapper::getXtcStateCode($address->getState())."'";
		$qry = xtc_db_query($qry);
		$zone = xtc_db_fetch_array($qry);
		if(empty($zone)) {
			$zone = array(
				'zone_id' => null,
				'zone_name' => $address->getState(),
			);
		}
		
		$_address = array(
			"customers_id" => $customerId,
			"entry_gender" => $address->getGender(),
			"entry_company" => $address->getCompany(),
			"entry_firstname" => $address->getFirstName(),
			"entry_lastname" => $address->getLastName(),
			"entry_street_address" => $address->getStreet1() . (strlen($address->getStreet2()) > 0 ? (' ' . $address->getStreet2()) : ''),
			"entry_suburb" => "",
			"entry_postcode" => $address->getZipcode(),
			"entry_city" => $address->getCity(),
			"entry_state" => $zone['zone_name'],
			"entry_country_id" => $country["countries_id"],
			"entry_zone_id" => $zone['zone_id'],
			"address_date_added" => date( 'Y-m-d H:i:s' ),
			"address_last_modified" => date( 'Y-m-d H:i:s' ),
		);
		xtc_db_perform(TABLE_ADDRESS_BOOK, $_address);
		$addressId = xtc_db_insert_id();

		$customer = array(
			"customers_default_address_id" =>$addressId
		);
		xtc_db_perform(TABLE_CUSTOMERS, $customer, "update", "customers_id = $customerId");

		$_info = array(
			"customers_info_id" => $customerId,
			"customers_info_date_of_last_logon" => date( 'Y-m-d H:i:s' ),
			"customers_info_number_of_logons" => '1',
			"customers_info_date_account_created" => date( 'Y-m-d H:i:s' ),
			"customers_info_date_account_last_modified" => date( 'Y-m-d H:i:s' ),
			"global_product_notifications" => 0
		);
		xtc_db_perform(TABLE_CUSTOMERS_INFO, $_info);

		$customerMemo = array();
		$customerMemo["customers_id"] = $customerId;
		$customerMemo["memo_date"] = date( 'Y-m-d' );
		$customerMemo["memo_title"] = "Shopgate - Account angelegt";
		$customerMemo["memo_text"] = "Account wurde von Shopgate angelegt";
		$customerMemo["poster_id"] = null;
		xtc_db_perform("customers_memo", $customerMemo);
		
		$result = xtc_db_query("SELECT * FROM ".TABLE_CUSTOMERS." WHERE customers_id = " . $customerId);
		$customer = xtc_db_fetch_array($result);
		return $customer;
	}
	
	private function _pushOrderToAfterbuy($iOrderId, ShopgateOrder $order) {
		if (!$order->getIsShippingBlocked() && defined('AFTERBUY_ACTIVATED') && AFTERBUY_ACTIVATED == 'true') {
			$this->log("START TO SEND ORDER TO AFTERBUY", ShopgateLogger::LOGTYPE_ACCESS);
	
			require_once (DIR_WS_CLASSES.'afterbuy.php');
			$aBUY = new xtc_afterbuy_functions( $iOrderId );
			if ($aBUY->order_send()) {
				$aBUY->process_order();
				$this->log("SUCCESSFUL ORDER SEND TO AFTERBUY", ShopgateLogger::LOGTYPE_ACCESS);
			} else {
				$this->log("ORDER ALREADY SEND TO AFTERBUY", ShopgateLogger::LOGTYPE_ACCESS);
			}

			$this->log("FINISH SEND ORDER TO AFTERBUY", ShopgateLogger::LOGTYPE_ACCESS);
		}
	}
	
	private function _pushOrderToDreamRobot($dbOrderId, ShopgateOrder $shopgateOrder) {
		if (!$shopgateOrder->getIsShippingBlocked() && file_exists(DIR_FS_CATALOG.'dreamrobot_checkout.inc.php')) {
			require_once(DIR_FS_CATALOG.'includes/classes/order.php');
			$this->log("START TO SEND ORDER TO DREAMROBOT", ShopgateLogger::LOGTYPE_ACCESS);
			
			if(class_exists('order')){
				$order = new order($dbOrderId);
			} elseif(class_exists('order_ORIGIN')) {
				$order = new order_ORIGIN($dbOrderId);
			} else {
				throw new ShopgateLibraryException(ShopgateLibraryException::UNKNOWN_ERROR_CODE, '_pushOrderToDreamRobot(): class order not found', true);
			}
			$order->info['shipping_cost'] = $shopgateOrder->getAmountShipping();
			$_SESSION['tmp_oID'] = $dbOrderId;
			include_once ('./dreamrobot_checkout.inc.php');
			
			$this->log("FINISH SEND ORDER TO DREAMROBOT", ShopgateLogger::LOGTYPE_ACCESS);
		}
	}
	
	public function cron($jobname, $params, &$message, &$errorcount) {
		switch ($jobname) {
			case 'set_shipping_completed': $this->cronSetOrdersShippingCompleted($message, $errorcount); break;
			default: throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_CRON_UNSUPPORTED_JOB, 'Job name: "'.$jobname.'"', true);
		}
	}
	
	/**
	 * Marks shipped orders as "shipped" at Shopgate.
	 *
	 * This will find all orders that are marked "shipped" in the shop system but not at Shopgate yet and marks them "shipped" at Shopgate via
	 * Shopgate Merchant API.
	 *
	 * @param string $message Process log will be appended to this reference.
	 * @param int $errorcount This reference gets incremented on errors.
	 */
	protected function cronSetOrdersShippingCompleted(&$message, &$errorcount) {
		$query =
			"SELECT `sgo`.`orders_id`, `sgo`.`shopgate_order_number` ".
			"FROM `".TABLE_SHOPGATE_ORDERS."` sgo ".
			"INNER JOIN `".TABLE_ORDERS."` xto ON (`xto`.`orders_id` = `sgo`.`orders_id`) ".
			"INNER JOIN `".TABLE_LANGUAGES."` xtl ON (`xtl`.`directory` = `xto`.`language`) ".
			"WHERE `sgo`.`is_sent_to_shopgate` = 0 ".
				"AND `xto`.`orders_status` = ".xtc_db_input($this->config->getOrderStatusShipped())." ".
				"AND `xtl`.`code` = '".xtc_db_input($this->config->getLanguage())."';";
		$result = xtc_db_query($query);
		
		if (empty($result)) {
			return;
		}
		
		while ($shopgateOrder = xtc_db_fetch_array($result)) {
			if (!$this->setOrderShippingCompleted($shopgateOrder['shopgate_order_number'], $shopgateOrder['orders_id'], $this->merchantApi)) {
				$errorcount++;
				$message .= 'Shopgate order number "'.$shopgateOrder['shopgate_order_number'].'": error'."\n";
			}
		}
	}
	
	/**
	 * Set the shipping status for a list of order IDs.
	 *
	 * @param int[] $orderIds The IDs of the orders in the shop system.
	 * @param int $status The ID of the order status that has been set in the shopping system.
	 */
	public function updateOrdersStatus($orderIds, $status) {
		$query = xtc_db_input(
			"SELECT `sgo`.`orders_id`, `sgo`.`shopgate_order_number`, `xtl`.`code` ".
			"FROM `".TABLE_SHOPGATE_ORDERS."` sgo ".
			"INNER JOIN `".TABLE_ORDERS."` xto ON (`xto`.`orders_id` = `sgo`.`orders_id`) ".
			"INNER JOIN `".TABLE_LANGUAGES."` xtl ON (`xtl`.`directory` = `xto`.`language`) ".
			"WHERE `sgo`.`orders_id` IN (".xtc_db_input(implode(", ", $orderIds)).")");
		$result = xtc_db_query($query);
		
		if (empty($result)) {
			return;
		}
		
		$configurations = array();
		$merchantApis = array();
		while ($shopgateOrder = xtc_db_fetch_array($result)) {
			$language = $shopgateOrder['code'];
			
			if (empty($merchantApis[$language])) {
				try {
					$config = new ShopgateConfigGambioGx();
					$config->loadByLanguage($language);
					$builder = new ShopgateBuilder($config);
					$merchantApis[$language] = &$builder->buildMerchantApi();
					$configurations[$language] = $config;
				} catch (ShopgateLibraryException $e) {
					// do not abort. the error will be logged
				}
			}
			
			if ($status != $configurations[$language]->getOrderStatusShipped()) {
				return;
			}
			
			$this->setOrderShippingCompleted($shopgateOrder['shopgate_order_number'], $shopgateOrder['orders_id'], $merchantApis[$language]);
		}
	}
	
	/**
	 * Sets the order status of a Shopgate order to "shipped" via Shopgate Merchant API
	 *
	 * @param string $shopgateOrderNumber The number of the order at Shopgate.
	 * @param int $orderId The ID of the order in the shop system.
	 * @param ShopgateMerchantApi The SMA object to use for the request.
	 * @return bool true on success, false on failure.
	 */
	protected function setOrderShippingCompleted($shopgateOrderNumber, $orderId, ShopgateMerchantApi &$merchantApi) {
		$success = false;
		
		// These are expected and should not be added to error count:
		$ignoreCodes = array(ShopgateMerchantApiException::ORDER_ALREADY_COMPLETED, ShopgateMerchantApiException::ORDER_SHIPPING_STATUS_ALREADY_COMPLETED);
		
		try {
			$merchantApi->setOrderShippingCompleted($shopgateOrderNumber);
			
			$statusArr = array(
					"orders_id" => $orderId,
					"orders_status_id" => $this->config->getOrderStatusShipped(),
					"date_added" => date( 'Y-m-d H:i:s' ),
					"customer_notified" => 1,
					"comments" => "[Shopgate] Bestellung wurde bei Shopgate als versendet markiert",
			);
			
			xtc_db_perform(TABLE_ORDERS_STATUS_HISTORY, $statusArr);
			
			$success = true;
		} catch (ShopgateLibraryException $e) {
			$response = $this->stringFromUtf8($e->getAdditionalInformation(), $this->config->getEncoding());
		
			$statusArr = array(
					"orders_id" => $orderId,
					"orders_status_id" => $this->config->getOrderStatusShipped(),
					"date_added" => date( 'Y-m-d H:i:s' ),
					"customer_notified" => 0,
					"comments" => "[Shopgate] Ein Fehler ist im Shopgate Modul aufgetreten ({$e->getCode()}): {$response}",
			);
		
			xtc_db_perform(TABLE_ORDERS_STATUS_HISTORY, $statusArr);
		} catch (ShopgateMerchantApiException $e) {
			$response = $this->stringFromUtf8($e->getMessage(), $this->config->getEncoding());
		
			$statusArr = array(
					"orders_id" => $orderId,
					"orders_status_id" => $this->config->getOrderStatusShipped(),
					"date_added" => date( 'Y-m-d H:i:s' ),
					"customer_notified" => 0,
					"comments" => "[Shopgate] Ein Fehler ist bei Shopgate aufgetreten ({$e->getCode()}): {$response}",
			);
		
			xtc_db_perform(TABLE_ORDERS_STATUS_HISTORY, $statusArr);
			
			$success = (in_array($e->getCode(), $ignoreCodes)) ? true : false;
		} catch (Exception $e) {
			$response = $this->stringFromUtf8($e->getMessage(), $this->config->getEncoding());
		
			$statusArr = array(
					"orders_id" => $orderId,
					"orders_status_id" => $this->config->getOrderStatusShipped(),
					"date_added" => date( 'Y-m-d H:i:s' ),
					"customer_notified" => 0,
					"comments" => "[Shopgate] Ein unbekannter Fehler ist aufgetreten ({$e->getCode()}): {$response}",
			);
		
			xtc_db_perform(TABLE_ORDERS_STATUS_HISTORY, $statusArr);
		}
		
		// Update shopgate order on success
		if($success) {
			$qry = 'UPDATE `'.TABLE_SHOPGATE_ORDERS.'` SET `is_sent_to_shopgate` = 1 WHERE `shopgate_order_number` = '.$shopgateOrderNumber.';';
			xtc_db_query($qry);
		}
		
		return $success;
	}
	
	private function xtc_get_products_stock($products_id) {
		$products_id = xtc_get_prid($products_id);
		$stock_query = xtc_db_query("select products_quantity from " . TABLE_PRODUCTS . " where products_id = '" . $products_id . "'");
		$stock_values = xtc_db_fetch_array($stock_query);
	
		return $stock_values['products_quantity'];
	}
	
	/**
	 * Checks if the given table exists
	 * @param string $tableName
	 * @throws ShopgateLibraryException
	 * @return boolean
	 */
	protected function tableExists($tableName) {
		$tableName = trim($tableName);
		if(empty($tableName)) {
			return false;
		}
	
		// Get all table names
		$query	= xtc_db_query("SHOW TABLES");
		if(!$query) {
			// DB-Error
			throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_DATABASE_ERROR, "Shopgate Plugin - Error checking for table \"$tableName\".", true);
		}
		while($array = xtc_db_fetch_array($query)) {
			$array = array_values($array);
	
			// Check for table name
			if($array[0] == $tableName) {
				return true;
			}
		}
	
		// The requested table has not been found if execution reaches here
		return false;
	}
}

class ShopgateXtcMapper {

	/**
	 * The countries with non-ISO-3166-2 state codes in xt:Commerce 3 are mapped here.
	 * @var string[][]
	 */
	protected static $stateCodesByCountryCode = array(
	'DE' => array(
	"BW" => "BAW",
	"BY" => "BAY",
	"BE" => "BER",
	"BB" => "BRG",
	"HB" => "BRE",
	"HH" => "HAM",
	"HE" => "HES",
	"MV" => "MEC",
	"NI" => "NDS",
	"NW" => "NRW",
	"RP" => "RHE",
	"SL" => "SAR",
	"SN" => "SAS",
	"ST" => "SAC",
	"SH" => "SCN",
	"TH" => "THE",
	),
	"AT" => array(
	"1" => "BL",
	"2" => "KN",
	"3" => "NO",
	"4" => "OO",
	"5" => "SB",
	"6" => "ST",
	"7" => "TI",
	"8" => "VB",
	"9" => "WI",
	),
	//"CH" => ist in xt:commerce bereits korrekt
	//"US" => ist in xt:commerce bereits korrekt
	);

	/**
	 * Finds the corresponding Shopgate state code for a given xt:Commerce 3 state code (zone_code).
	 *
	 * @param string $countryCode The code of the country to which the state belongs
	 * @param string $xtcStateCode The code of the state / zone as found in the default "zones" table of xt:Commerce 3
	 * @return string The state code as defined at Shopgate Wiki
	 *
	 * @throws ShopgateLibraryException if one of the given codes is unknown
	 */
	public static function getShopgateStateCode($countryCode, $xtcStateCode) {
		$countryCode = strtoupper($countryCode);
		$xtcStateCode = strtoupper($xtcStateCode);

		if (!isset(self::$stateCodesByCountryCode[$countryCode])) {
			return $countryCode.'-'.$xtcStateCode;
		}

		$codes = array_flip(self::$stateCodesByCountryCode[$countryCode]);
		if (!isset($codes[$xtcStateCode])) {
			return $countryCode.'-'.$xtcStateCode;
		}

		$stateCode = $codes[$xtcStateCode];
		return $countryCode.'-'.$stateCode;
	}

	/**
	 * Finds the corresponding xt:Commerce 3 state code (zone_code) for a given Shopgate state code
	 *
	 * @param string $shopgateStateCode The Shopgate state code as defined at Shopgate Wiki
	 * @return string The zone code for xt:Commerce 3
	 *
	 * @throws ShopgateLibraryException if the given code is unknown
	 */
	public static function getXtcStateCode($shopgateStateCode) {
		$splitCodes = null;
		preg_match('/^([A-Z]{2})\-([A-Z]{2})$/', $shopgateStateCode, $splitCodes);

		if (empty($splitCodes) || empty($splitCodes[1]) || empty($splitCodes[2])){
			return null;
		}

		if(!isset(self::$stateCodesByCountryCode[$splitCodes[1]]) || !isset(self::$stateCodesByCountryCode[$splitCodes[1]][$splitCodes[2]])) {
			//throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_UNKNOWN_STATE_CODE, 'Code: '.$shopgateStateCode);
			return $splitCodes[2];
		} else {
			return self::$stateCodesByCountryCode[$splitCodes[1]][$splitCodes[2]];
		}
	}
}