<?php
/* --------------------------------------------------------------
   Captcha.inc.php 2013-05-10 gm
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2013 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/
class Captcha {
	private $v_captcha_type = '';
	private $v_public_key = '';
	private $v_private_key = '';
	private $v_vv_code = '';
	private $v_vv_code_name = '';
	private $v_captcha_theme = '';
	private $v_error = '';
	
	public function __construct($p_vv_code_name = 'vvcode', $p_captcha_theme = 'white', $p_vv_code_length = 6)
	{
		$this->v_public_key = gm_get_conf('GM_RECAPTCHA_PUBLIC_KEY');
		$this->v_private_key = gm_get_conf('GM_RECAPTCHA_PRIVATE_KEY');
		$this->v_vv_code_name = $p_vv_code_name;
		$this->v_captcha_theme = $p_captcha_theme;
		
		//reCAPTCHA
		if ($this->recaptcha_active())
		{
			$this->v_captcha_type = 'recaptcha';
		}
		//vvCode
		else
		{
			$this->v_captcha_type = 'vv_code';
			$this->reload_vv_code($p_vv_code_length);
		}
	}
	
	public function reload_vv_code($p_vv_code_length = 6)
	{
		require_once (DIR_FS_INC . 'xtc_random_charcode.inc.php');
		$this->v_vv_code = $_SESSION['vvcode'];
		$t_vv_code = xtc_random_charcode($p_vv_code_length);
		$_SESSION['vvcode'] = $t_vv_code;
	}
	
	public function recaptcha_active()
	{
		return gm_get_conf('GM_CAPTCHA_TYPE') == 'recaptcha';
	}
	
	public function is_valid($p_request_data, $p_section = '', $p_is_ajax_request = false)
	{
		if (empty($p_section))
		{
			$t_section_is_secured = true;
		}
		else
		{
			$t_section_is_secured = gm_get_conf($p_section);
		}
		
		if ($t_section_is_secured === 'false')
		{
			return true;
		}
		
		switch ($this->v_captcha_type)
		{
			case 'recaptcha':
				if (empty($p_request_data['recaptcha_response_field']) || empty($p_request_data['recaptcha_challenge_field']))
				{
					return false;
				}
				
				require_once(DIR_FS_CATALOG . 'includes/recaptchalib.php');
				$t_response = recaptcha_check_answer ($this->v_private_key, $_SERVER["REMOTE_ADDR"], $p_request_data['recaptcha_challenge_field'], $p_request_data['recaptcha_response_field']);
				if (!$t_response->is_valid)
				{
					$this->v_error = $t_response->error;
					return false;
				}
				return true;
			case 'vv_code':
				if (empty($p_request_data[$this->v_vv_code_name]))
				{
					return false;
				}
				$t_vv_code = $this->v_vv_code;
				$this->v_vv_code = $_SESSION['vvcode'];
				if ($p_is_ajax_request)
				{
					$t_vv_code = $this->v_vv_code;
				}
				return strtoupper($p_request_data[$this->v_vv_code_name]) == $t_vv_code;
			default:
				return false;
		}
	}
	
	public function get_html()
	{
		switch ($this->v_captcha_type)
		{
			case 'recaptcha':
				require_once(DIR_FS_CATALOG . 'includes/recaptchalib.php');
				$t_html = recaptcha_get_html($this->v_public_key, $this->v_error);
				$this->v_error = '';
				$t_theme_html = '<script type="text/javascript" src="http://www.google.com/recaptcha/api/js/recaptcha_ajax.js"></script>
								<script type="text/javascript">
									var RecaptchaOptions = new Array();
									RecaptchaOptions["theme"] = "' . $this->v_captcha_theme . '";
									
									function showRecaptcha(element)
									{
										Recaptcha.create("' . $this->v_public_key . '", element, RecaptchaOptions);
									}
								</script>';
				return $t_theme_html . $t_html;
			case 'vv_code':
				$t_html = '	<img id="' . $this->v_vv_code_name . '_image" src="' . xtc_href_link('gm_create_vvcodes.php', 'rand=' . rand(), 'SSL', true) . '" alt="" title="" width="240" height="50" /><br />
							<input type="text" id="' . $this->v_vv_code_name . '" name="' . $this->v_vv_code_name . '" class="input-text" /> *';
				return $t_html;
			default:
				return '';
		}
	}
}
